/*
 * atmelUtil - access ATMEL AT88SC0808C using MCT Trigger 1+ USB2VGA Chip
 *
 * Copyright (C) 2009 by Magic Control Technology Inc. 
 *
 * If distributed as part of the Linux kernel, this code is licensed under the
 * terms of the GPL v2.
 *
 * Otherwise, the following license terms apply:
 *
 * * Redistribution and use in source and binary forms, with or without
 * * modification, are permitted provided that the following conditions
 * * are met:
 * * 1) Redistributions of source code must retain the above copyright
 * *    notice, this list of conditions and the following disclaimer.
 * * 2) Redistributions in binary form must reproduce the above copyright
 * *    notice, this list of conditions and the following disclaimer in the
 * *    documentation and/or other materials provided with the distribution.
 * * 3) The name of the author may not be used to endorse or promote products
 * *    derived from this software without specific psisusbr written permission.
 * *
 * * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESSED OR
 * * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	Authors:
 *		Louis Tsai	    <louis@mct.com.tw>
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <time.h>
#include <linux/ioctl.h>

#ifndef __KERNEL__ /* Urgh. The whole point of splitting this out into
                    separate files was to avoid #ifdef __KERNEL__ */
#define __user
#endif

typedef struct _READ_ATMEL {
	unsigned char Command;
	unsigned char Addr1;
	unsigned char Addr2;
	unsigned char N;
	unsigned char __user *read_buf;
}READ_ATMEL,*PREAD_ATMEL;



typedef struct _WRITE_ATMEL {
	int size;
	unsigned char __user *write_buf;
} WRITE_ATMEL,*PWRITE_ATMEL;

#define TUSB_IOC_MAGIC	'L'
#define TUSB_IOCTL_READ_ATMEL	_IOWR(TUSB_IOC_MAGIC, 0x02, PREAD_ATMEL)
#define TUSB_IOCTL_WRITE_ATMEL	_IOW(TUSB_IOC_MAGIC, 0x03, PWRITE_ATMEL)
#define TUSB_IOCTL_ACK_POLLING	_IO(TUSB_IOC_MAGIC,  0x04)
#define TUSB_IOCTL_LIGHT_LED	_IOW(TUSB_IOC_MAGIC, 0x05, int)


static void dump_buf(unsigned char *buf, int len)
{
	int i=0;
	printf("******************************%s******************************\n", __FUNCTION__);
	for(i=0;i<len;i++){
		if(i && !(i%16)) printf("\n");
		printf("0x%2x ", buf[i]);
	}
	printf("\n********************************************************************\n", __FUNCTION__);
}

void Ateml_User_Zone_N_ReadWrite_Test(int fd, int zone_no);
int	 Ateml_Write_Data(int fd, unsigned char *buf, int len);

int main(int argc, char **argv)
{

	int fd = 0, light = 1, i=0;
	WRITE_ATMEL	watmel;
	READ_ATMEL	ratmel;
	
	fd = open("/dev/triggerusbvga0" , O_RDWR);

	if(fd < 0) {
        printf("Error: cannot open trigger device.\n");
        exit(1);
    }
	
	//blink LED 
	light = 0; //off
	ioctl(fd, TUSB_IOCTL_LIGHT_LED, &light);
	sleep(1);
	light = 1;// on
	ioctl(fd, TUSB_IOCTL_LIGHT_LED, &light);

	//check ATMEL AT88SC0808C exist
	if(!ioctl(fd, TUSB_IOCTL_ACK_POLLING, NULL))
		printf("ATMEL AT88SC0808C is on i2c bus\n");
	else
		printf("there is no ATMEL AT88SC0808C\n");

	//Use the AT88SC0104C Initialization Example in the spec(P.46)
	//to do the function demo
	
	//read write test from zone 0~3
	for(i=0;i<4;i++)
		Ateml_User_Zone_N_ReadWrite_Test(fd, i);

/*
	//*UNLOCK CONFIGURATION MEMORY
	//BA 07 00 03 DD 42 97
	{
		char writedata[]={0xBA, 0x07, 0x00, 0x03 ,0xDD ,0x42 ,0x97};
		if(Ateml_Write_Data(fd, writedata, sizeof(writedata)))
			printf("\"UNLOCK CONFIGURATION MEMORY\" is ok.\n");
		else
			printf("\"UNLOCK CONFIGURATION MEMORY\" has failed.\n");
	}
	//*WRITE CODES IN CONFIGURATION MEMORY
	//*Write Card Mfg Code = P001
	//B4 00 0B 04 50 30 30 31
	{
		char writedata[]={0xB4, 0x00, 0x0B, 0x04, 0x50, 0x30, 0x30, 0x31};
		if(Ateml_Write_Data(fd, writedata, sizeof(writedata)))
			printf("\"WRITE CODES IN CONFIGURATION MEMORY\" is ok.\n");
		else
			printf("\"WRITE CODES IN CONFIGURATION MEMORY\" has failed.\n");
	}
	//*Write Identification Number = 00000000012345
	//B4 00 19 07 00 00 00 00 01 23 45
	{
		char writedata[]={0xB4, 0x00, 0x19, 0x07, 0x00, 0x00, 0x00, 0x00, 0x01, 0x23, 0x45};
		if(Ateml_Write_Data(fd, writedata, sizeof(writedata)))
			printf("\"Write Identification Number\" is ok.\n");
		else
			printf("\"Write Identification Number\" has failed.\n");
	}
	//*Write Issuer Code = STATION 035
	//B4 00 40 10 53 54 41 54 49 4F 4E 20 30 33 35 00 00 00 00 00
	{
		char writedata[]={0xB4, 0x00, 0x40, 0x10, 0x53, 0x54, 0x41, 0x54, 0x49, 0x4F, 
                          0x4E, 0x20, 0x30, 0x33, 0x35, 0x00, 0x00, 0x00, 0x00, 0x00};
		
		if(Ateml_Write_Data(fd, writedata, sizeof(writedata)))
			printf("\"Write Issuer Code\" is ok.\n");
		else
			printf("\"Write Issuer Code\" has failed.\n");
	}
	//*WRITE REGISTERS IN CONFIGURATION MEMORY
	//*Write Registers AR1/PR1 = 7F F9, AR2/PR2 = DF BF, AR3/PR3 = 57 B9
	//B4 00 22 06 7F F9 DF BF 57 B9
	{
		char writedata[]={0xB4, 0x00, 0x22, 0x06, 0x7F, 0xF9, 0xDF, 0xBF, 0x57, 0xB9};
		
		if(Ateml_Write_Data(fd, writedata, sizeof(writedata)))
			printf("\"WRITE REGISTERS\" is ok.\n");
		else
			printf("\"WRITE REGISTERS\" has failed.\n");
	}
	//*WRITE KEYS IN CONFIGURATION MEMORY
	//*Write Ci for set 2 = 22222222222222
	//B4 00 71 07 22 22 22 22 22 22 22
	{
		char writedata[]={0xB4, 0x00, 0x71, 0x07, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22};
		
		if(Ateml_Write_Data(fd, writedata, sizeof(writedata)))
			printf("\"WRITE KEYS\" is ok.\n");
		else
			printf("\"WRITE KEYS\" has failed.\n");
	}
	//*Write Gc for set 2 = 5B4F9AE4B5098BE7
	//B4 00 A0 08 5B 4F 9A E4 B5 09 8B E7
	{
		char writedata[]={0xB4, 0x00, 0xA0, 0x08, 0x5B, 0x4F, 0x9A, 0xE4, 0xB5, 0x09, 0x8B, 0xE7};
		
		if(Ateml_Write_Data(fd, writedata, sizeof(writedata)))
			printf("\"Write Gc for set 2\" is ok.\n");
		else
			printf("\"Write Gc for set 2\" has failed.\n");
	}

	//*WRITE PASSWORDS IN CONFIGURATION MEMORY
	//*Write Passwords, read 7 = 10 00 01, write 7 = 11 00 11
	//B4 00 B9 07 11 00 11 FF 10 00 01
	{
		char writedata[]={0xB4, 0x00, 0xB9, 0x07, 0x11, 0x00, 0x11, 0xFF, 0x10, 0x00, 0x01};
		
		if(Ateml_Write_Data(fd, writedata, sizeof(writedata)))
			printf("\"WRITE PASSWORDS\" is ok.\n");
		else
			printf("\"WRITE PASSWORDS\" has failed.\n");
	}
	//*READ ENTIRE CONFIGURATION MEMORY TO VERIFY
	//B6 00 00 F0
	ratmel.Command = 0xB6;
	ratmel.Addr1   = 0x00;
	ratmel.Addr2   = 0x00;	
	ratmel.N       = 0xF0;
	ratmel.read_buf = (unsigned char*) malloc(ratmel.N);
	if(!ioctl(fd, TUSB_IOCTL_READ_ATMEL, &ratmel)) {
		printf("\"READ ENTIRE CONFIGURATION MEMORY \" is ok.\n");
		dump_buf(ratmel.read_buf, ratmel.N);
	}
	else
		printf("\"READ ENTIRE CONFIGURATION MEMORY\" has failed.\n");
	
	free(ratmel.read_buf);
*/

	close(fd);

}

//return 0 is successful, otherwise failure.
int Ateml_Write_Data(int fd, unsigned char *buf, int len)
{
	WRITE_ATMEL	watmel;
	int bRet = 0;
	
	watmel.size = len;
	watmel.write_buf = (unsigned char*) malloc((size_t)watmel.size);
	memcpy((void*)watmel.write_buf, (void*)buf, watmel.size);

	if(!ioctl(fd, TUSB_IOCTL_WRITE_ATMEL, &watmel)) 
		bRet = 1;
	
	free(watmel.write_buf);

	return bRet;
}


void Ateml_User_Zone_N_ReadWrite_Test(int fd, int zone_no)
{
	WRITE_ATMEL	watmel;
	READ_ATMEL	ratmel;

	char writedata[]={0xB0, 0x00, 0x00, 0x0B, 0x5A, 0x6F, 0x6E, 0x65,
				      0x20, 0x30, 0x20, 0x44, 0x61 ,0x74 ,0x61};

	//*WRITE DATA TO USER ZONES
	//*Set User Zone to zone_no
	//B4 03 00 00
	watmel.size = 4;
	watmel.write_buf = (unsigned char*) malloc(4);
	watmel.write_buf[0] = 0xB4; watmel.write_buf[1] = 0x03;
	watmel.write_buf[2] = zone_no; watmel.write_buf[3] = 0x00;
	
	if(!ioctl(fd, TUSB_IOCTL_WRITE_ATMEL, &watmel))
		printf("\"Set User Zone %d\" is ok.\n", zone_no);
	else
		printf("\"Set User Zone %d\" has failed.\n", zone_no);

	free(watmel.write_buf);
	//*Write data = Zone zone_no Data
	//B0 00 00 0B 5A 6F 6E 65 20 30 20 44 61 74 61

	watmel.size = sizeof(writedata);
	watmel.write_buf = (unsigned char*) malloc(watmel.size);
	memcpy((void*)watmel.write_buf, (void*)writedata, watmel.size);

	if(!ioctl(fd, TUSB_IOCTL_WRITE_ATMEL, &watmel))
		printf("\"Write data = Zone %d Data\" is ok.\n", zone_no);
	else
		printf("\"Write data = Zone %d Data\" has failed.\n", zone_no);

	free(watmel.write_buf);

	//read zone zone_no data to comfirm writing
	ratmel.Command = 0xB2;
	ratmel.Addr1   = 0x00;
	ratmel.Addr2   = 0x00;	
	ratmel.N = watmel.size-4;
	ratmel.read_buf = (unsigned char*) malloc(ratmel.N);

	if(!ioctl(fd, TUSB_IOCTL_READ_ATMEL, &ratmel)) {
		printf("\"Read data = Zone %d Data\" is ok.\n", zone_no);
		//dump_buf(ratmel.read_buf, ratmel.N);
		if(!memcmp(ratmel.read_buf, writedata+4, ratmel.N))
			printf("\"Read data = Zone %d Data\" is correct.\n", zone_no);
		else
			printf("\"Read data = Zone %d Data\" is incorrect.\n", zone_no);
	}
	else
		printf("\"Read data = Zone %d Data\" has failed.\n", zone_no);
	
	free(ratmel.read_buf);
}
