/*
 * i2c.c - the trigger 1+ i2c function & ATMEL AT88SC0808C r/w
 *
 * Copyright (C) 2009 by Magic Control Technology Inc. 
 *
 * If distributed as part of the Linux kernel, this code is licensed under the
 * terms of the GPL v2.
 *
 * Otherwise, the following license terms apply:
 *
 * * Redistribution and use in source and binary forms, with or without
 * * modification, are permitted provided that the following conditions
 * * are met:
 * * 1) Redistributions of source code must retain the above copyright
 * *    notice, this list of conditions and the following disclaimer.
 * * 2) Redistributions in binary form must reproduce the above copyright
 * *    notice, this list of conditions and the following disclaimer in the
 * *    documentation and/or other materials provided with the distribution.
 * * 3) The name of the author may not be used to endorse or promote products
 * *    derived from this software without specific psisusbr written permission.
 * *
 * * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESSED OR
 * * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	Authors:
 *		Louis Tsai	    <louis@mct.com.tw>
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/poll.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/spinlock.h>
#include <asm/delay.h>
#include <linux/kref.h>
#include <linux/usb.h>
#include <linux/smp_lock.h>
#include <linux/vmalloc.h>
#include <linux/mutex.h>
#include "Tusb.h"
#include "i2c.h"

extern int ReadRegisterByte(struct sisusb_usb_data *sisusb, u16 reg, u8 *data);
extern int WriteRegisterByte(struct sisusb_usb_data *sisusb,u32 reg,u32 data);

/*
	2 I2C register set
	0xFCE0 0xFCE1 0xFCE2 0xFCE3 0xFCE4
	0xFCE8 0xFCE9 0xFCEA 0xFCEB 0xFCEC
*/
//**************************************************************
void I2CInitial(struct sisusb_usb_data *sisusb)
{
//	WriteRegisterByte (pDevExt, 0xFCEA, 0x00);
	WriteRegisterByte (sisusb, 0xFCE0, 0x4A);	// E8 0x40: 80KHz, 0x4A = 30M/(5* 80K) -1
	WriteRegisterByte (sisusb, 0xFCE1, 0x00);	//	E9
	// core enable
	WriteRegisterByte (sisusb, 0xFCE2, 0x80);	//EA
}
//*************************************************************
BOOLEAN I2CStart_Fake (struct sisusb_usb_data *sisusb, UCHAR addr)
{
	unsigned char I2CStatus;
	int retry = 10;
	
	printk("I2CStart_Fake\n");
	WriteRegisterByte (sisusb, 0xFCE3, addr); //address	//EB
	WriteRegisterByte (sisusb, 0xFCE4, 0x90); //command	//EC
	// polling the ack status and TIP busy
	ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);		
	// wait TIP
	while ((I2CStatus&0x02) && (retry--)){
		ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);
	}
/*	
	if (!(I2CStatus&0x02))
	{
		retry = 10;
		// wait ready
		while((I2CStatus & 0x80)&& (retry--))
		{
			ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);
		}
		if (retry <=0)
		{
			//printk ("I2CStart_Fake: wait ACK timeout\n");
			return FALSE;
		}
	}
	else
	{
		printk ("I2CStart: wait bit1 timeout\n");
		return FALSE;
	}
*/	
	WriteRegisterByte (sisusb, 0xFCE4, 0x40); 	//stop	
	//printk ("I2CStart_FAKE: get ACK\n");
	return TRUE;
}
//*************************************************************
BOOLEAN I2CStart (struct sisusb_usb_data *sisusb, UCHAR addr)
{
	unsigned char I2CStatus;
	int retry = 10;
		
	WriteRegisterByte (sisusb, 0xFCE3, addr); //address	//EB
	WriteRegisterByte (sisusb, 0xFCE4, 0x90); //command	//EC
	// polling the ack status and TIP busy
	ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);		
	// wait TIP
	while ((I2CStatus&0x02) && (retry--)) {
		ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);
	}
	
	if (!(I2CStatus&0x02)) {
		retry = 100;
		// wait ack
		while((I2CStatus & 0x80)&& (retry--)) {
			ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);
		}
		if (retry <=0) {
			printk ("I2CStart: wait ACK timeout\n");
			return FALSE;
		}
	}
	else {
		printk ("I2CStart: wait bit1 timeout\n");
		return FALSE;
	}
	printk ("I2CStart: get ACK\n");
	return TRUE;
}

//*************************************************************
BOOLEAN I2CWrite(struct sisusb_usb_data *sisusb, unsigned char I2CCommand, unsigned char I2CDataIn)
{
	unsigned char I2CStatus;
	int retry = 10;
	
	WriteRegisterByte (sisusb, 0xFCE3, I2CDataIn);
	WriteRegisterByte (sisusb, 0xFCE4, I2CCommand);
	// polling the ack status and TIP busy
	ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);
	// wait TIP
	while ((I2CStatus&0x02) && (retry--)) {
		ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);
	}
	
	if (!(I2CStatus&0x02)) {
		retry = 100;
		// wait ready
		while((I2CStatus & 0x80)&& (retry--)) {
			ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);
		}
		if (retry <=0) {
			printk ("I2CWrite: wait ACK timeout. %x %x\n",I2CCommand, I2CDataIn );
			return FALSE;
		}
	}
	else {
		printk ("I2CWrite: wait bit1 timeout. %x %x\n\n",I2CCommand, I2CDataIn );
		return FALSE;
	}
	
	return TRUE;
}
//*************************************************************
BOOLEAN I2CRead(struct sisusb_usb_data *sisusb, unsigned char I2CCommand, unsigned char *I2CDataout)
{
	unsigned char I2CStatus;
	int retry = 10;
	
	WriteRegisterByte (sisusb, 0xFCE4, I2CCommand);

	// polling the ack status 
	ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);
	while((I2CStatus & 0x02)&& (retry--)) {
		ReadRegisterByte (sisusb, 0xFCE4, &I2CStatus);
	}
	if (retry <=0) {
		//printk ("I2CRead: wait Bit1 timeout\n");	
		return FALSE;
	}
	// Read data
	ReadRegisterByte (sisusb, 0xFCE3, I2CDataout);
	return TRUE;
}

//*************************************************************
BOOLEAN I2C_WriteByte (struct sisusb_usb_data *sisusb, UCHAR SlaveID, UCHAR addr, UCHAR data)
{
	printk ("I2C_WriteByte: ID:%x, addr:%x, data:%x\n", SlaveID, addr, data);
	if (I2CStart (sisusb, SlaveID)) {
		I2CWrite (sisusb, 0x10, addr);
		I2CWrite (sisusb, 0x50, data);	
		//WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
	}
	else {	//try again
		printk ("I2C_WriteByte: try again\n");
		I2CStart_Fake (sisusb, SlaveID);
		mdelay(6); //6ms: wait for write cycle
		if (I2CStart (sisusb, SlaveID)) {
			I2CWrite (sisusb, 0x10, addr);
			I2CWrite (sisusb, 0x50, data);	
			//WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
		}
	}
	
	mdelay(6);  //6ms: wait for write cycle
	return TRUE;	
}
//*************************************************************
BOOLEAN I2C_ReadByte (struct sisusb_usb_data *sisusb, UCHAR SlaveID, UCHAR addr, UCHAR *data)
{
	UCHAR buf = 0xff;
	BOOLEAN ret = TRUE;
	
	printk("I2C_ReadByte: SlaveID:%x Addr:%x\n", SlaveID, addr);
	if (I2CStart (sisusb, SlaveID)) {
		I2CWrite (sisusb, 0x10, addr);
		if (I2CStart (sisusb, SlaveID|0x01))	//read
			ret = I2CRead (sisusb, 0x68, &buf);	//read and stop
	}
	else {	//try again
		printk("I2C_ReadByte: try again\n");
		I2CStart_Fake (sisusb, SlaveID);
		mdelay (6); //6ms: wait for write cycle
		if (I2CStart (sisusb, SlaveID)) {
			I2CWrite (sisusb, 0x10, addr);
			if (I2CStart (sisusb, SlaveID|0x01))	//read
				ret =I2CRead (sisusb, 0x68, &buf);	//read and stop
		}
		else
			ret = FALSE;
	}
	*data = buf;
	return ret;
}

//*************************************************************
BOOLEAN I2C_WriteMultipleBytes (struct sisusb_usb_data *sisusb, UCHAR SlaveID, UCHAR addr, USHORT count, UCHAR *pdata)
{
	UCHAR *pBuf = pdata;
	USHORT i, j, d;
	
	printk ("I2C_WriteMultipleBytes: ID:%x, addr:%x, count:%x\n", SlaveID, addr, count);
	if (count == 0)
		return TRUE;
	d = addr % I2C_PAGE_SIZE;
	i = 0;
	if (d != 0) {
		for (i=0; i< d; i++) {
			printk ("I2C_WriteMultipleBytes: single bye add: %x data: %x\n", addr+i, *(pdata+i));
			I2C_WriteByte (sisusb, SlaveID, addr+i, *(pdata+i));
		}
	}
	while ((i+I2C_PAGE_SIZE) <= count) {
		printk ("I2C_WriteMultipleBytes: write a page, addr:%x data: ", addr+i);
		if (I2CStart (sisusb, SlaveID)) {
			I2CWrite (sisusb, 0x10, addr+i);
			for (j=0; j<I2C_PAGE_SIZE; j++) {
				I2CWrite (sisusb, 0x10, *(pdata+i));
				i++;	
			}
			WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
			mdelay (6); //6ms: wait for write cycle
		}
		else {	//try again
			printk ("I2C_WriteMultipleBytes: try again\n");
			I2CStart_Fake (sisusb, SlaveID);
			mdelay (6); //6ms: wait for write cycle
			if (I2CStart (sisusb, SlaveID)) {
				I2CWrite (sisusb, 0x10, addr+i);
				for (j=0; j<I2C_PAGE_SIZE; j++) {
					I2CWrite (sisusb, 0x10, *(pdata+i));	
					i++;
				}
				WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
				mdelay (6); //6ms: wait for write cycle
			}
			else {
				printk ("I2C_WriteMultipleBytes: Fail\n");
				WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
				mdelay (6); //6ms: wait for write cycle
				return FALSE;
			}
		}
		printk ("\n");		
	}
	//remain data
	if (i < count) {
		printk ("I2C_WriteMultipleBytes: write remain data addr:%x data: ", addr+i);
		if (I2CStart (sisusb, SlaveID)) {
			I2CWrite (sisusb, 0x10, addr+i);
			for (j=i; j<count; j++) {
				printk(" %x ", *(pdata+i));
				I2CWrite (sisusb, 0x10, *(pdata+i));
				i++;
			}
			WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
			mdelay (6); //6ms: wait for write cycle
		}
		else {	//try again
			printk ("I2C_WriteMultipleBytes: try again\n");
			I2CStart_Fake (sisusb, SlaveID);
			mdelay (6); //6ms: wait for write cycle
			if (I2CStart (sisusb, SlaveID)) {
				I2CWrite (sisusb, 0x10, addr+i);
				for (j=i; j<count; j++) {
					I2CWrite (sisusb, 0x10, *(pdata+i));	
					i++;
				}
				WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
				mdelay (6); //6ms: wait for write cycle
			}
			else {
				printk ("I2C_WriteMultipleBytes: Fail\n");
				WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
				mdelay (6); //6ms: wait for write cycle
				return FALSE;
			}
		}
		printk ("\n");
	}
	return TRUE;	
}
//*************************************************************
BOOLEAN I2C_ReadMultipleBytes (struct sisusb_usb_data *sisusb, UCHAR SlaveID, UCHAR addr, USHORT count, UCHAR *pdata)
{
	UCHAR *pbuf = pdata;
	USHORT i;
	
	printk ("I2C_ReadByte: SlaveID:%x Addr:%x count:%d data: ", SlaveID, addr, count);
	if (count <= 0)
		return FALSE;
	if (I2CStart (sisusb, SlaveID)) {
		I2CWrite (sisusb, 0x10, addr);
		if (I2CStart (sisusb, SlaveID|0x01)) {	//read
			for (i=0; i <count-1; i++) {
				I2CRead (sisusb, 0x20, pbuf+i);	//read and stop ACK
				//KdPrint ((" %x ", *(pbuf+i)));
			}
			//last byte
			I2CRead (sisusb, 0x68, pbuf+i);	//read and stop NACK
			//KdPrint ((" %x ", *(pbuf+i)));
		}
		else {
			printk ("I2C_ReadMultipleBytes: fail\n");
			return FALSE;
		}
	}
	else {	//try again
		printk ("I2C_ReadByte: try again\n");
		I2CStart_Fake (sisusb, SlaveID);
		mdelay (6); //6ms: wait for write cycle
		if (I2CStart (sisusb, SlaveID)){
			I2CWrite (sisusb, 0x10, addr);
			if (I2CStart (sisusb, SlaveID|0x01)) {	//read
				for (i=0; i <count-1; i++) {
					I2CRead (sisusb, 0x20, pbuf+i);	//read and stop ACK
					//printk (" %x ", *(pbuf+i));
				}
				//last byte
				I2CRead (sisusb, 0x68, pbuf+i);	//read and stop NACK
				//printk (" %x ", *(pbuf+i));
			}
			else
				return FALSE;
		}
		else {
			printk ("I2C_ReadMultipleBytes: try again fail\n");
			return FALSE;
		}		
	}	
	return TRUE;
}

//*********************************************************************************
//Read / Write Atmel EEPROM
BOOLEAN Read_Atmel_Func (struct sisusb_usb_data *sisusb, PREAD_ATMEL pReadAtmel, UCHAR *outBuf)
{
	BOOLEAN bRet = FALSE;
	ULONG i, count;
	
	printk ("Read_Atmel_Func : Cmd:%x Addr1:%x Addr2:%x N:%x\n", 
		pReadAtmel->Command, pReadAtmel->Addr1, pReadAtmel->Addr2, pReadAtmel->N);
	if (!I2CStart (sisusb, pReadAtmel->Command))
		goto Read_Atmel_End;
	if (!I2CWrite (sisusb, 0x10, pReadAtmel->Addr1))
		goto Read_Atmel_End;
	if (!I2CWrite (sisusb, 0x10, pReadAtmel->Addr2))
		goto Read_Atmel_End;
	if (!I2CWrite (sisusb, 0x10, pReadAtmel->N))
		goto Read_Atmel_End;
	if (pReadAtmel->N == 0)
		count = 256;
	else
		count = pReadAtmel->N;
		
	for (i=0; i<count-1; i++) {
		if (!I2CRead (sisusb, 0x20, outBuf+i)) {
			printk ("Read_Atmel_Func: Read data fail\n");
			goto Read_Atmel_End;
		}
		printk ("Read_Atmel_Func: [%d] = %x\n", i, outBuf[i]);
	}
	
	I2CRead (sisusb, 0x68, outBuf+i);
	bRet = TRUE;
Read_Atmel_End:	
	//WriteRegisterByte (pDevExt, 0xFCE4, 0x40);	//stop
	printk ("Read_Atmel_Func: result:%d\n", bRet);
	return bRet;
}
//******************************************************************************
BOOLEAN Write_Ateml_Func (struct sisusb_usb_data *sisusb, UCHAR *buf, ULONG count)
{
	ULONG i;
	
	if (count >= 4)
		printk ("Write_Ateml_Func : Cmd:%x Addr1:%x Addr2:%x N:%x\n", buf[0], buf[1], buf[2], buf[3]);
	else {
		printk ("Write_Ateml_Func: size must larger than 4\n");
		return FALSE;
	}
	if (I2CStart (sisusb, buf[0]))	{//Command
		if (!I2CWrite (sisusb, 0x10, buf[1]))	//Addr 1
			return FALSE;
		if (!I2CWrite (sisusb, 0x10, buf[2]))	//Addr 2
			return FALSE;
		if (!I2CWrite (sisusb, 0x10, buf[3]))	//N
			return FALSE;	
		//KeStallExecutionProcessor (20000);
		for (i=4; i<count; i++) {
			if (!I2CWrite (sisusb, 0x10, buf[i])) {
				printk ("Write_Atmel_Func: fail\n");
				WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
				mdelay (20); //20ms: wait for write cycle
				return FALSE;
			}
			//KeStallExecutionProcessor (20000); //6ms: wait for write cycle
		}
		WriteRegisterByte (sisusb, 0xFCE4, 0x40);	//stop
		mdelay (21); //20ms: wait for write cycle
		return TRUE;
	}	
	else
		printk ("Write_Atmel_func: send start fail\n");
	return FALSE;
}
//****************************************************************
BOOLEAN ATmelAckPolling (struct sisusb_usb_data *sisusb, ULONG Cmd)
{
	if (I2CStart (sisusb, (UCHAR)Cmd)) {
		printk ("ATmelAckPolling: Get ACK cmd:%x\n",Cmd);
		return TRUE;
	}
	else {
		printk ("ATmelAckPolling: NO ACK, cmd:%x\n", Cmd);
		return FALSE;
	}
}

BOOLEAN CheckATMel (struct sisusb_usb_data *sisusb)
{	
	return (ATmelAckPolling (sisusb, 0xB2));
}