//#define userTest_DBG
//#define userTest_DRAWIMG

#include <stdio.h>
#include <fcntl.h>
#include <sys/mman.h>
#include "gtf.h"
//#include "Tusb.h"

typedef enum {
	MODE_640X480X8X60=0,
	MODE_640X480X8X75,		//0x01
	MODE_640X480X16X60,		//0x02
	MODE_640X480X16X75,		//0x03

	MODE_640X480X24X60,		//0x04
	MODE_640X480X24X75,		//0x05
	MODE_640X480X32X60, 	//0x06
	MODE_640X480X32X75,		//0x07

	MODE_800X600X8X60,		//0x08
	MODE_800X600X8X75,		//0x09
	MODE_800X600X16X60,		//0x0a
	MODE_800X600X16X75,		//0x0b

	MODE_800X600X24X60,		//0x0c
	MODE_800X600X24X75,		//0x0d
	MODE_800X600X32X60,		//0x0e
	MODE_800X600X32X75,		//0x0f

	MODE_1024X768X8X60,		//0x10
	MODE_1024X768X8X75,		//0x11
	MODE_1024X768X16X60,	//0x12
	MODE_1024X768X16X75,	//0x13

	MODE_1024X768X24X60,	//0x14
	MODE_1024X768X24X75,	//0x15
	MODE_1024X768X32X60,	//0x16
	MODE_1024X768X32X75,	//0x17

	MODE_1280X1024X8X60,	//0x18
	MODE_1280X1024X8X75,	//0x19

	MODE_1280X1024X16X60,	//0x1a
	MODE_1280X1024X24X60,	//0x1b
	MODE_1280X1024X32X60,	//0x1c
//
	MODE_1280X1024X16X75,	//0x1d
	MODE_1280X1024X24X75,	//0x1e
	MODE_1280X1024X32X75,	//0x1f
//
	MODE_320X240X8X60,		//0x20
	MODE_320X240X16X60,		//0x21
	MODE_320X240X32X60,		//0x22
	MODE_800X480X8X60,		//0x23
	MODE_800X480X16X60,		//0x24
	MODE_800X480X32X60,		//0x25
}MODEDEF;

typedef struct _SISUSB_COMMAND sisusb_command, *psisusb_command;

struct _SISUSB_COMMAND {
	char operation;	/* see below */
	char data0;		/* operation dependent */
	char data1;		/* operation dependent */
	char data2;		/* operation dependent */
	unsigned long data3;		/* operation dependent */
	unsigned long data4;		/* for future use */
	unsigned long data5;		/* operation dependent */
	unsigned long data6;		/* for future use */
	unsigned long data7;		/* for future use */
	unsigned long data8;		/* for future use */
	unsigned long data9;		/* for future use */	
	unsigned char *data10;
	unsigned long data11; 
	unsigned long data12; 
	unsigned long data13; 
};

#define UPPER_NIBBLE( x ) \
        (((128|64|32|16) & (x)) >> 4)

#define LOWER_NIBBLE( x ) \
        ((1|2|4|8) & (x))

#define COMBINE_HI_8LO( hi, lo ) \
        ( (((unsigned)hi) << 8) | (unsigned)lo )

#define COMBINE_HI_4LO( hi, lo ) \
        ( (((unsigned)hi) << 4) | (unsigned)lo )

const unsigned char edid_v1_header[] = { 0x00, 0xff, 0xff, 0xff,
                                0xff, 0xff, 0xff, 0x00 };

const unsigned char edid_v1_descriptor_flag[] = { 0x00, 0x00 };


#define EDID_LENGTH                             0x80

#define EDID_HEADER                             0x00
#define EDID_HEADER_END                         0x07

#define ID_MANUFACTURER_NAME                    0x08
#define ID_MANUFACTURER_NAME_END                0x09
#define ID_MODEL				0x0a

#define ID_SERIAL_NUMBER			0x0c

#define MANUFACTURE_WEEK			0x10
#define MANUFACTURE_YEAR			0x11

#define EDID_STRUCT_VERSION                     0x12
#define EDID_STRUCT_REVISION                    0x13

#define DPMS_FLAGS				0x18
#define ESTABLISHED_TIMING_1                    0x23
#define ESTABLISHED_TIMING_2                    0x24
#define MANUFACTURERS_TIMINGS                   0x25
#define STANDARD_TIMING                    		0x26

#define DETAILED_TIMING_DESCRIPTIONS_START      0x36
#define DETAILED_TIMING_DESCRIPTION_SIZE        18
#define NO_DETAILED_TIMING_DESCRIPTIONS         4



#define DETAILED_TIMING_DESCRIPTION_1           0x36
#define DETAILED_TIMING_DESCRIPTION_2           0x48
#define DETAILED_TIMING_DESCRIPTION_3           0x5a
#define DETAILED_TIMING_DESCRIPTION_4           0x6c



#define PIXEL_CLOCK_LO     (unsigned)dtd[ 0 ]
#define PIXEL_CLOCK_HI     (unsigned)dtd[ 1 ]
#define PIXEL_CLOCK        (COMBINE_HI_8LO( PIXEL_CLOCK_HI,PIXEL_CLOCK_LO )*10000)

#define H_ACTIVE_LO        (unsigned)dtd[ 2 ]

#define H_BLANKING_LO      (unsigned)dtd[ 3 ]

#define H_ACTIVE_HI        UPPER_NIBBLE( (unsigned)dtd[ 4 ] )

#define H_ACTIVE           COMBINE_HI_8LO( H_ACTIVE_HI, H_ACTIVE_LO )

#define H_BLANKING_HI      LOWER_NIBBLE( (unsigned)dtd[ 4 ] )

#define H_BLANKING         COMBINE_HI_8LO( H_BLANKING_HI, H_BLANKING_LO )


#define V_ACTIVE_LO        (unsigned)dtd[ 5 ]

#define V_BLANKING_LO      (unsigned)dtd[ 6 ]

#define V_ACTIVE_HI        UPPER_NIBBLE( (unsigned)dtd[ 7 ] )

#define V_ACTIVE           COMBINE_HI_8LO( V_ACTIVE_HI, V_ACTIVE_LO )

#define V_BLANKING_HI      LOWER_NIBBLE( (unsigned)dtd[ 7 ] )

#define V_BLANKING         COMBINE_HI_8LO( V_BLANKING_HI, V_BLANKING_LO )



#define H_SYNC_OFFSET_LO   (unsigned)dtd[ 8 ]
#define H_SYNC_WIDTH_LO    (unsigned)dtd[ 9 ]

#define V_SYNC_OFFSET_LO   UPPER_NIBBLE( (unsigned)dtd[ 10 ] )
#define V_SYNC_WIDTH_LO    LOWER_NIBBLE( (unsigned)dtd[ 10 ] )

#define V_SYNC_WIDTH_HI    ((unsigned)dtd[ 11 ] & (1|2))
#define V_SYNC_OFFSET_HI   (((unsigned)dtd[ 11 ] & (4|8)) >> 2)

#define H_SYNC_WIDTH_HI    (((unsigned)dtd[ 11 ] & (16|32)) >> 4)
#define H_SYNC_OFFSET_HI   (((unsigned)dtd[ 11 ] & (64|128)) >> 6)


#define V_SYNC_WIDTH       COMBINE_HI_4LO( V_SYNC_WIDTH_HI, V_SYNC_WIDTH_LO )
#define V_SYNC_OFFSET      COMBINE_HI_4LO( V_SYNC_OFFSET_HI, V_SYNC_OFFSET_LO )

#define H_SYNC_WIDTH       COMBINE_HI_4LO( H_SYNC_WIDTH_HI, H_SYNC_WIDTH_LO )
#define H_SYNC_OFFSET      COMBINE_HI_4LO( H_SYNC_OFFSET_HI, H_SYNC_OFFSET_LO )

#define H_SIZE_LO          (unsigned)dtd[ 12 ]
#define V_SIZE_LO          (unsigned)dtd[ 13 ]

#define H_SIZE_HI          UPPER_NIBBLE( (unsigned)dtd[ 14 ] )
#define V_SIZE_HI          LOWER_NIBBLE( (unsigned)dtd[ 14 ] )

#define H_SIZE             COMBINE_HI_8LO( H_SIZE_HI, H_SIZE_LO )
#define V_SIZE             COMBINE_HI_8LO( V_SIZE_HI, V_SIZE_LO )

#define H_BORDER           (unsigned)dtd[ 15 ]
#define V_BORDER           (unsigned)dtd[ 16 ]

#define FLAGS              (unsigned)dtd[ 17 ]

#define INTERLACED         (FLAGS&128)
#define SYNC_TYPE	   (FLAGS&3<<3)  /* bits 4,3 */
#define SYNC_SEPARATE	   (3<<3)
#define HSYNC_POSITIVE	   (FLAGS & 4)
#define VSYNC_POSITIVE     (FLAGS & 2)

#define MONITOR_NAME            0xfc
#define MONITOR_LIMITS          0xfd
#define UNKNOWN_DESCRIPTOR      -1
#define DETAILED_TIMING_BLOCK   -2


#define DESCRIPTOR_DATA         5
#define V_MIN_RATE              block[ 5 ]
#define V_MAX_RATE              block[ 6 ]
#define H_MIN_RATE              block[ 7 ]
#define H_MAX_RATE              block[ 8 ]

#define MAX_PIXEL_CLOCK         (((int)block[ 9 ]) * 10)
#define GTF_SUPPORT		block[10]

#define DPMS_ACTIVE_OFF		(1 << 5)
#define DPMS_SUSPEND		(1 << 6)
#define DPMS_STANDBY		(1 << 7)

char* myname;
char gMonitorName[200];
char gVendorSign[200];

char gHorizSyncMax;
char gHorizSyncMin;
char gVertRefreshMax;
char gVertRefreshMin;
int  gH_ACTIVE, gV_ACTIVE;

void MSG( const char* x )
{
	printf("%s: %s\n", __FUNCTION__, x ); 
	
	//printf( "WHS: %s: %s\n", __FUNCTION__, x ); 
}

int parse_timing_description( unsigned char* dtd );

int parse_monitor_limits( unsigned char* block );

int block_type( unsigned char* block );

char* get_monitor_name( unsigned char const*  block );

char* get_vendor_sign( unsigned char const* block );
void get_max_resolution(block);

int parse_dpms_capabilities( unsigned char flags );


#define MCT_USB2VGA_XORGCONF		"/etc/X11/xorg.conf.tusbvga"
#define MCT_USB2VGA_XORGCONF_TMP	"/etc/X11/xorg.conf.tusbvga.tmp"

int parse_edid( unsigned char* edid )
{
  	unsigned i;
  	unsigned char* block;
  	char *monitor_name;
  	char monitor_alt_name[100];
  	unsigned char checksum = 0;
  	char *vendor_sign;
  	int ret = 0;
	


  	monitor_name = 0;
  
  	for( i = 0; i < EDID_LENGTH; i++ )
    	checksum += edid[ i ];

	if (  checksum != 0  ) {
		MSG( "EDID checksum failed - data is corrupt. Continuing anyway." );
		ret = 1;
  	} else
		MSG( "EDID checksum passed." );
  

	if ( strncmp( edid+EDID_HEADER, edid_v1_header, EDID_HEADER_END+1 ) ) {
		MSG( "first unsigned chars don't match EDID version 1 header" );
		MSG( "do not trust output (if any)." );
		ret = 1;
	}
	
	if(ret) return ret;
	
	vendor_sign = get_vendor_sign( edid + ID_MANUFACTURER_NAME ); 
	printf("The Monitor's supporting Mode is listed as below,\n");
	get_max_resolution(edid);
	printf("\nMax resolution: %dx%d\n", gH_ACTIVE, gV_ACTIVE);
	//printf( "Section \"Monitor\"\n" );

	block = edid + DETAILED_TIMING_DESCRIPTIONS_START;
	for( i = 0; i < NO_DETAILED_TIMING_DESCRIPTIONS; i++,
		block += DETAILED_TIMING_DESCRIPTION_SIZE ) {
		if ( block_type( block ) == MONITOR_NAME ) {
	  		monitor_name = get_monitor_name( block );
	  		break;
		}
    }
	if (!monitor_name) {
    	/* Stupid djgpp hasn't snprintf so we have to hack something together */
    	if(strlen(vendor_sign) + 10 > sizeof(monitor_alt_name))
      		vendor_sign[3] = 0;
    
    	sprintf(monitor_alt_name, "%s:%02x%02x",
	    vendor_sign, edid[ID_MODEL], edid[ID_MODEL+1]) ;
    	monitor_name = monitor_alt_name;
	}
	//printf( "\tIdentifier \"%s\"\n", monitor_name );
	//printf( "\tVendorName \"%s\"\n", vendor_sign );
	//printf( "\tModelName \"%s\"\n", monitor_name );

	strcpy(gMonitorName, monitor_name);
	strcpy(gVendorSign, vendor_sign);
	//printf( "\t gMonitorName \"%s\"\n",  gMonitorName);
	block = edid + DETAILED_TIMING_DESCRIPTIONS_START;
	for( i = 0; i < NO_DETAILED_TIMING_DESCRIPTIONS; i++,
	 	block += DETAILED_TIMING_DESCRIPTION_SIZE )	{

		if ( block_type( block ) == MONITOR_LIMITS )
			parse_monitor_limits( block );
	}
	return ret;

	// WHS: below is useless

	//parse_dpms_capabilities(edid[DPMS_FLAGS]);
	block = edid + DETAILED_TIMING_DESCRIPTIONS_START;

	for( i = 0; i < NO_DETAILED_TIMING_DESCRIPTIONS; i++, 
		block += DETAILED_TIMING_DESCRIPTION_SIZE ) {
		if ( block_type( block ) == DETAILED_TIMING_BLOCK )
			parse_timing_description( block );
    }
	return ret;
}

void get_max_resolution( unsigned char* edid)
{
	int i;
	unsigned short usData, usHorActPix, usVerActPix, usRefRate;
	unsigned char ucData, *block;
	gH_ACTIVE = 0;
	gV_ACTIVE = 0;	

	block = edid + ESTABLISHED_TIMING_1;
	usData =  * ((unsigned short *)(block));
	if (usData & 0x0004) {
		gH_ACTIVE = 640;
    	gV_ACTIVE = 480;
		printf(" 640x480 75Hz ");
	} //if

	if (usData & 0x0008) {
		gH_ACTIVE = 640;
    	gV_ACTIVE = 480;
		printf(" 640x480 72Hz ");
	} //if

	if (usData & 0x0020) {
		gH_ACTIVE = 640;
    	gV_ACTIVE = 480;
		printf(" 640x480 60Hz ");
	} //if
	//800x600
	if (usData & 0x4000) {
		gH_ACTIVE = 800;
    	gV_ACTIVE = 600;
		printf(" 800x600 75Hz ");
	} //if

	if (usData & 0x8000){
		gH_ACTIVE = 800;
    	gV_ACTIVE = 600;
		printf((" 800x600 72Hz "));
	} //if

	if (usData & 0x0001) {
		gH_ACTIVE = 800;
    	gV_ACTIVE = 600;
    	printf((" 800x600 60Hz "));
    } //if

    if (usData & 0x0002) {
		gH_ACTIVE = 800;
    	gV_ACTIVE = 600;
		printf ((" 800x600 56Hz "));
	} //if

	//1024x768
	if (usData & 0x0200){
		gH_ACTIVE = 1024;
    	gV_ACTIVE = 768;
    	printf ((" 1024x768 75Hz "));
    } //if

    if (usData & 0x0400) {
		gH_ACTIVE = 1024;
    	gV_ACTIVE = 768;
    	printf ((" 1024x768 70Hz "));
    } //if

    if (usData & 0x0800) {
		gH_ACTIVE = 1024;
    	gV_ACTIVE = 768;
    	printf ((" 1024x768 60Hz "));
    } //if

    //1280x1024
    if (usData & 0x0100){
		gH_ACTIVE = 1280;
    	gV_ACTIVE = 1024;
    	printf ((" 1280x1024 75Hz "));
    } //if
    
    //printf (("\nStandard Timing: \n"));
	block = edid + STANDARD_TIMING;
	for (i = 0; i < 8; i++) {
		if (*((unsigned short*)block) != 0x0101) {
			usHorActPix = (*block + 31) * 8;
          	block++;
          	usRefRate = (*block & 0x3F) + 60;
          	ucData = ((*block) >>6 );
			
          	switch (ucData) {
          	case 0:
           		usVerActPix = usHorActPix;             // Aspect ratio = 1:1
             	break;
          	case 1:
           		usVerActPix = (usHorActPix * 3) / 4;   // Aspect ratio = 4:3
             	break;
          	case 2:
            	usVerActPix = (usHorActPix * 4) / 5;   // Aspect ratio = 5:4
            	break;
          	case 3:
             	usVerActPix = (usHorActPix * 9) / 16;  // Aspect ratio = 16:9
             	break;
          	default :
            	usVerActPix = 0;
            	break;
			} //switch
		    printf (" %dx%d %dHz ", usHorActPix, usVerActPix, usRefRate);
			if((usHorActPix >= gH_ACTIVE) && (usVerActPix > gV_ACTIVE)) {
				gH_ACTIVE = usHorActPix;
    			gV_ACTIVE = usVerActPix;
			}
				
            block++;
       	}//End if
       	else {
          block= block + 2;
       	} //if-else
	}//End for
	//printf (("\n"));
	block = edid + DETAILED_TIMING_DESCRIPTIONS_START;
	for( i = 0; i < NO_DETAILED_TIMING_DESCRIPTIONS; i++,
		block += DETAILED_TIMING_DESCRIPTION_SIZE ) {
		if ( block_type( block ) == 0xFA ) {
			for (i = 0; i < 6; i++) {
				if (*((unsigned short*)block) != 0x0101) {
					usHorActPix = (*block + 31) * 8;
          			block++;
          			usRefRate = (*block & 0x3F) + 60;
          			ucData = ((*block) >>6 );
			
          			switch (ucData) {
          			case 0:
           				usVerActPix = usHorActPix;             // Aspect ratio = 1:1
             			break;
          			case 1:
           				usVerActPix = (usHorActPix * 3) / 4;   // Aspect ratio = 4:3
             			break;
          			case 2:
            			usVerActPix = (usHorActPix * 4) / 5;   // Aspect ratio = 5:4
            			break;
          			case 3:
             			usVerActPix = (usHorActPix * 9) / 16;  // Aspect ratio = 16:9
             			break;
          			default :
            			usVerActPix = 0;
            			break;
					} //switch
		    		printf (" %dx%d %dHz ", usHorActPix, usVerActPix, usRefRate);
					if((usHorActPix >= gH_ACTIVE) && (usVerActPix > gV_ACTIVE)) {
						gH_ACTIVE = usHorActPix;
    					gV_ACTIVE = usVerActPix;
					}
            		block++;
       			}//End if
       		else {
          		block= block + 2;
       		} //if-else
		}//End for
		//printf (("\n"));
	  		break;
		}
    }
}
int parse_timing_description( unsigned char* dtd )
{
	int htotal, vtotal;
	int a1,a2,a3,a4;

	//added by Louis Tsai
	gH_ACTIVE = H_ACTIVE;
    gV_ACTIVE = V_ACTIVE;
	return  0;


  htotal = H_ACTIVE + H_BLANKING;
  vtotal = V_ACTIVE + V_BLANKING;

  printf( "\tMode \t\"%dx%d\"", H_ACTIVE, V_ACTIVE );

	a1 = PIXEL_CLOCK / (vtotal*htotal);
	a2 = ((PIXEL_CLOCK - (a1*vtotal*htotal)) *1000) / (vtotal*htotal);

	a3 = PIXEL_CLOCK/(htotal*1000);
	a4 = ((PIXEL_CLOCK - (a3*htotal*1000)) * 1000) / (htotal*1000);

	printf( "\t# vfreq %d.%dHz, hfreq %d.%dkHz\n", a1, a2, a3, a4);
	
	a1 = PIXEL_CLOCK / 1000000;
	a2 = ((PIXEL_CLOCK - (a1*1000000)) * 1000) / 1000000;
	printf( "\t\tDotClock\t%d.%d\n", a1, a2);
/*
  printf( "\t# vfreq %3.3fHz, hfreq %6.3fkHz\n",
	  (double)PIXEL_CLOCK/((double)vtotal*(double)htotal),
	  (double)PIXEL_CLOCK/(double)(htotal*1000));

  printf( "\t\tDotClock\t%f\n", (double)PIXEL_CLOCK/1000000.0 );
*/
  printf( "\t\tHTimings\t%u %u %u %u\n", H_ACTIVE,
	  H_ACTIVE+H_SYNC_OFFSET, 
	  H_ACTIVE+H_SYNC_OFFSET+H_SYNC_WIDTH,
	  htotal );

  printf( "\t\tVTimings\t%u %u %u %u\n", V_ACTIVE,
	  V_ACTIVE+V_SYNC_OFFSET,
	  V_ACTIVE+V_SYNC_OFFSET+V_SYNC_WIDTH,
	  vtotal );

  if ( INTERLACED || (SYNC_TYPE == SYNC_SEPARATE)) {
    printf( "\t\tFlags\t%s\"%sHSync\" \"%sVSync\"\n",
	INTERLACED ? "\"Interlace\" ": "",
	HSYNC_POSITIVE ? "+": "-",
	VSYNC_POSITIVE ? "+": "-");
  }

  printf( "\tEndMode\n" );

  return 0;
}


int block_type( unsigned char* block )
{
  if ( !strncmp( edid_v1_descriptor_flag, block, 2 ) )
    {
      //printf("\t# Block type: 2:%x 3:%x\n", block[2], block[3]);

      /* descriptor */

      if ( block[ 2 ] != 0 )
	return UNKNOWN_DESCRIPTOR;


      return block[ 3 ];
    } else {

      /* detailed timing block */

      return DETAILED_TIMING_BLOCK;
    }
}

char* get_monitor_name( unsigned char const* block )
{
  static char name[ 13 ];
  unsigned i;
  unsigned char const* ptr = block + DESCRIPTOR_DATA;


  for( i = 0; i < 13; i++, ptr++ )
    {

      if ( *ptr == 0xa )
	{
	  name[ i ] = 0;
	  return name;
	}

      name[ i ] = *ptr;
    }


  return name;
}

char* get_vendor_sign( unsigned char const* block )
{
  static char sign[4];
  unsigned short h;

  /*
     08h	WORD	big-endian manufacturer ID (see #00136)
		    bits 14-10: first letter (01h='A', 02h='B', etc.)
		    bits 9-5: second letter
		    bits 4-0: third letter
  */
  h = COMBINE_HI_8LO(block[0], block[1]);
  sign[0] = ((h>>10) & 0x1f) + 'A' - 1;
  sign[1] = ((h>>5) & 0x1f) + 'A' - 1;
  sign[2] = (h & 0x1f) + 'A' - 1;
  sign[3] = 0;
  return sign;
}

int parse_monitor_limits( unsigned char* block )
{
	printf( "The capability of the attached monitor: ");	
  printf( "HorizSync %u-%u, ", H_MIN_RATE, H_MAX_RATE ); 
  printf( "VertRefresh %u-%u\n", V_MIN_RATE, V_MAX_RATE );
  gHorizSyncMax = H_MAX_RATE;
  gVertRefreshMax = V_MAX_RATE;
  gHorizSyncMin = H_MIN_RATE;
  gVertRefreshMin = V_MIN_RATE;
  return 0;
  gHorizSyncMax = H_MAX_RATE;
  gVertRefreshMax = V_MAX_RATE;
  gHorizSyncMin = H_MIN_RATE;
  gVertRefreshMin = V_MIN_RATE;
  if ( MAX_PIXEL_CLOCK == 10*0xff )
    printf( "\t# Max dot clock not given\n" );
  else
    printf( "\t# Max dot clock (video bandwidth) %u MHz\n", (int)MAX_PIXEL_CLOCK );

  if ( GTF_SUPPORT )
    {
      printf( "\t# EDID version 3 GTF given: contact author\n" );
    }
  
  return 0;
}

int parse_dpms_capabilities(unsigned char flags)
{
  printf("\t# DPMS capabilities: Active off:%s  Suspend:%s  Standby:%s\n\n",
    (flags & DPMS_ACTIVE_OFF) ? "yes" : "no",
    (flags & DPMS_SUSPEND)    ? "yes" : "no",
    (flags & DPMS_STANDBY)    ? "yes" : "no");
  return 0;
}

void print_usage()
{
	printf("Usage: triggerEDID [OPTION]...\n\n");
	printf("Crate or Update the reference xorg.conf file - %s \n", MCT_USB2VGA_XORGCONF);
	printf("With no option, the application will create %s "
	       "depend on original xorg.conf at first time. Next time,"
		   "it will just modify the %s.\n\n\n",
			MCT_USB2VGA_XORGCONF, MCT_USB2VGA_XORGCONF);

	printf("  -c,\tforce to create new %s depend on xorg.conf\n", MCT_USB2VGA_XORGCONF);
	printf("  -h,\tprintf this usage.\n\n");


}


int main(int argc ,char **argv){

	int fd = 0, num;
	char buf;
	unsigned long ioctlCmd[5], rectCmd[5], color, width, height;
	FILE *xorgFp, *xorgFpTmp;
	unsigned char select, vgaMode, name[16], findit = 0;
	int f1, index = 0, pause;

	unsigned char edidBuf[512];

	unsigned char xorgBuf[512][512];
	unsigned char tmp[512], tmp1[512];
	unsigned char lineBuf[512];
	char isFound = 0;

	sisusb_command x;

	if(argc == 2) {
		if(!strcmp(argv[1], "-h")) {
			print_usage();
			exit(0);
		}
		if(!strcmp(argv[1], "-c")) {
			unlink(MCT_USB2VGA_XORGCONF);
		}
	}else
	if(argc > 2) {
		print_usage();
		exit(1);
	}
	
	fd = open("/dev/triggerusbvga0" , O_RDWR);

	if(fd < 0)

    {

        printf("Error: cannot open trigger device.\n");

        exit(1);

    }
		
	x.operation = 0x14; // WHS: TUCMD_EDID
	x.data10 = edidBuf;	
	
	num = ioctl(fd, 0, &x);

	if(num){

		printf("Error: EDID failed.\n");
		close(fd);
        exit(1);
		
	}		

	if(parse_edid(edidBuf)) {
		printf("Error: Please attach monitor to read EDID data first.\n");
		close(fd);
        exit(1);
	}
	// [WHS: update xorg.conf

	xorgFp = fopen(MCT_USB2VGA_XORGCONF, "ra");
	if(!xorgFp){
		char cmdline[64];
		printf("Initialize the %s file.\n", MCT_USB2VGA_XORGCONF);
		if((xorgFp=fopen("/etc/X11/xorg.conf", "ra")) != NULL) {
			sprintf(cmdline, "cp /etc/X11/xorg.conf %s\n", MCT_USB2VGA_XORGCONF);
			system(cmdline);
			fclose(xorgFp);
			xorgFp = fopen(MCT_USB2VGA_XORGCONF, "ra");
		}else {
			//new linux dist dosen't have xorg.conf default,
			//we should help user to create here.
			xorgFp = fopen(MCT_USB2VGA_XORGCONF, "w");
			fprintf(xorgFp,"Section \"Device\"\n");
        	fprintf(xorgFp,"Identifier\t\"Configured Video Device\"\n");
			fprintf(xorgFp,"EndSection\n\n");
			fprintf(xorgFp,"Section \"Monitor\"\n");
        	fprintf(xorgFp,"Identifier\t\"Configured Monitor\"\n");
			fprintf(xorgFp,"EndSection\n\n");
			fprintf(xorgFp,"Section \"Screen\"\n");
        	fprintf(xorgFp,"Identifier\t\"Default Screen\"\n");
        	fprintf(xorgFp,"Monitor\t\t\"Configured Monitor\"\n");
        	fprintf(xorgFp,"Device\t\t\"Configured Video Device\"\n");
			fprintf(xorgFp,"EndSection\n\n");
			fprintf(xorgFp,"Section \"ServerLayout\"\n");
        	fprintf(xorgFp,"Identifier\t\"Default Layout\"\n");
        	fprintf(xorgFp,"Screen\t\t\"Default Screen\"\n");
			fprintf(xorgFp,"EndSection\n\n");
			fclose(xorgFp);
			xorgFp = fopen(MCT_USB2VGA_XORGCONF, "ra");
		}
	}

	while(!feof(xorgFp)){

		fscanf(xorgFp, "%s", tmp);
		//printf("fscanf: %s\n", tmp);
		//fgets(lineBuf, 512, xorgFp);
		//printf("fgets: %s\n", lineBuf);		
		#if 1
		if(!strcmp("Section", tmp)){ 

			//printf("%s\n", tmp);		
			fscanf(xorgFp, "%s", tmp);
			//printf("fscanf: %s\n", tmp);				
			if(!strcmp("\"Monitor\"", tmp)){

				//printf("fscanf: %s\n", tmp);				
				//fscanf(xorgFp, "%s", tmp);
				do{
				fgets(lineBuf, 512, xorgFp);
				//if(!strcmp("\"Monitor\"", tmp)){
				sscanf(lineBuf, "%s", tmp1);

				}while(strcmp("Identifier", tmp1) && strcmp("identifier", tmp1));
			
				//printf("fgets: %s", lineBuf);				
				for(f1=0; f1<strlen(lineBuf); f1++){
					//printf("%c", lineBuf[f1]);				
					if(lineBuf[f1] == '"') break;
					
				}

				index = 0;
				
				for(f1=f1+1; f1<strlen(lineBuf); f1++){

					if(lineBuf[f1] == '"'){ 
						tmp1[index] = '\0';
						break;
					}
					tmp1[index++] = lineBuf[f1];
					
				}
				
				//printf("tmp1: %s\n", tmp1);
				
				if(!strcmp(gMonitorName, tmp1)){

					isFound = 1;
					break;
					
				}												
				
			}
									
		}
		#endif

		
	}

	if(!isFound){
		mode *m;

		m = vert_refresh(gH_ACTIVE, gV_ACTIVE, 60, 0, 0);

		fclose(xorgFp);
		xorgFp = fopen(MCT_USB2VGA_XORGCONF, "a");
		
		fprintf(xorgFp, "\nSection \"Monitor\"\n");
		fprintf(xorgFp, "\tIdentifier \"%s\"\n", gMonitorName);
    	fprintf(xorgFp, "\tVendorName \"%s\"\n", gVendorSign);
    	fprintf(xorgFp, "\tModelName \"%s\"\n", gMonitorName);
		fprintf(xorgFp, "\tHorizSync %u-%u\n", gHorizSyncMin, gHorizSyncMax); 
		fprintf(xorgFp, "\tVertRefresh %u-%u\n", gVertRefreshMin, gVertRefreshMax);
		if(m) {
			fprintf(xorgFp, " \tModeLine \"%dx%d\"  %.2f"
							"  %d %d %d %d"
							"  %d %d %d %d"
							"  -HSync +Vsync\n\n",
							m->hr, m->vr, m->pclk,
							m->hr, m->hss, m->hse, m->hfl,
							m->vr, m->vss, m->vse, m->vfl);
		}
		fprintf(xorgFp, "EndSection\n");

		printf("Create the monitor section for the current monitor.\n");

		//printf("\nSection \"Monitor\"\n");
		//printf("\tIdentifier \"%s\"\n", gMonitorName);
    	//printf("\tVendorName \"%s\"\n", gVendorSign);
    	//printf("\tModelName \"%s\"\n", gMonitorName);
		//printf("EndSection\n");

		//sscanf("Identifier \"HannStar T500HSD\"", "%s %s", tmp, tmp1);
		//printf("%s\n", tmp1);		

	}

	isFound = 0;

	// ]

	// [ WHS: to find mct screen section

	fclose(xorgFp);
	xorgFp = fopen(MCT_USB2VGA_XORGCONF, "r");

	while(!feof(xorgFp)){

		fscanf(xorgFp, "%s", tmp);
		//printf("fscanf: %s\n", tmp);
		//fgets(lineBuf, 512, xorgFp);
		//printf("fgets: %s\n", lineBuf);		
		#if 1
		if(!strcmp("#MCT-USB-Screen", tmp)){ 

			isFound = 1;

			// [WHS: update monitor in screen section

			fclose(xorgFp);
			xorgFp = fopen(MCT_USB2VGA_XORGCONF, "r");	

			index = 0;
		
			while(!feof(xorgFp)){

				fgets(xorgBuf[index], 512, xorgFp);
				//sscanf(xorgBuf[index], "%s", tmp1);
				if(!strcmp("\tIdentifier \"MCT-USB-Screen\"\n", xorgBuf[index])){ 
					
					index++;
					fgets(xorgBuf[index], 512, xorgFp);
					//printf("fgets: %s\n", xorgBuf[index]);
					sscanf(xorgBuf[index], "%s", tmp1);
					//printf("sscanf: %s\n", tmp1);
					index++;
					fgets(xorgBuf[index], 512, xorgFp);
					//printf("fgets: %s\n", xorgBuf[index]);
					sscanf(xorgBuf[index], "%s", tmp1);
					//printf("sscanf: %s\n", tmp1);
					while(strcmp("Monitor", tmp1)){
						index++;
						fgets(xorgBuf[index], 512, xorgFp);
						sscanf(xorgBuf[index], "%s", tmp1);
						//printf("sscanf: %s\n", tmp1);
					}					
					sprintf(xorgBuf[index], "\tMonitor \"%s\"\n", gMonitorName);
					while(strcmp("Modes", tmp1)){
						index++;
						fgets(xorgBuf[index], 512, xorgFp);
						sscanf(xorgBuf[index], "%s", tmp1);
						//printf("sscanf: %s\n", tmp1);
					}					
					sprintf(xorgBuf[index], "\t\tModes \"%dx%d\"\n", gH_ACTIVE, gV_ACTIVE);
					printf("Update MCT-USB-Screen's monitor.\n");
				}
				index++;
			}

			fclose(xorgFp);
			xorgFp = fopen(MCT_USB2VGA_XORGCONF, "w");	

			for(f1=0; f1<index-1; f1++) fprintf(xorgFp, "%s", xorgBuf[f1]);

			break;

			// WHS: update monitor in screen section]
									
		}
		#endif

		
	}

	// WHS: to find mct screen section]

	// [WHS: append mct screen section

	if(!isFound){

		fclose(xorgFp);
		xorgFp = fopen(MCT_USB2VGA_XORGCONF, "a");
				
		fprintf(xorgFp, "\n#MCT-USB-Screen\n");
		fprintf(xorgFp, "Section \"Device\"\n");
		fprintf(xorgFp, "\tIdentifier \"trigger-plus\"\n");
		fprintf(xorgFp, "\tDriver \"tusb\"\n");
		fprintf(xorgFp, "EndSection\n\n");
		
		fprintf(xorgFp, "Section \"Screen\"\n");
		fprintf(xorgFp, "\tIdentifier \"MCT-USB-Screen\"\n");
		fprintf(xorgFp, "\tDevice \"trigger-plus\"\n");
		fprintf(xorgFp, "\tMonitor \"%s\"\n", gMonitorName);
		fprintf(xorgFp, "\tDefaultDepth 24\n");
		//added by Louis Tsai 
		fprintf(xorgFp, "\tSubSection \"Display\"\n");
		fprintf(xorgFp, "\t\tModes \"%dx%d\"\n", gH_ACTIVE, gV_ACTIVE);
		fprintf(xorgFp, "\tEndSubSection\n");
		fprintf(xorgFp, "EndSection\n");

		printf("Create mandatory sections for MCT-USB-Screen.\n");
		
	}

	//  WHS: append mct screen section]

	fclose(xorgFp);

	// Louis Tsai: Try to put the trigger's screen into ServerLayout Section
	//             Now, we assume there only one screen in configuration file.
	//			   The logical is too complicate for mutiple screen, maybe can 
	//			   implement in future.

	
	if(!isFound){
		char primary_id[32];
		unsigned char have_layout = 0;

		xorgFp = fopen(MCT_USB2VGA_XORGCONF, "r");
		xorgFpTmp = fopen(MCT_USB2VGA_XORGCONF_TMP, "w");

		while(fgets(lineBuf, 512, xorgFp) != NULL){
			if(strstr(lineBuf, "\"ServerLayout\"") != NULL) {
				fprintf(xorgFpTmp, "%s", lineBuf);
				while(fgets(lineBuf, 512, xorgFp) != NULL ){
					if(strstr(lineBuf, "Screen") != NULL){
						char *tmpstr;
						fprintf(xorgFpTmp, "%s", lineBuf);
						tmpstr = strtok(lineBuf,"\"");
						tmpstr = strtok(NULL,"\"");
						if(tmpstr != NULL)
							strcpy(primary_id, tmpstr);
						else
							strcpy(primary_id, "NO-SCREEN");
						
						
						if(strcmp(primary_id, "NO-SCREEN") != 0) 
							fprintf(xorgFpTmp, "\tScreen\t\"MCT-USB-Screen\" leftof \"%s\"\n" ,primary_id);
						break;
					}else
					if(strstr(lineBuf, "EndSection") != NULL) {
						fprintf(xorgFpTmp, "%s", lineBuf);
						break;
					}
					else
						fprintf(xorgFpTmp, "%s", lineBuf);
				}
				have_layout = 1;
			}
			else
				fprintf(xorgFpTmp, "%s", lineBuf);
		}
		if(!have_layout) {
			fprintf(xorgFpTmp, "Section \"ServerLayout\"\n");
        	fprintf(xorgFpTmp, "Identifier\t\"Default Layout\"\n");
        	fprintf(xorgFpTmp, "Screen\t\t\"Default Screen\"\n");
			fprintf(xorgFpTmp, "Screen\t\"MCT-USB-Screen\" leftof \"Default Screen\"\n");
			fprintf(xorgFpTmp, "EndSection\n\n");
		}

		fclose(xorgFp);
		fclose(xorgFpTmp);
		
		unlink(MCT_USB2VGA_XORGCONF);
		rename(MCT_USB2VGA_XORGCONF_TMP, MCT_USB2VGA_XORGCONF);
	}
	

	close(fd);

	return 0;
}




