/*
 * tusbvga - usb kernel driver for MCT Trigger based USB2VGA dongles
 *
 * Main part
 *
 * Copyright (C) 2008 by Magic Control Technology Inc. 
 * Copyright (C) 2005 by Thomas Winischhofer, Vienna, Austria
 *
 * If distributed as part of the Linux kernel, this code is licensed under the
 * terms of the GPL v2.
 *
 * Otherwise, the following license terms apply:
 *
 * * Redistribution and use in source and binary forms, with or without
 * * modification, are permitted provided that the following conditions
 * * are met:
 * * 1) Redistributions of source code must retain the above copyright
 * *    notice, this list of conditions and the following disclaimer.
 * * 2) Redistributions in binary form must reproduce the above copyright
 * *    notice, this list of conditions and the following disclaimer in the
 * *    documentation and/or other materials provided with the distribution.
 * * 3) The name of the author may not be used to endorse or promote products
 * *    derived from this software without specific psisusbr written permission.
 * *
 * * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESSED OR
 * * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	Authors:
 *		Louis Tsai	    <louis@mct.com.tw>
 *		Ed Hsieh	    <ed.hsieh@mct.com.tw>
 * 	 	Thomas Winischhofer <thomas@winischhofer.net>
 */

//#define tusbvga_DBG
//#define tusbvga_OPT
//#define FB_SUPPORT
//#define TRIGGER_DEMO
//#define tusbvga_SMP
//#define tusbvga_MT
//#define tusbvga_HOTPLUG // WHS: driver_HOTPLUG, shadow_HOTPLUG
#define tusbvga_EDID // WHS: driver_EDID, shadow_EDID
//#define I2C_ATMEL	 //LOUIS: when atmel AT88SC0808C connected on trigger's i2c bus(shared with D-SUB i2c pin)
//#define tusbvga_FLIP // WHS: shadow_FLIP
#define V12 1

//#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/poll.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/spinlock.h>

#include <asm/delay.h>
  
#include <linux/kref.h>
#include <linux/usb.h>
#include <linux/smp_lock.h>
#include <linux/vmalloc.h>

#include <linux/fb.h> // WHS: framebuffer supporting
#include <linux/mutex.h>

#include "Tusb.h"
#include "ball1.c"
#include "ball2.c"
#include "i2c.h"
//#include "msg.c"

struct sisusb_usb_data *in_sisusb;
static u32 in_modeindex;
static u8 info_module_exit = 0;

#include <linux/syscalls.h>
//#include <asm/ipc.h>

//extern asmlinkage long sys_msgget(key_t key, int msgflg);

#define SISUSB_DONTSYNC

static wait_queue_head_t setmode_wq, done_setmode_wq;	
/* Forward declarations / clean-up routines */
static struct usb_driver triggerusb_driver;

#ifdef tusbvga_MT
static unsigned char gBuf[1024*768*2];
#endif

static unsigned char gSetMode = 0;
int gTimeOut = 2*HZ;
//kmem_cache_alloc
DECLARE_MUTEX(disconnect_sem_t);

typedef enum {
	MODE_640X480X8X60=0,
	MODE_640X480X8X75,
	MODE_640X480X16X60,
	MODE_640X480X16X75,
	MODE_640X480X24X60,
	MODE_640X480X24X75,
	MODE_640X480X32X60,
	MODE_640X480X32X75,	
	MODE_800X600X8X60,
	MODE_800X600X8X75,
	MODE_800X600X16X60,
	MODE_800X600X16X75,
	MODE_800X600X24X60,
	MODE_800X600X24X75,
	MODE_800X600X32X60,
	MODE_800X600X32X75,
	MODE_1024X768X8X60,
	MODE_1024X768X8X75,
	MODE_1024X768X16X60,
	MODE_1024X768X16X75,
	MODE_1024X768X24X60, 
	MODE_1024X768X24X75,
	MODE_1024X768X32X60, // WHS: 22: 0x16
	MODE_1024X768X32X75,
	MODE_1280X1024X8X60, // WHS: 0x18
	MODE_1280X1024X8X75, // 0x19
	MODE_1280X1024X16X60, //0x1a
	MODE_1280X1024X24X60, // 0x1b
	MODE_1280X1024X32X60, // 0x1c
	MODE_1280X1024X16X75, // 0x1d
	MODE_1280X1024X24X75, // 0x1e
	MODE_1280X1024X32X75, // 0x1f	
	MODE_320X240X8X60, // 0x20
	MODE_320X240X16X60, // 0x21
	MODE_320X240X24X60, // 0x22
	MODE_320X240X32X60, // 0x23
	MODE_800X480X8X60, // 0x24
	MODE_800X480X16X60, // 0x25
	MODE_800X480X24X60, // 0x26
	MODE_800X480X32X60, // WHS: 39 = 0x27
//
	MODE_1152X864X8X60,
	MODE_1152X864X16X60,
	MODE_1152X864X24X60,
	MODE_1152X864X32X60,
	
	MODE_1280X720X8X60,
	MODE_1280X720X16X60,
	MODE_1280X720X24X60,
	MODE_1280X720X32X60,
//
	MODE_1280X800X8X60,
	MODE_1280X800X16X60,
	MODE_1280X800X24X60,
	MODE_1280X800X32X60,
//		
	MODE_1280X768X8X60,
	MODE_1280X768X16X60,
	MODE_1280X768X24X60,
	MODE_1280X768X32X60,
	
	MODE_1280X960X8X60,
	MODE_1280X960X16X60,
	MODE_1280X960X24X60,
	MODE_1280X960X32X60,
//
	MODE_1360X768X8X60,
	MODE_1360X768X16X60,
	MODE_1360X768X24X60,
	MODE_1360X768X32X60,	
//	
	MODE_1366X768X8X60,
	MODE_1366X768X16X60,
	MODE_1366X768X24X60,
	MODE_1366X768X32X60,	
//
	MODE_1400X1050X8X60,
	MODE_1400X1050X16X60,
	MODE_1400X1050X24X60,
	MODE_1400X1050X32X60,
	
	MODE_1440X900X8X60,
	MODE_1440X900X16X60,
	MODE_1440X900X24X60,
	MODE_1440X900X32X60,
	
	MODE_1600X1200X8X60,
	MODE_1600X1200X16X60,
	MODE_1600X1200X24X60,
	MODE_1600X1200X32X60,
	
	MODE_1680X1050X8X60,
	MODE_1680X1050X16X60,
	MODE_1680X1050X24X60,
	MODE_1680X1050X32X60,
//
	MODE_1920X1080X8X60,
	MODE_1920X1080X16X60,
	MODE_1920X1080X24X60,
	MODE_1920X1080X32X60,
	
	MODE_1920X1200X8X60,
	MODE_1920X1200X16X60,
	MODE_1920X1200X24X60,
	MODE_1920X1200X32X60,	
}MODEDEF;



void TransferBlockData(struct sisusb_usb_data *sisusb,
		       u32		 SrcX,
		       u32		 SrcY,
		       u32		 DestX,
		       u32		 DestY,
		       u32		 ColorFormat,
		       u32		 SrcBitsPlane,
		       u32		 ScrXPoint,
		       u32		 ScrYPoint,
		       u32		 ScreenLineLen,
		       u32		 HScale,
		       u32		 VScale,
		       unsigned char*	 pdata,
		       int               users,
		       u32		 len);

extern BOOLEAN VgaSetModeByPanelCfg(struct sisusb_usb_data *, ULONG);
void LightOn(struct sisusb_usb_data *sisusb, BOOLEAN On);
//*************************************************************
static void sisusb_free_buffers(struct sisusb_usb_data *sisusb)
{
	int i;

#ifdef tusbvga_DBG
	
printk("enter %s\n", __FUNCTION__);

#endif

	for (i = 0; i < NUMOBUFS; i++) 
	{
		if (sisusb->obuf[i]) 
		{
		    usb_buffer_free(sisusb->sisusb_dev, sisusb->obufsize,
				    sisusb->obuf[i], sisusb->transfer_dma_out[i]);
		    sisusb->obuf[i] = NULL;
		}
	}
	if (sisusb->ibuf) 
	{				
	    usb_buffer_free(sisusb->sisusb_dev, sisusb->ibufsize,
			    sisusb->ibuf, sisusb->transfer_dma_in);
	    sisusb->ibuf = NULL;
	}
}

//**********************************************************
static void sisusb_free_urbs(struct sisusb_usb_data *sisusb)
{
	int i;
	#ifdef tusbvga_DBG
printk("enter %s\n", __FUNCTION__);
	#endif
	for (i = 0; i < NUMOBUFS; i++) 
	{
		usb_free_urb(sisusb->sisurbout[i]);
		sisusb->sisurbout[i] = NULL;
	}
	usb_free_urb(sisusb->sisurbin);
	sisusb->sisurbin = NULL;
}

/* Level 0: USB transport layer */
/* 1. out-bulks */
/* out-urb management */
/* Return 1 if all free, 0 otherwise */
static int sisusb_all_free(struct sisusb_usb_data *sisusb)
{
	int i;

	for (i = 0; i < sisusb->numobufs; i++) 
	{
	    if (sisusb->urbstatus[i] & SU_URB_BUSY)
		return 0;
	}
	return 1;
}

/* Kill all busy URBs */
//**************************************************************
static void sisusb_kill_all_busy(struct sisusb_usb_data *sisusb)
{
	int i;

	if (sisusb_all_free(sisusb))
	    return;

	for (i = 0; i < sisusb->numobufs; i++) 
	{

	     if (sisusb->urbstatus[i] & SU_URB_BUSY)
		 usb_kill_urb(sisusb->sisurbout[i]);

	}
}

/* Return 1 if ok, 0 if error (not all complete within timeout) */
static int sisusb_wait_all_out_complete(struct sisusb_usb_data *sisusb)
{
	int timeout = 5 * HZ, i = 1;

	wait_event_timeout(sisusb->wait_q,
			   (i = sisusb_all_free(sisusb)),
			    //timeout);
			    gTimeOut);

	return i;
}

//****************************************************************
static int sisusb_outurb_available(struct sisusb_usb_data *sisusb)
{
	int i;

	for (i = 0; i < sisusb->numobufs; i++) 
	{
	     if ((sisusb->urbstatus[i] & (SU_URB_BUSY|SU_URB_ALLOC)) == 0)
		  return i;
	}
	return -1;
}

//***************************************************************
static int sisusb_get_free_outbuf(struct sisusb_usb_data *sisusb)
{
	int i, timeout = 5 * HZ;

	wait_event_timeout(sisusb->wait_q,
			   ((i = sisusb_outurb_available(sisusb)) >= 0),
			   //timeout);
			   gTimeOut);

	return i;
}


/* completion callback */
static void sisusb_bulk_completeout(struct urb *urb, struct pt_regs *regs)
{
	
	unsigned long begWait, endWait;

	struct sisusb_urb_context *context = urb->context;
	struct sisusb_usb_data *sisusb;
	
	if (!context)
	    return;

	sisusb = context->sisusb;

	if (!sisusb || !sisusb->sisusb_dev || !sisusb->present)
	    return;
 
        #ifndef SISUSB_DONTSYNC
	if (context->actual_length)
	    *(context->actual_length) += urb->actual_length;
        #endif

	sisusb->urbstatus[context->urbindex] &= ~SU_URB_BUSY;
	wake_up(&sisusb->wait_q);
}

static int
sisusb_bulkout_msg(struct sisusb_usb_data *sisusb, 
                   int                    index, 
                   unsigned int           pipe, 
                   void                   *data,
		   int                    len, 
		   int                    *actual_length, 
		   int                    timeout, 
		   unsigned int           tflags,
		   dma_addr_t             transfer_dma)
{
	struct urb *urb = sisusb->sisurbout[index];
	int retval, byteswritten = 0;
	unsigned long begWait, endWait;

	/* Set up URB */
	urb->transfer_flags = 0;

	usb_fill_bulk_urb(urb, sisusb->sisusb_dev, pipe, data, len,
		          (usb_complete_t)sisusb_bulk_completeout, &sisusb->urbout_context[index]); 
	// WHS: still have completion function

	urb->transfer_flags |= tflags;
	urb->actual_length = 0;

	if ((urb->transfer_dma = transfer_dma))
	     urb->transfer_flags |= URB_NO_TRANSFER_DMA_MAP;

	/* Set up context */
	sisusb->urbout_context[index].actual_length = (timeout) ? NULL : actual_length;

	/* Declare this urb/buffer in use */
	sisusb->urbstatus[index] |= SU_URB_BUSY;

	/* Submit URB */
	retval = usb_submit_urb(urb, GFP_ATOMIC);

	/* If OK, and if timeout > 0, wait for completion */
	if ((retval == 0) && timeout) 
	{

		begWait = jiffies;
	
		wait_event_timeout(sisusb->wait_q,
				   (!(sisusb->urbstatus[index] & SU_URB_BUSY)),
				   timeout);

		endWait = jiffies;

		#ifdef tusbvga_OPT

		printk(KERN_INFO"triggerusb: WHS: send bulk wait elapse time: %d\n", endWait - begWait);

		#endif 
		
		if (sisusb->urbstatus[index] & SU_URB_BUSY) {
			/* URB timed out... kill it and report error */
			usb_kill_urb(urb);
			retval = -ETIMEDOUT;
		} else {
			/* Otherwise, report urb status */
			retval = urb->status;
			byteswritten = urb->actual_length;
		}
	}

	if (actual_length)
	    *actual_length = byteswritten;

	return retval;
}

/* 2. in-bulks */

/* completion callback */

static void
sisusb_bulk_completein(struct urb *urb, struct pt_regs *regs)
{
	struct sisusb_usb_data *sisusb = urb->context;

	if (!sisusb || !sisusb->sisusb_dev || !sisusb->present)
	    return;

	sisusb->completein = 1;
	wake_up(&sisusb->wait_q);
}

static int
sisusb_bulkin_msg(struct sisusb_usb_data *sisusb, 
                  unsigned int           pipe, 
                  void                   *data, 
                  int                    len,
		  int                    *actual_length, 
		  int                    timeout, 
		  unsigned int           tflags, 
		  dma_addr_t             transfer_dma)
{
	struct urb *urb = sisusb->sisurbin;
	int retval, readbytes = 0;

	urb->transfer_flags = 0;

	usb_fill_bulk_urb(urb, sisusb->sisusb_dev, pipe, data, len,
			  (usb_complete_t)sisusb_bulk_completein, sisusb);

	urb->transfer_flags |= tflags;
	urb->actual_length = 0;

	if ((urb->transfer_dma = transfer_dma))
	     urb->transfer_flags |= URB_NO_TRANSFER_DMA_MAP;

	sisusb->completein = 0;
	retval = usb_submit_urb(urb, GFP_ATOMIC);
	if (retval == 0) 
	{
		wait_event_timeout(sisusb->wait_q, sisusb->completein, timeout);
		if (!sisusb->completein) {
			/* URB timed out... kill it and report error */
			usb_kill_urb(urb);
			retval = -ETIMEDOUT;
		} else {
			/* URB completed within timout */
			retval = urb->status;
			readbytes = urb->actual_length;
		}
	}

	if (actual_length)
		*actual_length = readbytes;

	return retval;
}


/* Level 1:  */

/* Send a bulk message of variable size
 *
 * To copy the data from userspace, give pointer to "userbuffer",
 * to copy from (non-DMA) kernel memory, give "kernbuffer". If
 * both of these are NULL, it is assumed, that the transfer
 * buffer "sisusb->obuf[index]" is set up with the data to send.
 * Index is ignored if either kernbuffer or userbuffer is set.
 * If async is nonzero, URBs will be sent without waiting for
 * completion of the previous URB.
 *
 * (return 0 on success)
 */
static int sisusb_send_bulk_msg(struct sisusb_usb_data *sisusb,int ep, 
                                int len,char *kernbuffer, const char __user *userbuffer, 
                                int index,ssize_t *bytes_written, unsigned int tflags, int async)
{

	unsigned long begTime = jiffies, endTime;
	unsigned long bTime, eTime;
	unsigned long bmTime, emTime;
	unsigned long begWait, endWait;

	int result = 0, retry, count = len;
	int passsize, thispass, transferred_len = 0;
	int fromuser = (userbuffer != NULL) ? 1 : 0;
	int fromkern = (kernbuffer != NULL) ? 1 : 0;
	unsigned int pipe;
	char *buffer;

	(*bytes_written) = 0;

	/* Sanity check */
	if (!sisusb || !sisusb->present || !sisusb->sisusb_dev){		
		printk(KERN_INFO"WHS: fault at: %d\n", __LINE__);
	    return -ENODEV;
	}

	/* If we copy data from kernel or userspace, force the
	 * allocation of a buffer/urb. If we have the data in
	 * the transfer buffer[index] already, reuse the buffer/URB
	 * if the length is > buffer size. (So, transmitting
	 * large data amounts directly from the transfer buffer
	 * treats the buffer as a ring buffer. However, we need
	 * to sync in this case.)
	 */
	if (fromuser || fromkern)
	    index = -1;
	else if (len > sisusb->obufsize)
		async = 0;

	pipe = usb_sndbulkpipe(sisusb->sisusb_dev, ep);

	do 
	{

		bTime = jiffies;
	
		passsize = thispass = (sisusb->obufsize < count) ?
				       sisusb->obufsize : count;
		
		if (index < 0)
		    index = sisusb_get_free_outbuf(sisusb);

		if (index < 0){
			printk(KERN_INFO"WHS: fault at: %d\n", __LINE__);
		    return -EIO;
		}

		buffer = sisusb->obuf[index];

		bmTime = jiffies;

		if (fromuser) 
		{

		    if (copy_from_user(buffer, userbuffer, passsize)){
			printk(KERN_INFO"WHS: fault at: %d\n", __LINE__);
			return -EFAULT;
		    }

		    userbuffer += passsize;

		} else if (fromkern) {
			memcpy(buffer, kernbuffer, passsize);
			kernbuffer += passsize;
		}

		emTime = jiffies;
		//printk(KERN_INFO"triggerusb: WHS: memcpy time: %d\n", emTime - bmTime);

		retry = 5;
		while (thispass) 
		{
			if (!sisusb->sisusb_dev){
				printk(KERN_INFO"WHS: fault at: %d\n", __LINE__);
			    return -ENODEV;
			}
			result = sisusb_bulkout_msg(sisusb,
						index,
						pipe,
						buffer,
						thispass,
						&transferred_len,
						//async ? 0 : 5 * HZ,
						async ? 0 : gTimeOut,
						tflags,
						sisusb->transfer_dma_out[index]);

			if (result == -ETIMEDOUT) 
			{
			    /* Will not happen if async */
			    if (!retry--){
				printk(KERN_INFO"WHS: fault at: %d\n", __LINE__);
				return -ETIME;
			    }
			    continue;
			} else if ((result == 0) && !async && transferred_len) {

				thispass -= transferred_len;
				if (thispass) {
					if (sisusb->transfer_dma_out) {
						/* If DMA, copy remaining
						 * to beginning of buffer
						 */
						memcpy(buffer,
						       buffer + transferred_len,
						       thispass);
					} else {
						/* If not DMA, simply increase
						 * the pointer
						 */
						buffer += transferred_len;
					}
				}

			} else
				break;
		};

		if (result)
			return result;

		(*bytes_written) += passsize;
		count            -= passsize;

		/* Force new allocation in next iteration */
		if (fromuser || fromkern)
			index = -1;

		eTime = jiffies;

		
		//printk(KERN_INFO"triggerusb: WHS: send bulk elapse time: %d\n\n", eTime - bTime);

	} while (count > 0);

	if (async) {
#ifdef SISUSB_DONTSYNC
		(*bytes_written) = len;
		/* Some URBs/buffers might be busy */
#else

		begWait = jiffies;
		sisusb_wait_all_out_complete(sisusb);			
		endWait = jiffies;
				
		(*bytes_written) = transferred_len;
		/* All URBs and all buffers are available */
#endif
	}

	endTime = jiffies;

	#ifdef tusbvga_OPT

	printk(KERN_INFO"triggerusb: WHS: send bulk elapse time: %d\n\n", endTime - begTime);

	#endif 

	return ((*bytes_written) == len) ? 0 : -EIO;
	
}

/* Receive a bulk message of variable size
 *
 * To copy the data to userspace, give pointer to "userbuffer",
 * to copy to kernel memory, give "kernbuffer". One of them
 * MUST be set. (There is no technique for letting the caller
 * read directly from the ibuf.)
 *
 */
static int sisusb_recv_bulk_msg(struct sisusb_usb_data *sisusb, 
                                int ep, 
                                int len,
		                void *kernbuffer, 
		                char __user *userbuffer, 
		                ssize_t *bytes_read,
		                unsigned int tflags)
{
	int result = 0, retry, count = len;
	int bufsize, thispass, transferred_len;
	unsigned int pipe;
	char *buffer;

	(*bytes_read) = 0;

	/* Sanity check */
	if (!sisusb || !sisusb->present || !sisusb->sisusb_dev)
		return -ENODEV;

	pipe = usb_rcvbulkpipe(sisusb->sisusb_dev, ep);
	buffer = sisusb->ibuf;
	bufsize = sisusb->ibufsize;

	retry = 5;

        #ifdef SISUSB_DONTSYNC
	if (!(sisusb_wait_all_out_complete(sisusb)))
	      return -EIO;
        #endif

	while (count > 0) 
	{

		if (!sisusb->sisusb_dev)
			return -ENODEV;

		thispass = (bufsize < count) ? bufsize : count;

		result = sisusb_bulkin_msg(sisusb,
					   pipe,
					   buffer,
					   thispass,
					   &transferred_len,
					   //5 * HZ,
					   gTimeOut,
					   tflags,
					   sisusb->transfer_dma_in);

		if (transferred_len)
		    thispass = transferred_len;

		else if (result == -ETIMEDOUT) {
			 if (!retry--)
			     return -ETIME;
			 continue;

		} else
		     return -EIO;


		if (thispass) 
		{
		    (*bytes_read) += thispass;
		    count         -= thispass;

		    if (userbuffer) 
		    {
				if (copy_to_user(userbuffer, buffer, thispass))
					return -EFAULT;
				userbuffer += thispass;
		    } else {
				memcpy(kernbuffer, buffer, thispass);
				kernbuffer += thispass;
		    }

		}

	}
	return ((*bytes_read) == len) ? 0 : -EIO;
}


//*****************************************************************************************************
int SendBlockCommand(struct sisusb_usb_data *sisusb,u32 epindex,unsigned char *pdata,u32 users,u32 len)
{
      u32           bytes_transferred;      
      int           ret = 0;
      
      if(users)
      {      	 	
          ret = sisusb_send_bulk_msg(sisusb, 
                               epindex,
                               len,
			       NULL, 
			       pdata, 
			       0, 
			       &bytes_transferred, 
			       0, 
//			       1);
//Louis Tsai, do async
			       0);          			
      } 
      else
      {                 
         ret = sisusb_send_bulk_msg(sisusb, 
                                 epindex,
                                 len,
			         pdata, 
			         NULL, 
			         0, 
			         &bytes_transferred, 
			         0, 
//			         1);
//Louis Tsai, do async
			         0);      
      }			         
      	                     	
      return(ret);
}

//*********************************************************************
int WriteRegisterByte(struct sisusb_usb_data *sisusb,u32 reg,u32 data)
{
     unsigned char     packet[10];
     u32	       j,bytes_transferred;     
     int               ret=0;
     
     
     #ifdef SISUSB_DONTSYNC
	if (!(sisusb_wait_all_out_complete(sisusb)))
	      return 1;
     #endif

     j=0;
     memset(packet,0,10);
     packet[j++]=0x02;
     packet[j++]=0x00;
     packet[j++]=0x01;
     packet[j++]=(unsigned char)((reg>>8)&0xff);
     packet[j++]=(unsigned char)(reg&0xff);
     packet[j++]=(unsigned char)data;
          
     ret = sisusb_send_bulk_msg(sisusb, 
                                3,    //end point 2.
                                j,
			        (char *)packet, 
			        NULL, 
			        0, 
			        &bytes_transferred, 
			        0, 
//			       1);
//Louis Tsai, do async
			       0);      

    return(ret);               
}


int ReadRegisterByte(struct sisusb_usb_data *sisusb, u16 reg, u8 *data)
{
	unsigned char packet[3];
	u32 j,bytes_transferred, byte_read;     
  int ret=0;
     
#ifdef SISUSB_DONTSYNC
	if (!(sisusb_wait_all_out_complete(sisusb)))
	      return 1;
#endif
	 j=0;
   memset(packet,0,3);
   packet[j++]=0x05;
   packet[j++]=reg&0xff;
   packet[j++]=0xff & (reg >> 8);
   ret = sisusb_send_bulk_msg(sisusb, 
                                3,    //end point 2.
                                j,
			        (char *)packet, 
			        NULL, 
			        0, 
			        &bytes_transferred, 
			        0, 
//			        1);
//Louis Tsai, do async
			       0);      
	
	if(ret == 0) { //read command successfully, start to get result from endpoint 0
		ret = sisusb_recv_bulk_msg(sisusb, 1, 1, (void*) data, NULL, &byte_read, 0); 
		return ret;
	}
	else
		return ret;
	
}

//always 16M in trigger plus solution
static int Trigger_Get_VRAM_Size(struct sisusb_usb_data *sisusb)
{
	printk("%s: 16M solution\n", __FUNCTION__);
	return sisusb->HW_VRAMSize;
}

static unsigned short Trigger_ComputeScaleFactor(float scale)
{  
  int i;
  int kk;
  unsigned short shiftbit=0x2000, sr;
 
  sr=0;
  kk=(int)scale;
  
  sr=(kk<<14);  
  scale-=(float)kk; 
 
  for(i=0;i<14;i++)
  {      
      scale*=2.0;
      if(scale>1){
          sr|=shiftbit;
          scale-=1.0;
      }else if(scale==1.0){
               sr|=shiftbit;
               break;
      }   
      shiftbit>>=1;     
  } 
  return(sr);
}

// ColorFormat =

static int Trigger_VGA_TX_Cmd(struct sisusb_usb_data *sisusb, u32 DestStartAddr, u16 src_width, u16 src_heigh, u16 dst_width, u16 dst_heigh, u32 HScale,
                     u32 VScale, u32 Format, u32 len)
{
	unsigned char   BlockData[30];
	u32             j,HScaleFactor,VScaleFactor;
  unsigned char   ColorInfo = 0;
  u32             bytes;
  u32		 		 			ColorFormat;
  	
 
  ColorFormat = Format;    

  if(ColorFormat == YUV444)	/// YUV444
  	{ColorInfo = 0x08;bytes=3;}	/// YUV444 enable
  if(ColorFormat == RGB32)	/// 32-bit mode
  	{ColorInfo = 0x0e;bytes=4;}	/// 32-bit enable
  if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
  	{ColorInfo=0x04;bytes=2;}	/// BGR565 enable
  if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
  	{ColorInfo = 0x8a;bytes=2;}	/// YUV422 enable
  if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
  	{ColorInfo = 0x00;bytes=1;}	/// BitMap enable, yulw set to 0
	if(ColorFormat == RGB24)
		{ColorInfo = 0x00;bytes=3;}      
	

	HScaleFactor = HScale;
	VScaleFactor = VScale;

    j=0;
    BlockData[j++]=0x11;
    //Start addr. of destination.
    BlockData[j++]=(unsigned char)(DestStartAddr&0xff);	     //low
    BlockData[j++]=(unsigned char)((DestStartAddr>>8)&0xff); //middle
    BlockData[j++]=(unsigned char)((DestStartAddr>>16)&0xff);//high
    BlockData[j++]=0;
    //H	width of source.
    BlockData[j++]=(unsigned char)(src_width&0xff);	      //low
    BlockData[j++]=(unsigned char)((src_width>>8)&0xff);   //high
    //V	height of source.
    BlockData[j++]=(unsigned char)(src_heigh&0xff);	       //low
    BlockData[j++]=(unsigned char)((src_heigh>>8)&0xff);    //high
    //H	width of destination.
    BlockData[j++]=(unsigned char)(dst_width&0xff);	       //low
    BlockData[j++]=(unsigned char)((dst_width>>8)&0xff);   //high
    //V	height of destination.
    BlockData[j++]=(unsigned char)(dst_heigh&0xff);		//low
    BlockData[j++]=(unsigned char)((dst_heigh>>8)&0xff);	//high
    //H	scale factor.
    BlockData[j++]=(unsigned char)(HScaleFactor&0xff);	      //low
    BlockData[j++]=(unsigned char)(HScaleFactor>>8)&0xff;     //high
    //V	scale factor.
    BlockData[j++]=(unsigned char)(VScaleFactor&0xff);		//low
    BlockData[j++]=(unsigned char)((VScaleFactor>>8)&0xff);	//high

    //Color format.
    BlockData[j++]=ColorInfo;

    //Data Length.
    BlockData[j++]=(unsigned char)(len&0xff);
    BlockData[j++]=(unsigned char)((len>>8)&0xff);
    BlockData[j++]=(unsigned char)((len>>16)&0xff);

    return SendBlockCommand(sisusb, 2,
                     BlockData,
                     0,           //users or kernels buffer.
                     j);    
}

static int Trigger_VGA_TX_Data(struct sisusb_usb_data *sisusb, u32 len, unsigned char *txbuf)
{

	u32 bufType;

	//#ifdef tusbvga_MT
	//bufType = 0; // WHS: kernel buf
	//#else 
	bufType = 1; // WHS: user buf
	//#endif

    if(txbuf && len > 0) {       
       return SendBlockCommand(sisusb,2,
                        txbuf,
                        bufType,      //users or kernels buffer.
                        len);
    }    		
}

static void Trigger_VGA_TX_Set_Frame0_Addr(struct sisusb_usb_data *sisusb, u32 BaseAddr)
{
	unsigned char    BlockData[30];
	int j;
	
	BlockData[0] = 0x22; 
	BlockData[1] = 4;
	j = 2;  	
  //WriteRegisterByte (DevExt, 0xFCB4, 0x11);	// set Frame 0 address
  BlockData[j++] = 0xFC;
  BlockData[j++] = 0xB4;
  BlockData[j++] = 0x11;
	//WriteRegisterByte (DevExt, 0xFCB1, (UCHAR)((BaseAddr>>2)&0xff));
	BlockData[j++] = 0xFC;
	BlockData[j++] = 0xB1;
	BlockData[j++] = (u8)((BaseAddr>>2)&0xff);
	//WriteRegisterByte (DevExt, 0xFCB2, (UCHAR)((BaseAddr>>10)&0xff));
	BlockData[j++] = 0xFC;
	BlockData[j++] = 0xB2;
	BlockData[j++] = (u8)((BaseAddr>>10)&0xff);
	//WriteRegisterByte (DevExt, 0xFCB3, (UCHAR)((BaseAddr>>18)&0xff));	
	BlockData[j++] = 0xFC;
	BlockData[j++] = 0xB3;
	BlockData[j++] = (u8)((BaseAddr>>18)&0xff);		
	SendBlockCommand(sisusb, 3,
                   BlockData,
                   0,           //users or kernels buffer.
                   j);    
}

#if 0

#define TEST_BLK_SIZE 131072
static unsigned char transferBuf[TEST_BLK_SIZE];

//**********************************************
void TransferBlockData(struct sisusb_usb_data *sisusb,
		       u32		 SrcX,
		       u32		 SrcY,
		       u32		 DestX,
		       u32		 DestY,
		       u32		 ColorFormat,
		       u32		 SrcBitsPlane, // NULL argument
		       u32		 ScrXPoint,
		       u32		 ScrYPoint,
		       u32		 ScreenLineLen,
		       u32		 HScale,
		       u32		 VScale,
		       unsigned char*	 pdata,
		       int               users,
		       u32		 len)		       
{
    unsigned char    BlockData[30];
    u32              DestStartAddr,j,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo;
    u32              status,bytes;

    if(ColorFormat == YUV444)	/// YUV444
       {ColorInfo = 0x08;bytes=3;}	/// YUV444 enable
    if(ColorFormat == RGB32)	/// 32-bit mode
       {ColorInfo = 0x0e;bytes=4;}	/// 32-bit enable
    if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
       {ColorInfo=0x04;bytes=2;}	/// BGR565 enable
    if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
       {ColorInfo = 0x8a;bytes=2;}	/// YUV422 enable
    if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
       {ColorInfo = 0x00;bytes=1;}	/// BitMap enable, yulw set to 0
	if (ColorFormat == RGB24)
	    {ColorInfo = 0x00;bytes=3;}      

    if(HScale == UP_2TIMES)
       HScaleFactor=0x2000;
    else if(HScale == UP_4TIMES)
	    HScaleFactor=0x1000;
	 else if(HScale	== UP_QUADTIME)
		 HScaleFactor=0x3000;
	      else
		 HScaleFactor=0x4000;

    if(VScale == UP_2TIMES)
       VScaleFactor=0x2000;
    else if(VScale == UP_4TIMES)
	    VScaleFactor=0x1000;
	 else if(VScale	== UP_QUADTIME)
		 VScaleFactor=0x3000;
	      else
		 VScaleFactor=0x4000;
   
    DestStartAddr = (ScrYPoint*ScreenLineLen) + (ScrXPoint*3);
    if(len==0) len=SrcX*bytes*SrcY;
    
    #if 0
    //yulw
    if (!sisusb->bOnFrame0Active)
    	DestStartAddr += 4*1024*1024;
    #endif    	

    j=0;
    BlockData[j++] = 0x11;
    //Start addr. of destination.
    BlockData[j++]=(unsigned char)(DestStartAddr&0xff);	     //low
    BlockData[j++]=(unsigned char)((DestStartAddr>>8)&0xff); //middle
    BlockData[j++]=(unsigned char)((DestStartAddr>>16)&0xff);//high
    BlockData[j++]=0;
    //H	width of source.
    BlockData[j++]=(unsigned char)(SrcX&0xff);	      //low
    BlockData[j++]=(unsigned char)((SrcX>>8)&0xff);   //high
    //V	height of source.
    BlockData[j++]=(unsigned char)(SrcY&0xff);	       //low
    BlockData[j++]=(unsigned char)((SrcY>>8)&0xff);    //high
    //H	width of destination.
    BlockData[j++]=(unsigned char)(DestX&0xff);	       //low
    BlockData[j++]=(unsigned char)((DestX>>8)&0xff);   //high
    //V	height of destination.
    BlockData[j++]=(unsigned char)(DestY&0xff);		//low
    BlockData[j++]=(unsigned char)((DestY>>8)&0xff);	//high
    //H	scale factor.
    BlockData[j++]=(unsigned char)(HScaleFactor&0xff);	      //low
    BlockData[j++]=(unsigned char)(HScaleFactor>>8)&0xff;     //high
    //V	scale factor.
    BlockData[j++]=(unsigned char)(VScaleFactor&0xff);		//low
    BlockData[j++]=(unsigned char)((VScaleFactor>>8)&0xff);	//high

    //Color format.
    BlockData[j++]=ColorInfo;

    //Data Length.
    BlockData[j++]=(unsigned char)(len&0xff);
    BlockData[j++]=(unsigned char)((len>>8)&0xff);
    BlockData[j++]=(unsigned char)((len>>16)&0xff);

    SendBlockCommand(sisusb,2,
                     BlockData,
                     0,           //users or kernels buffer.
                     j);    
    if(pdata)
    {   

		u32 txlen;    

		while(len){
			
	   		txlen = (len > TEST_BLK_SIZE)? (TEST_BLK_SIZE):len;
			memcpy(transferBuf, pdata += txlen, txlen);
	   		SendBlockCommand
			(
				sisusb, 2, 
				transferBuf,
 	           0,      //users or kernels buffer.
              txlen
            );	
        	len -= txlen;
    	}
						
    }    	
    else
    {

		#if 1
		
        //unsigned char *buf;
        u32 txlen;    
                        	
    	//if(!(buf = kmalloc(64*1024, GFP_KERNEL))){
		//	printk(KERN_INFO"triggerusb: Failed to allocate memory .\n");
		//	return ;
		//}			
		//memset(buf, 0, 64*1024);
		
		while(len){
	   		txlen = (len>64*1024)? (64*1024):len;
			memcpy(pdata += txlen, transferBuf, txlen);
	   		SendBlockCommand
			(
				sisusb,2, transferBuf,
                0,      //users or kernels buffer.
                txlen
            );	
        	len-=txlen;
    	}
    	//kfree(buf);

		#endif
    	
	}    	    
	
}

#endif

#if 1

#define TEST_BLK_SIZE 1024*1024 // WHS
static unsigned char transferBuf[TEST_BLK_SIZE];

//**********************************************

int TransferBlock(struct sisusb_usb_data *sisusb,
		       u32		 SrcX,
		       u32		 SrcY,
		       u32		 DestX,
		       u32		 DestY,
		       u32		 ColorFormat,
		       u32		 SrcBitsPlane,
		       u32		 ScrXPoint,
		       u32		 ScrYPoint,
		       u32		 ScreenLineLen,
		       u32		 HScale,
		       u32		 VScale,
		       unsigned char*	 pdata,
		       int               users,
		       u32		 len)		        
{
    unsigned char    BlockData[30];
    u32              DestStartAddr,j,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo = 0;
    u32              bytes = 0;
	int ret = 0;
	unsigned long begTime, endTime;

	#ifdef tusbvga_FLIP
	
	unsigned long screenSize;
	
	//screenSize = sisusb->hres*sisusb->vres*3;
	if(sisusb->flipable){
		DestStartAddr = 1024*1024*4*sisusb->flipBuf; // WHS: flips
	}
	Trigger_VGA_TX_Set_Frame0_Addr
	(sisusb, 1024*1024*4*(1 - sisusb->flipBuf));
	
	#endif

	//WriteRegisterByte(sisusb, 0xFCB0, 0x00);

	//printk(KERN_INFO"triggerusb: %s\n", __FUNCTION__);

    if(ColorFormat == YUV444)	/// YUV444
       {ColorInfo = 0x08;bytes=3;}	/// YUV444 enable
    if(ColorFormat == RGB32)	/// 32-bit mode
       {ColorInfo = 0x0e;bytes=4;}	/// 32-bit enable
    if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
       {ColorInfo=0x04;bytes=2;}	/// BGR565 enable
    if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
       {ColorInfo = 0x8a;bytes=2;}	/// YUV422 enable
    if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
       {ColorInfo = 0x00;bytes=1;}	/// BitMap enable, yulw set to 0
	if (ColorFormat == RGB24)
	    {ColorInfo = 0x00;bytes=3;}      

    if(HScale == UP_2TIMES)
       HScaleFactor=0x2000;
    else if(HScale == UP_4TIMES)
	    HScaleFactor=0x1000;
	 else if(HScale	== UP_QUADTIME)
		 HScaleFactor=0x3000;
	      else
		 HScaleFactor=0x4000;

	HScaleFactor = HScale;

    if(VScale == UP_2TIMES)
       VScaleFactor=0x2000;
    else if(VScale == UP_4TIMES)
	    VScaleFactor=0x1000;
	 else if(VScale	== UP_QUADTIME)
		 VScaleFactor=0x3000;
	      else
		 VScaleFactor=0x4000;

	VScaleFactor = VScale;	

	#ifdef tusbvga_FLIP
   
    DestStartAddr += (ScrYPoint*ScreenLineLen)+(ScrXPoint*3);

	#else

	DestStartAddr = (ScrYPoint*ScreenLineLen)+(ScrXPoint*3);

	#endif
	//printk(KERN_INFO"WHS: SrcX: %d SrcY: %d.\n", SrcX, SrcY);
	
    if(len==0)
       len=SrcX*bytes*SrcY;
    
    #if 0
    //yulw
    if (!sisusb->bOnFrame0Active)
    	DestStartAddr += 4*1024*1024;
    #endif    	

    j=0;
    BlockData[j++]=0x11;
    //Start addr. of destination.
    BlockData[j++]=(unsigned char)(DestStartAddr&0xff);	     //low
    BlockData[j++]=(unsigned char)((DestStartAddr>>8)&0xff); //middle
    BlockData[j++]=(unsigned char)((DestStartAddr>>16)&0xff);//high
    BlockData[j++]=0;
    //H	width of source.
    BlockData[j++]=(unsigned char)(SrcX&0xff);	      //low
    BlockData[j++]=(unsigned char)((SrcX>>8)&0xff);   //high
    //V	height of source.
    BlockData[j++]=(unsigned char)(SrcY&0xff);	       //low
    BlockData[j++]=(unsigned char)((SrcY>>8)&0xff);    //high
    //H	width of destination.
    BlockData[j++]=(unsigned char)(DestX&0xff);	       //low
    BlockData[j++]=(unsigned char)((DestX>>8)&0xff);   //high
    //V	height of destination.
    BlockData[j++]=(unsigned char)(DestY&0xff);		//low
    BlockData[j++]=(unsigned char)((DestY>>8)&0xff);	//high
    //H	scale factor.
    BlockData[j++]=(unsigned char)(HScaleFactor&0xff);	      //low
    BlockData[j++]=(unsigned char)(HScaleFactor>>8)&0xff;     //high
    //V	scale factor.
    BlockData[j++]=(unsigned char)(VScaleFactor&0xff);		//low
    BlockData[j++]=(unsigned char)((VScaleFactor>>8)&0xff);	//high

    //Color format.
    BlockData[j++]=ColorInfo;

    //Data Length.
    BlockData[j++]=(unsigned char)(len&0xff);
    BlockData[j++]=(unsigned char)((len>>8)&0xff);
    BlockData[j++]=(unsigned char)((len>>16)&0xff);

    ret = SendBlockCommand(sisusb,2,
                     BlockData,
                     0,           //users or kernels buffer.
                     j);    

	if(ret < 0){

		printk(KERN_INFO"triggerusb: WHS: VGA command phase error\n");
		return ret;
			
	}

	begTime = jiffies;
	
    if(pdata)
    //if(users == 1)
    {       
       u32 txlen;    

		while(len){ 
			
	   		txlen = (len > TEST_BLK_SIZE)? (TEST_BLK_SIZE):len;
						
			//memcpy(transferBuf, pdata, txlen);
			//printk(KERN_INFO"triggerusb: WHS: memcpy_fromio\n");
			//memcpy_fromio(transferBuf, pdata, txlen);
	   		ret = SendBlockCommand
			(
				sisusb, 2,  
				//transferBuf,
				pdata,
 	           1,      //users or kernels buffer.
 	           //0,
              txlen
            );	

			if(ret < 0){

				printk(KERN_INFO"triggerusb: WHS: VGA data phase error\n");
				return ret;
			
			}
			
			pdata += txlen;
        	len -= txlen;
    	}
    }    	
    else // WHS: for cleaning screen: 20090515: use to implement by memcpy_fromio
    {

		u32 txlen, line, bytesPerLine, transferSize;    
		unsigned char *src, *dst;
		u8 f1;
		src = pdata;
		
		while(len){ 

			printk(KERN_INFO"triggerusb: WHS: current len: %d\n", len);

			bytesPerLine = bytes*SrcX;			
			transferSize = TEST_BLK_SIZE/bytesPerLine*bytesPerLine;
			txlen = (len > transferSize)? transferSize: len;		
			line = txlen/bytesPerLine;
			dst = transferBuf;
			
			for(f1=0; f1<line; f1++){

				memcpy_fromio(dst, src, bytesPerLine);
				//memcpy(dst, src, bytesPerLine);
				//copy_from_user(dst, (void __user *)src, bytesPerLine);
				src += 4096;
				dst += bytesPerLine;

			}

			#if 0
			
	   		//txlen = (len > TEST_BLK_SIZE)? (TEST_BLK_SIZE):len;						
			//memcpy(transferBuf, pdata, txlen);
	   		ret = SendBlockCommand
			(
				sisusb, 2,  
				transferBuf,
				//pdata,
 	           //1,      //users or kernels buffer.
 	           0,
              txlen
            );	

			if(ret < 0){

				printk(KERN_INFO"triggerusb: WHS: VGA data phase error\n");
				return ret;
			
			}

			#endif
			
			//pdata += txlen;
        	len -= txlen;
			
    	}
		
		#if 0
		
        unsigned char *buf;
        u32           txlen;    
		//u32 begin_idx = 0;
		//u32 k, x;
                
        memcpy_fromio(transferBuf, pdata, txlen); 
    	if (!(buf = kmalloc(64*1024, GFP_KERNEL))) {
				printk(KERN_INFO
					"triggerusb: Failed to allocate memory .\n");
				return ;
		}
		memset(buf, 0, 64*1024);
		while(len) {
	   		txlen = (len>64*1024)? (64*1024):len;
					
	   		SendBlockCommand(sisusb,2,
                            	buf,
                            0,      //users or kernels buffer.
                            txlen);	
        	len-=txlen;
    	}
	   	kfree(buf);

		#endif
		
	}   

	endTime = jiffies;

	//printk("triggerusb: transferBlock time: %d\n", endTime - begTime);

	#ifdef tusbvga_FLIP

	sisusb->flipBuf ^= 1;

	#endif

	//WriteRegisterByte(sisusb, 0xFCB0, 0x01);

	return ret;
	
}


void TransferBlockData(struct sisusb_usb_data *sisusb,
		       u32		 SrcX,
		       u32		 SrcY,
		       u32		 DestX,
		       u32		 DestY,
		       u32		 ColorFormat,
		       u32		 SrcBitsPlane,
		       u32		 ScrXPoint,
		       u32		 ScrYPoint,
		       u32		 ScreenLineLen,
		       u32		 HScale,
		       u32		 VScale,
		       unsigned char*	 pdata,
		       int               users,
		       u32		 len)		        
{
    unsigned char    BlockData[30];
    u32              DestStartAddr,j,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo = 0;
    u32              bytes = 0;
	int ret;

	//if(sisusb->flipable) DestStartAddr = 1024*1024*4*sisusb->flipBuf; // WHS: flips
	//Trigger_VGA_TX_Set_Frame0_Addr(sisusb, 1024*1024*4*(1 - sisusb->flipBuf));
		
    if(ColorFormat == YUV444)	/// YUV444
       {ColorInfo = 0x08;bytes=3;}	/// YUV444 enable
    if(ColorFormat == RGB32)	/// 32-bit mode
       {ColorInfo = 0x0e;bytes=4;}	/// 32-bit enable
    if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
       {ColorInfo=0x04;bytes=2;}	/// BGR565 enable
    if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
       {ColorInfo = 0x8a;bytes=2;}	/// YUV422 enable
    if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
       {ColorInfo = 0x00;bytes=1;}	/// BitMap enable, yulw set to 0
	if (ColorFormat == RGB24)
	    {ColorInfo = 0x00;bytes=3;}      

    if(HScale == UP_2TIMES)
       HScaleFactor=0x2000;
    else if(HScale == UP_4TIMES)
	    HScaleFactor=0x1000;
	 else if(HScale	== UP_QUADTIME)
		 HScaleFactor=0x3000;
	      else
		 HScaleFactor=0x4000;

    if(VScale == UP_2TIMES)
       VScaleFactor=0x2000;
    else if(VScale == UP_4TIMES)
	    VScaleFactor=0x1000;
	 else if(VScale	== UP_QUADTIME)
		 VScaleFactor=0x3000;
	      else
		 VScaleFactor=0x4000;

	//DestStartAddr = 0;
	//ScreenSize = sisusb->hres*sisusb->vres*3;
		   
    DestStartAddr = ((ScrYPoint*ScreenLineLen)+(ScrXPoint*3));
	//printk(KERN_INFO"WHS: SrcX: %d SrcY: %d.\n", SrcX, SrcY);
	
    if(len==0)
       len=SrcX*bytes*SrcY;
    
    #if 0
    //yulw
    if (!sisusb->bOnFrame0Active)
    	DestStartAddr += 4*1024*1024;
    #endif    	

    j=0;
    BlockData[j++]=0x11;
    //Start addr. of destination.
    BlockData[j++]=(unsigned char)(DestStartAddr&0xff);	     //low
    BlockData[j++]=(unsigned char)((DestStartAddr>>8)&0xff); //middle
    BlockData[j++]=(unsigned char)((DestStartAddr>>16)&0xff);//high
    BlockData[j++]=0;
    //H	width of source.
    BlockData[j++]=(unsigned char)(SrcX&0xff);	      //low
    BlockData[j++]=(unsigned char)((SrcX>>8)&0xff);   //high
    //V	height of source.
    BlockData[j++]=(unsigned char)(SrcY&0xff);	       //low
    BlockData[j++]=(unsigned char)((SrcY>>8)&0xff);    //high
    //H	width of destination.
    BlockData[j++]=(unsigned char)(DestX&0xff);	       //low
    BlockData[j++]=(unsigned char)((DestX>>8)&0xff);   //high
    //V	height of destination.
    BlockData[j++]=(unsigned char)(DestY&0xff);		//low
    BlockData[j++]=(unsigned char)((DestY>>8)&0xff);	//high
    //H	scale factor.
    BlockData[j++]=(unsigned char)(HScaleFactor&0xff);	      //low
    BlockData[j++]=(unsigned char)(HScaleFactor>>8)&0xff;     //high
    //V	scale factor.
    BlockData[j++]=(unsigned char)(VScaleFactor&0xff);		//low
    BlockData[j++]=(unsigned char)((VScaleFactor>>8)&0xff);	//high

    //Color format.
    BlockData[j++]=ColorInfo;

    //Data Length.
    BlockData[j++]=(unsigned char)(len&0xff);
    BlockData[j++]=(unsigned char)((len>>8)&0xff);
    BlockData[j++]=(unsigned char)((len>>16)&0xff);

    ret = SendBlockCommand(sisusb,2,
                     BlockData,
                     0,           //users or kernels buffer.
                     j);    

	if(ret < 0){

		printk(KERN_INFO"WHS: %s: VGA command phase error\n", __FUNCTION__);
		return ;
			
	}
	
    if(pdata)
    {       
       u32 txlen;    

		while(len){ 
			
	   		txlen = (len > TEST_BLK_SIZE)? (TEST_BLK_SIZE):len;
			//memcpy(transferBuf, pdata, txlen);
	   		ret = SendBlockCommand
			(
				sisusb, 2,  
				//transferBuf,
				pdata,
 	           1,      //users or kernels buffer.
              txlen
            );	

			if(ret < 0){

				printk(KERN_INFO"WHS: %s: VGA data phase error\n", __FUNCTION__);
				return ;
			
			}
			
			pdata += txlen;
        	len -= txlen;
    	}
    }    	
    else // WHS: for cleaning screen
    {

		#if 0

		begin_idx = 0;
    while(len) {
    	txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   
	   	for(k=0;k<begin_idx;k++) buf[k] = 0;
	   	for(x=0;x<(txlen/(sisusb->bpp/8));x++) {
	   		buf[(x*(sisusb->bpp/8))+begin_idx] = b;
	   		buf[(x*(sisusb->bpp/8))+1+begin_idx] = g;
	   		buf[(x*(sisusb->bpp/8))+2+begin_idx] = r;
	   	}
	   	//add zero pading DWORD alignment
	   	if((txlen % (sisusb->bpp/8)) != 0) {
	   		for(k=0;k<(txlen % (sisusb->bpp/8));k++) 
	   			buf[(x*(sisusb->bpp/8))+k] = 0x0;	
	   		
	   		begin_idx = (begin_idx + (sisusb->bpp/8) - (txlen % (sisusb->bpp/8))) % (sisusb->bpp/8);
	   	}else
	   		begin_idx = begin_idx;
	   				
	   	Trigger_VGA_TX_Data(sisusb, txlen, buf);
	   	len-=txlen;                               
   	}	   			
    kfree(buf);   	    

		#endif

        unsigned char *buf;
        u32           txlen;    
		//u32 begin_idx = 0;
		//u32 k, x;
                
        	
    	if (!(buf = kmalloc(64*1024, GFP_KERNEL))) {
				printk(KERN_INFO
					"triggerusb: Failed to allocate memory .\n");
				return ;
			}
			memset(buf, 0, 64*1024);
			while(len) {
	   		txlen = (len>64*1024)? (64*1024):len;
					
	   		ret = SendBlockCommand(sisusb,2,
                            	buf,
                            0,      //users or kernels buffer.
                            txlen);	

			if(ret < 0){

				printk(KERN_INFO"WHS: %s: VGA data phase error\n", __FUNCTION__);
				return ;
			
			}
						
        len-=txlen;
    	}
	   kfree(buf);
		}   

	#ifdef tusbvga_FLIP

	DestStartAddr = 4*1024*1024;
	len=SrcX*bytes*SrcY;

	j=0;
    BlockData[j++]=0x11;
    //Start addr. of destination.
    BlockData[j++]=(unsigned char)(DestStartAddr&0xff);	     //low
    BlockData[j++]=(unsigned char)((DestStartAddr>>8)&0xff); //middle
    BlockData[j++]=(unsigned char)((DestStartAddr>>16)&0xff);//high
    BlockData[j++]=0;
    //H	width of source.
    BlockData[j++]=(unsigned char)(SrcX&0xff);	      //low
    BlockData[j++]=(unsigned char)((SrcX>>8)&0xff);   //high
    //V	height of source.
    BlockData[j++]=(unsigned char)(SrcY&0xff);	       //low
    BlockData[j++]=(unsigned char)((SrcY>>8)&0xff);    //high
    //H	width of destination.
    BlockData[j++]=(unsigned char)(DestX&0xff);	       //low
    BlockData[j++]=(unsigned char)((DestX>>8)&0xff);   //high
    //V	height of destination.
    BlockData[j++]=(unsigned char)(DestY&0xff);		//low
    BlockData[j++]=(unsigned char)((DestY>>8)&0xff);	//high
    //H	scale factor.
    BlockData[j++]=(unsigned char)(HScaleFactor&0xff);	      //low
    BlockData[j++]=(unsigned char)(HScaleFactor>>8)&0xff;     //high
    //V	scale factor.
    BlockData[j++]=(unsigned char)(VScaleFactor&0xff);		//low
    BlockData[j++]=(unsigned char)((VScaleFactor>>8)&0xff);	//high

    //Color format.
    BlockData[j++]=ColorInfo;

    //Data Length.
    BlockData[j++]=(unsigned char)(len&0xff);
    BlockData[j++]=(unsigned char)((len>>8)&0xff);
    BlockData[j++]=(unsigned char)((len>>16)&0xff);

    ret = SendBlockCommand(sisusb,2,
                     BlockData,
                     0,           //users or kernels buffer.
                     j);    

	if(ret < 0){

		printk(KERN_INFO"WHS: %s: VGA command phase error\n", __FUNCTION__);
		return ;
			
	}
	
    if(pdata)
    {       
       u32 txlen;    

		while(len){ 
			
	   		txlen = (len > TEST_BLK_SIZE)? (TEST_BLK_SIZE):len;
			//memcpy(transferBuf, pdata, txlen);
	   		ret = SendBlockCommand
			(
				sisusb, 2,  
				//transferBuf,
				pdata,
 	           1,      //users or kernels buffer.
              txlen
            );	

			if(ret < 0){

				printk(KERN_INFO"WHS: %s: VGA data phase error\n", __FUNCTION__);
				return ;
			
			}
			
			pdata += txlen;
        	len -= txlen;
    	}
    }    	
    else // WHS: for cleaning screen
    {

		#if 0

		begin_idx = 0;
    while(len) {
    	txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   
	   	for(k=0;k<begin_idx;k++) buf[k] = 0;
	   	for(x=0;x<(txlen/(sisusb->bpp/8));x++) {
	   		buf[(x*(sisusb->bpp/8))+begin_idx] = b;
	   		buf[(x*(sisusb->bpp/8))+1+begin_idx] = g;
	   		buf[(x*(sisusb->bpp/8))+2+begin_idx] = r;
	   	}
	   	//add zero pading DWORD alignment
	   	if((txlen % (sisusb->bpp/8)) != 0) {
	   		for(k=0;k<(txlen % (sisusb->bpp/8));k++) 
	   			buf[(x*(sisusb->bpp/8))+k] = 0x0;	
	   		
	   		begin_idx = (begin_idx + (sisusb->bpp/8) - (txlen % (sisusb->bpp/8))) % (sisusb->bpp/8);
	   	}else
	   		begin_idx = begin_idx;
	   				
	   	Trigger_VGA_TX_Data(sisusb, txlen, buf);
	   	len-=txlen;                               
   	}	   			
    kfree(buf);   	    

		#endif

        unsigned char *buf;
        u32           txlen;    
		//u32 begin_idx = 0;
		//u32 k, x;
                
        	
    	if (!(buf = kmalloc(64*1024, GFP_KERNEL))) {
				printk(KERN_INFO
					"triggerusb: Failed to allocate memory .\n");
				return ;
			}
			memset(buf, 0, 64*1024);
			while(len) {
	   		txlen = (len>64*1024)? (64*1024):len;
					
	   		ret = SendBlockCommand(sisusb,2,
                            	buf,
                            0,      //users or kernels buffer.
                            txlen);	

			if(ret < 0){

				printk(KERN_INFO"WHS: %s: VGA data phase error\n", __FUNCTION__);
				return ;
			
			}
						
        len-=txlen;
    	}
	   kfree(buf);
		}   

	
	sisusb->flipBuf ^= 1;

	#endif
	
}

#endif

void FillBlockData(struct sisusb_usb_data *sisusb,
		       u32		 SrcX,
		       u32		 SrcY,
		       u32		 DestX,
		       u32		 DestY,
		       u32		 ColorFormat,
		       u32		 SrcBitsPlane,
		       u32		 ScrXPoint,
		       u32		 ScrYPoint,
		       u32		 ScreenLineLen,
		       u32		 HScale,
		       u32		 VScale,
		       unsigned char*	 pdata,
		       int               users,
		       u32		 len,
				u32 Color
)		       
{
    unsigned char    BlockData[30];
    u32              DestStartAddr,j,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo = 0;
    u32              bytes = 0;

    if(ColorFormat == YUV444)	/// YUV444
       {ColorInfo = 0x08;bytes=3;}	/// YUV444 enable
    if(ColorFormat == RGB32)	/// 32-bit mode
       {ColorInfo = 0x0e;bytes=4;}	/// 32-bit enable
    if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
       {ColorInfo=0x04;bytes=2;}	/// BGR565 enable
    if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
       {ColorInfo = 0x8a;bytes=2;}	/// YUV422 enable
    if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
       {ColorInfo = 0x00;bytes=1;}	/// BitMap enable, yulw set to 0
	if (ColorFormat == RGB24)
	    {ColorInfo = 0x00;bytes=3;}      

    if(HScale == UP_2TIMES)
       HScaleFactor=0x2000;
    else if(HScale == UP_4TIMES)
	    HScaleFactor=0x1000;
	 else if(HScale	== UP_QUADTIME)
		 HScaleFactor=0x3000;
	      else
		 HScaleFactor=0x4000;

    if(VScale == UP_2TIMES)
       VScaleFactor=0x2000;
    else if(VScale == UP_4TIMES)
	    VScaleFactor=0x1000;
	 else if(VScale	== UP_QUADTIME)
		 VScaleFactor=0x3000;
	      else
		 VScaleFactor=0x4000;
   
    DestStartAddr=(ScrYPoint*ScreenLineLen)+(ScrXPoint*3);

	#ifdef tusbvga_DBG
	printk(KERN_INFO"WHS: SrcX: %d SrcY: %d.\n", SrcX, SrcY);
	#endif
	
    if(len==0)
       len=SrcX*bytes*SrcY;
    
    #if 0
    //yulw
    if (!sisusb->bOnFrame0Active)
    	DestStartAddr += 4*1024*1024;
    #endif    	

    j=0;
    BlockData[j++]=0x11;
    //Start addr. of destination.
    BlockData[j++]=(unsigned char)(DestStartAddr&0xff);	     //low
    BlockData[j++]=(unsigned char)((DestStartAddr>>8)&0xff); //middle
    BlockData[j++]=(unsigned char)((DestStartAddr>>16)&0xff);//high
    BlockData[j++]=0;
    //H	width of source.
    BlockData[j++]=(unsigned char)(SrcX&0xff);	      //low
    BlockData[j++]=(unsigned char)((SrcX>>8)&0xff);   //high
    //V	height of source.
    BlockData[j++]=(unsigned char)(SrcY&0xff);	       //low
    BlockData[j++]=(unsigned char)((SrcY>>8)&0xff);    //high
    //H	width of destination.
    BlockData[j++]=(unsigned char)(DestX&0xff);	       //low
    BlockData[j++]=(unsigned char)((DestX>>8)&0xff);   //high
    //V	height of destination.
    BlockData[j++]=(unsigned char)(DestY&0xff);		//low
    BlockData[j++]=(unsigned char)((DestY>>8)&0xff);	//high
    //H	scale factor.
    BlockData[j++]=(unsigned char)(HScaleFactor&0xff);	      //low
    BlockData[j++]=(unsigned char)(HScaleFactor>>8)&0xff;     //high
    //V	scale factor.
    BlockData[j++]=(unsigned char)(VScaleFactor&0xff);		//low
    BlockData[j++]=(unsigned char)((VScaleFactor>>8)&0xff);	//high

    //Color format.
    BlockData[j++]=ColorInfo;

    //Data Length.
    BlockData[j++]=(unsigned char)(len&0xff);
    BlockData[j++]=(unsigned char)((len>>8)&0xff);
    BlockData[j++]=(unsigned char)((len>>16)&0xff);

    SendBlockCommand(sisusb,2,
                     BlockData,
                     0,           //users or kernels buffer.
                     j);    
    if(pdata)
    {       
       
		u32 txlen;    

		while(len){
			
	   		txlen = (len > TEST_BLK_SIZE)? (TEST_BLK_SIZE):len;
			memcpy(transferBuf, pdata, txlen);
	   		SendBlockCommand
			(
				sisusb, 2, 
				transferBuf,
 	           0,      //users or kernels buffer.
              txlen
            );	
			pdata += txlen;
        	len -= txlen;
    	}

    }    	
    else // WHS: for cleaning screen
    {

		u32 txlen, begin_idx = 0, r, g, b, k, x;    

		b = Color & 0x000000ff;
		g = (Color & 0x0000ff00) >> 8;
		r = (Color & 0x00ff0000) >> 16;

		#ifdef tusbvga_DBG

		printk("color: %x, r: %d, g: %d, b: %d\n", Color, r, g, b);   

		#endif

		while(len){
			
	   		txlen = (len > TEST_BLK_SIZE)? (TEST_BLK_SIZE):len;

			for(k=0;k<begin_idx;k++) transferBuf[k] = 0;
		   	for(x=0;x<(txlen/(sisusb->bpp/8));x++){
	   			transferBuf[(x*(sisusb->bpp/8))+begin_idx] = b;
	   			transferBuf[(x*(sisusb->bpp/8))+1+begin_idx] = g;
	   			transferBuf[(x*(sisusb->bpp/8))+2+begin_idx] = r;
	   		}

			if((txlen % (sisusb->bpp/8)) != 0){
		   		for(k=0;k<(txlen % (sisusb->bpp/8));k++) transferBuf[(x*(sisusb->bpp/8))+k] = 0x0;		   		
		   		begin_idx = (begin_idx + (sisusb->bpp/8) - (txlen % (sisusb->bpp/8))) % (sisusb->bpp/8);

		   	}
			else begin_idx = begin_idx;

			//memcpy(transferBuf, pdata, txlen);
	   		SendBlockCommand
			(
				sisusb, 2, 
				transferBuf,
 	           0,      //users or kernels buffer.
              txlen
            );	
			//pdata += txlen;
        	len -= txlen;
    	}
	}    	    
}


void TransferBlockData1(struct sisusb_usb_data *sisusb,
		       u32		 SrcX,
		       u32		 SrcY,
		       u32		 DestX,
		       u32		 DestY,
		       u32		 ColorFormat,
		       u32		 SrcBitsPlane,
		       u32		 ScrXPoint,
		       u32		 ScrYPoint,
		       u32		 ScreenLineLen,
		       u32		 HScale,
		       u32		 VScale,
		       unsigned char*	 pdata,
		       int               users,
		       u32		 len)		       
{
    unsigned char    BlockData[30];
    u32              DestStartAddr,j,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo = 0;
    u32              bytes = 0;

    if(ColorFormat == YUV444)	/// YUV444
       {ColorInfo = 0x08;bytes=3;}	/// YUV444 enable
    if(ColorFormat == RGB32)	/// 32-bit mode
       {ColorInfo = 0x0e;bytes=4;}	/// 32-bit enable
    if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
       {ColorInfo=0x04;bytes=2;}	/// BGR565 enable
    if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
       {ColorInfo = 0x8a;bytes=2;}	/// YUV422 enable
    if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
       {ColorInfo = 0x00;bytes=1;}	/// BitMap enable, yulw set to 0
	if (ColorFormat == RGB24)
	    {ColorInfo = 0x00;bytes=3;}      

    if(HScale == UP_2TIMES)
       HScaleFactor=0x2000;
    else if(HScale == UP_4TIMES)
	    HScaleFactor=0x1000;
	 else if(HScale	== UP_QUADTIME)
		 HScaleFactor=0x3000;
	      else
		 HScaleFactor=0x4000;

    if(VScale == UP_2TIMES)
       VScaleFactor=0x2000;
    else if(VScale == UP_4TIMES)
	    VScaleFactor=0x1000;
	 else if(VScale	== UP_QUADTIME)
		 VScaleFactor=0x3000;
	      else
		 VScaleFactor=0x4000;
   
    DestStartAddr=(ScrYPoint*ScreenLineLen)+(ScrXPoint*3);
    if(len==0)
       len=SrcX*bytes*SrcY;
    
    #if 0
    //yulw
    if (!sisusb->bOnFrame0Active)
    	DestStartAddr += 4*1024*1024;
    #endif    	

    j=0;
    BlockData[j++]=0x11;
    //Start addr. of destination.
    BlockData[j++]=(unsigned char)(DestStartAddr&0xff);	     //low
    BlockData[j++]=(unsigned char)((DestStartAddr>>8)&0xff); //middle
    BlockData[j++]=(unsigned char)((DestStartAddr>>16)&0xff);//high
    BlockData[j++]=0;
    //H	width of source.
    BlockData[j++]=(unsigned char)(SrcX&0xff);	      //low
    BlockData[j++]=(unsigned char)((SrcX>>8)&0xff);   //high
    //V	height of source.
    BlockData[j++]=(unsigned char)(SrcY&0xff);	       //low
    BlockData[j++]=(unsigned char)((SrcY>>8)&0xff);    //high
    //H	width of destination.
    BlockData[j++]=(unsigned char)(DestX&0xff);	       //low
    BlockData[j++]=(unsigned char)((DestX>>8)&0xff);   //high
    //V	height of destination.
    BlockData[j++]=(unsigned char)(DestY&0xff);		//low
    BlockData[j++]=(unsigned char)((DestY>>8)&0xff);	//high
    //H	scale factor.
    BlockData[j++]=(unsigned char)(HScaleFactor&0xff);	      //low
    BlockData[j++]=(unsigned char)(HScaleFactor>>8)&0xff;     //high
    //V	scale factor.
    BlockData[j++]=(unsigned char)(VScaleFactor&0xff);		//low
    BlockData[j++]=(unsigned char)((VScaleFactor>>8)&0xff);	//high

    //Color format.
    BlockData[j++]=ColorInfo;

    //Data Length.
    BlockData[j++]=(unsigned char)(len&0xff);
    BlockData[j++]=(unsigned char)((len>>8)&0xff);
    BlockData[j++]=(unsigned char)((len>>16)&0xff);

    SendBlockCommand(sisusb,2,
                     BlockData,
                     0,           //users or kernels buffer.
                     j);    
    if(pdata)
    {       
       u32 txlen;    

		while(len){
			
	   		txlen = (len > TEST_BLK_SIZE)? (TEST_BLK_SIZE):len;
			memcpy(transferBuf, pdata += txlen, txlen);
	   		SendBlockCommand
			(
				sisusb, 2, 
				transferBuf,
 	           0,      //users or kernels buffer.
              txlen
            );	
        	len -= txlen;
    	}
    }    	
    else
    {

		#if 0

		begin_idx = 0;
    while(len) {
    	txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   
	   	for(k=0;k<begin_idx;k++) buf[k] = 0;
	   	for(x=0;x<(txlen/(sisusb->bpp/8));x++) {
	   		buf[(x*(sisusb->bpp/8))+begin_idx] = b;
	   		buf[(x*(sisusb->bpp/8))+1+begin_idx] = g;
	   		buf[(x*(sisusb->bpp/8))+2+begin_idx] = r;
	   	}
	   	//add zero pading DWORD alignment
	   	if((txlen % (sisusb->bpp/8)) != 0) {
	   		for(k=0;k<(txlen % (sisusb->bpp/8));k++) 
	   			buf[(x*(sisusb->bpp/8))+k] = 0x0;	
	   		
	   		begin_idx = (begin_idx + (sisusb->bpp/8) - (txlen % (sisusb->bpp/8))) % (sisusb->bpp/8);
	   	}else
	   		begin_idx = begin_idx;
	   				
	   	Trigger_VGA_TX_Data(sisusb, txlen, buf);
	   	len-=txlen;                               
   	}	   			
    kfree(buf);   	    

		#endif

        unsigned char *buf;
        u32           txlen;    
		//u32 begin_idx = 0;
		//u32 k, x;
                
        	
    	if (!(buf = kmalloc(64*1024, GFP_KERNEL))) {
				printk(KERN_INFO
					"triggerusb: Failed to allocate memory .\n");
				return ;
			}
			memset(buf, 255, 64*1024);
			while(len) {
	   		txlen = (len>64*1024)? (64*1024):len;
					
	   		SendBlockCommand(sisusb,2,
                            	buf,
                            0,      //users or kernels buffer.
                            txlen);	
        len-=txlen;
    	}
	   kfree(buf);
		}    	    
}


//#define TEST_BLK_SIZE 131072
static void Trigger_Draw_Rectangle_On_MainScreen(struct sisusb_usb_data *sisusb , u16 x, u16 y, int width, 
																								 int height, u8 red, u8 green, u8 blue)
{
    u32              SrcX,SrcY,DstX,DstY,ColorFormat;
    u32	             len;           		       		       
    u32              DestStartAddr,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo;
    unsigned char    *buf;
    u32              txlen,r,g,b,i,k,begin_idx;
   
   
    SrcX = width;
    SrcY = height;
    DstX = SrcX;
    DstY = SrcY;		  
    
    ColorFormat=sisusb->ColorFormat;    

    if(ColorFormat == YUV444)	/// YUV444
       ColorInfo = 0x08;	/// YUV444 enable
    if(ColorFormat == RGB32)	/// 32-bit mode
       ColorInfo = 0x0e;	/// 32-bit enable
    if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
       ColorInfo=0x04;		/// BGR565 enable
    if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
       ColorInfo = 0x8a;	       /// YUV422 enable
    if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
       ColorInfo = 0x00;	/// BitMap enable, yulw set to 0
		if (ColorFormat == RGB24)
	    ColorInfo = 0x00;       

		
		HScaleFactor = 0x4000;
		VScaleFactor = 0x4000;	
		 
  
    len=SrcX*(sisusb->bpp/8)*SrcY;
    
    if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
			printk(KERN_ERR
									"triggerusb: Failed to allocate memory .\n");
				return ;
    }

		#ifdef tusbvga_DBG
	
		printk("%s: x=%d y=%d\n", __FUNCTION__, x, y);   

		#endif
		
		//VRAM is always RGB24 , so each pixels is equal to 3 bytes   
   	DestStartAddr = (((y-1) * sisusb->hres) + x ) * 3 ;  		
    
    Trigger_VGA_TX_Cmd(sisusb, DestStartAddr, SrcX, SrcY, DstX, DstY, HScaleFactor,
                     	 	 VScaleFactor, ColorFormat, len);
    	
    r=red;g=green;b=blue;
    begin_idx = 0;
    while(len) {
    	txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   
	   	for(k=0;k<begin_idx;k++) buf[k] = 0;
	   	for(i=0;i<(txlen/(sisusb->bpp/8));i++) {
	   		buf[(i*(sisusb->bpp/8))+begin_idx] = b;
	   		buf[(i*(sisusb->bpp/8))+1+begin_idx] = g;
	   		buf[(i*(sisusb->bpp/8))+2+begin_idx] = r;
	   	}
	   	//add zero pading DWORD alignment
	   	if((txlen % (sisusb->bpp/8)) != 0) {
	   		for(k=0;k<(txlen % (sisusb->bpp/8));k++) 
	   			buf[(i*(sisusb->bpp/8))+k] = 0x0;	
	   		
	   		begin_idx = (begin_idx + (sisusb->bpp/8) - (txlen % (sisusb->bpp/8))) % (sisusb->bpp/8);
	   	}else
	   		begin_idx = begin_idx;
	   				
	   	Trigger_VGA_TX_Data(sisusb, txlen, buf);
	   	len-=txlen;                               
   	}	   			
    kfree(buf);   	    
}

static void Trigger_Draw_Backgroud(struct sisusb_usb_data *sisusb , u8 red, u8 green, u8 blue)
{
    u32              SrcX,SrcY,DstX,DstY,ColorFormat;
    u32	             len;        		       		       
    u32              DestStartAddr,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo;
    unsigned char    *buf;
    u32              txlen,r,g,b,x,k,begin_idx;
   
   
    SrcX = sisusb->hres;
    SrcY = sisusb->vres;
    DstX = sisusb->hres;
    DstY = sisusb->vres;		  
    
    ColorFormat=sisusb->ColorFormat;    

    if(ColorFormat == YUV444)	/// YUV444
       ColorInfo = 0x08;	/// YUV444 enablef
    if(ColorFormat == RGB32)	/// 32-bit mode
       ColorInfo = 0x0e;	/// 32-bit enable
    if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
       ColorInfo=0x04;		/// BGR565 enable
    if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
       ColorInfo = 0x8a;	       /// YUV422 enable
    if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
       ColorInfo = 0x00;	/// BitMap enable, yulw set to 0
		if (ColorFormat == RGB24)
	    ColorInfo = 0x00;       

		
		HScaleFactor = 0x4000;
		VScaleFactor = 0x4000;	
		 
  
    len=SrcX*(sisusb->bpp/8)*SrcY;
    
    if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
			printk(KERN_ERR
									"triggerusb: Failed to allocate memory .\n");
				return ;
    }
      
   	//DestStartAddr = ((0*sisusb->hres) + 0)*3;  		

	DestStartAddr = 0;

	#if 0
	TransferBlockData1(sisusb, 
     	                   sisusb->hres, sisusb->vres, 
     	                   sisusb->hres, sisusb->vres,
     			   ColorFormat, sisusb->bpp, 
     			   0, 0, sisusb->hres*3, 
     			   NO_SCALE, NO_SCALE, 
     			   NULL,0, 0);

	return;	
	#endif

    len=SrcX*(sisusb->bpp/8)*SrcY; 
    Trigger_VGA_TX_Cmd(sisusb, DestStartAddr, SrcX, SrcY, DstX, DstY, HScaleFactor,
                     	 	 VScaleFactor, ColorFormat, len);
    	
    r=red;g=green;b=blue;
    begin_idx = 0;
    while(len) {
    	txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   
	   	for(k=0;k<begin_idx;k++) buf[k] = 0;
	   	for(x=0;x<(txlen/(sisusb->bpp/8));x++) {
	   		buf[(x*(sisusb->bpp/8))+begin_idx] = b;
	   		buf[(x*(sisusb->bpp/8))+1+begin_idx] = g;
	   		buf[(x*(sisusb->bpp/8))+2+begin_idx] = r;
	   	}
	   	//add zero pading DWORD alignment
	   	if((txlen % (sisusb->bpp/8)) != 0) {
	   		for(k=0;k<(txlen % (sisusb->bpp/8));k++) 
	   			buf[(x*(sisusb->bpp/8))+k] = 0x0;	
	   		
	   		begin_idx = (begin_idx + (sisusb->bpp/8) - (txlen % (sisusb->bpp/8))) % (sisusb->bpp/8);
	   	}else
	   		begin_idx = begin_idx;
	   				
	   	Trigger_VGA_TX_Data(sisusb, txlen, buf);
	   	len-=txlen;                               
   	}	   			
    kfree(buf);   	    
}

//*********************************************************
static void Trigger_Demo_MainScreen_Display(struct sisusb_usb_data *sisusb)
{
    Trigger_Draw_Backgroud(sisusb, 255,0,0);
    mdelay(1000);
    Trigger_Draw_Backgroud(sisusb, 0,255,0);
    mdelay(1000);
    Trigger_Draw_Backgroud(sisusb, 0,0,255);
    mdelay(1000);
}

//==========================================================================
//	if ratio = 4 , it means src:dst = 1:4
//==========================================================================
static void Trigger_Demo_Stretch(struct sisusb_usb_data *sisusb, int ratio)
{
    u32              SrcX,SrcY,DstX,DstY,ColorFormat,ScrXPoint,ScrYPoint;
    u32	             len;    
    u32              DestStartAddr,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo;
    unsigned char    *buf;
    u32              txlen = 0,r = 0,g = 0,b = 0,i = 0,x = 0,k = 0,begin_idx = 0;
  
    ScrXPoint=ScrYPoint=0;
    
    SrcX=sisusb->hres / ratio;
    SrcY=sisusb->vres / ratio;
    DstX=sisusb->hres;
    DstY=sisusb->vres;		  
    
    ColorFormat=sisusb->ColorFormat;    

		//printk(" ColorFormat = %x \n", ColorFormat);
    if(ColorFormat == YUV444)	/// YUV444
       ColorInfo = 0x08;	/// YUV444 enable
    if(ColorFormat == RGB32)	/// 32-bit mode
       ColorInfo = 0x0e;	/// 32-bit enable
    if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
       ColorInfo=0x04;		/// BGR565 enable
    if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
       ColorInfo = 0x8a;	       /// YUV422 enable
    if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
       ColorInfo = 0x00;	/// BitMap enable, yulw set to 0
		if (ColorFormat == RGB24)
	    ColorInfo = 0x00;       

		if(SrcX != DstX) 
			//HScaleFactor = Trigger_ComputeScaleFactor ((float)((float)SrcX/(float)DstX));
			if(ratio == 2) 
				HScaleFactor = 0x2000;
			else
				HScaleFactor = 0x1000;
		else
			HScaleFactor = 0x4000;
		
		if(SrcY != DstY) 
			//VScaleFactor = Trigger_ComputeScaleFactor ((float)((float)SrcY/(float)DstY));
			if(ratio == 2) 
				VScaleFactor = 0x2000;
			else
				VScaleFactor = 0x1000;
		else
			VScaleFactor = 0x4000;	
		 
  
    len=SrcX*(sisusb->bpp/8)*SrcY;
    
    if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
			printk(KERN_ERR
									"triggerusb: Failed to allocate memory .\n");
				return ;
    }
    
      
    for(i=0;i<8;i++) {
    	if(i%2) 
    		DestStartAddr = 4*1024*1024;
    	else	
    		DestStartAddr = 0;   
    		
    	len=SrcX*(sisusb->bpp/8)*SrcY; 
    	Trigger_VGA_TX_Cmd(sisusb, DestStartAddr, SrcX, SrcY, DstX, DstY, HScaleFactor,
                     	 	 VScaleFactor, ColorFormat, len);
    	
    	
      switch(i % 4) {
      case 0:
      	r=0;g=255;b=0;
      	break;
      case 1:
      	r=255;g=0;b=0;
      	break;
   		case 2:
      	r=0;g=0;b=255;
      	break;
      case 3:
      	r=0;g=255;b=255;
        break;        	
      } 
      
    	begin_idx = 0;
    	
			while(len) {
	   			txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   
	   			//printk("txlen = %ld , TEST_BLK_SIZE = %ld\n", txlen, TEST_BLK_SIZE);
	   			for(k=0;k<begin_idx;k++) buf[k] = 0;
	   			for(x=0;x<(txlen/(sisusb->bpp/8));x++) {
	   				buf[(x*(sisusb->bpp/8))+begin_idx] = b;
	   				buf[(x*(sisusb->bpp/8))+1+begin_idx] = g;
	   				buf[(x*(sisusb->bpp/8))+2+begin_idx] = r;
	   			}
	   			//printk(" begin_idx=%d %d %d %d\n",begin_idx,  buf[0],buf[1],buf[2]);
	   			
	   			//add zero pading DWORD alignment
	   			if((txlen % (sisusb->bpp/8)) != 0) {
	   				for(k=0;k<(txlen % (sisusb->bpp/8));k++) 
	   					buf[(x*(sisusb->bpp/8))+k] = 0x0;
	   				
	   				begin_idx = (begin_idx + (sisusb->bpp/8) - (txlen % (sisusb->bpp/8))) % (sisusb->bpp/8);
	   			}else
	   				begin_idx = begin_idx;
	   				
	   			Trigger_VGA_TX_Data(sisusb, txlen, buf);
	   			len-=txlen;                               
    	}	   			
      Trigger_VGA_TX_Set_Frame0_Addr(sisusb, DestStartAddr);
      mdelay(500);
    }
    
    Trigger_VGA_TX_Set_Frame0_Addr(sisusb, 0); // WHS: reset frame0 baseaddr to 0
    kfree(buf);   	    
}



static int Trigger_VGA_Overlay_Cmd
(
	struct sisusb_usb_data *sisusb, u32 FrameBaseAddr, 
	u16 over_left, u16 over_top, u16 over_right, u16 over_bottom,
	u8 *src_ckey, u8 *dst_ckey, int active_frame
)
{
	unsigned char    BlockData[256], Start;
	int              j,Count;
	
	
	BlockData[0] = 0x22;
	j = 2;
	Count = 0;      
 	if( dst_ckey != NULL) { 
  	//WriteRegisterByte(pDevExt, 0xFB70,pByte[0]); // blue
   	BlockData[j++] = 0xFB;
   	BlockData[j++] = 0x70;
    BlockData[j++] = dst_ckey[0];
    //WriteRegisterByte(pDevExt, 0xFB71,pByte[1]);	// green
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x71;
    BlockData[j++] = dst_ckey[1];    		
    //WriteRegisterByte(pDevExt, 0xFB72,pByte[2]);	// red
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x72;
    BlockData[j++] = dst_ckey[2];  
    Count += 3;
  }
  if (src_ckey != NULL) {
  	//WriteRegisterByte(pDevExt, 0xFB74,pByte[0]);	// blue
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x74;
    BlockData[j++] = src_ckey[0];			
		//WriteRegisterByte(pDevExt, 0xFB75,pByte[1]);	// green
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x75;
    BlockData[j++] = src_ckey[1];			
		//WriteRegisterByte(pDevExt, 0xFB76,pByte[2]);	// red
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x76;
    BlockData[j++] = src_ckey[2];
    Count += 3;
	}
	
	
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x66;
  BlockData[j++] = (unsigned char)(over_top&0xff);    
  
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x67;
 	BlockData[j++] = (unsigned char)(over_top>>8);    
 
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x68;
  BlockData[j++] = (unsigned char)(over_left&0xff);
  
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x69;
  BlockData[j++] = (unsigned char)(over_left>>8);

  
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x6a;
  BlockData[j++] = (unsigned char)(over_bottom & 0xff);
   
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x6b;
  BlockData[j++] = (unsigned char)(over_bottom>>8);
    //WriteRegisterByte(pDevExt, 0xFB6c,(UCHAR)((pOverlayContext->rDest.right-1)&0xff));
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x6c;
  BlockData[j++] = (unsigned char)(over_right&0xff);
  
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x6d;
  BlockData[j++] = (unsigned char)(over_right>>8);
  Count += 8;
  
  if (active_frame == 0) {	// Overlay frame 0 will be actived
		//WriteRegisterByte (pDevExt, 0xFB65, 0x01);	// set Frame 0 address
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x65;
    BlockData[j++] = 0x21;	//0x01;		
		//WriteRegisterByte (pDevExt, 0xFB62, 0xc0);
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x62;
    BlockData[j++] = (unsigned char)FrameBaseAddr; //0xc0;		
		//WriteRegisterByte (pDevExt, 0xFB63, 0x27);
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x63;
    BlockData[j++] = (unsigned char)(FrameBaseAddr>>8);//0x27;		
		//WriteRegisterByte (pDevExt, 0xFB64, 0x09);	// Frame 0 base address 0x0927c0
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x64;
    BlockData[j++] = (unsigned char)(FrameBaseAddr>>16); //0x09;		
		//WriteRegisterByte (pDevExt, 0xFB65, 0x10);	//switch 0 Frame 0   	
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x65;
    BlockData[j++] = 0x12;	//0x10;
    Count += 5;
 	}
 	else if (active_frame == 1) {	// Overlay frame 1 will be actived
		//WriteRegisterByte (pDevExt, 0xFB65, 0x02);	// set Frame 0 address
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x65;
    BlockData[j++] = 0x12; //0x02;		
		//WriteRegisterByte (pDevExt, 0xFB62, 0x80);
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x62;
    BlockData[j++] = (unsigned char)(FrameBaseAddr); //0x80;		
		//WriteRegisterByte (pDevExt, 0xFB63, 0x4f);
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x63;
    BlockData[j++] = (unsigned char)(FrameBaseAddr>>8); //0x4f;		
		//WriteRegisterByte (pDevExt, 0xFB64, 0x12);	// Frame 0 base address 0x124f80
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x64;
    BlockData[j++] = (unsigned char)(FrameBaseAddr>>16); //0x12;		
		//WriteRegisterByte (pDevExt, 0xFB65, 0x20);	//switch 0 Frame 0    	
    BlockData[j++] = 0xFB;
    BlockData[j++] = 0x65;
    BlockData[j++] = 0x21; //0x20;
    Count += 5;	  	
  }
  
  Start = 0x01;
 
	if (src_ckey != NULL) {
  	if(dst_ckey != NULL) 
    	Start |= 0x06;
    else
    	Start |= 0x04;
	}
 	else {
 		if (dst_ckey != NULL)
  		Start |= 0x2;
 	}
    
	//WriteRegisterByte(pDevExt, 0xFCB5,0x03); // enable main and object
	//WriteRegisterByte(pDevExt, 0xFB6e,0x1);
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x6e;
  BlockData[j++] = 0x01;    
  //WriteRegisterByte(pDevExt, 0xFB60,Start);   
  BlockData[j++] = 0xFB;
  BlockData[j++] = 0x60;
  BlockData[j++] = Start;
  Count += 2;
	
	BlockData[1] = Count;
	return SendBlockCommand(sisusb, 3,
                   BlockData,
                   0,           //users or kernels buffer.
                   j);   
}

#ifdef tusbvga_EDID

int ReadEDID(struct sisusb_usb_data *sisusb, unsigned char *edid_buf, int buf_len)
{
        unsigned char packet[8];
        u32 bytes_transferred, byte_read;
  int ret=0;
    
#ifdef SISUSB_DONTSYNC
        if (!(sisusb_wait_all_out_complete(sisusb)))
              return 1;
#endif
   memset(packet,0,8);
   packet[0]=0x23;
   ret = sisusb_send_bulk_msg(sisusb,
                                3,    //end point 2.
                                4,
                                (char *)packet,
                                NULL,
                                0,
                                &bytes_transferred,
                                0,
                                1);
//Louis Tsai, do async
//                             0);

        if(ret == 0) { //read command successfully, start to get result from endpoint 0
                if(edid_buf != NULL && buf_len>=512) {
                        ret = sisusb_recv_bulk_msg(sisusb, 1, 512, (char*) edid_buf, NULL, &byte_read, 0);
                        return ret;
                }
                else
                        return -EIO;
        }
        else
                return ret;
}


#endif

static int DoCopyVramExa(struct sisusb_usb_data *sisusb){

	unsigned long begTime = jiffies, endTime;
		
	int ret = 0;

	#if 1
	
	u32              SrcX,SrcY,DstX,DstY,ColorFormat;
    u16							 O_left, O_top, O_right, O_bottom;	
    u32	             bytes, len;     		       		       
    u32              DestStartAddr,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo;
    unsigned char    *buf;
    u32              txlen=0, r=0, g=0, b=0, x=0, i=0, k=0, begin_idx = 0;
    unsigned long		 ScreenSize, FrameBaseAddr, showBaseAddr;
	unsigned char 	 src_ckey[3], dst_ckey[3];

	#ifdef tusbvga_DBG
    
	printk(KERN_INFO "WHS: DoCopyVramExa\n");

	#endif
				
	#if 0

	sisusb->par0 = y->data3; // WHS: DestX
	sisusb->par1 = y->data4; // WHS: DestY
	sisusb->par2 = y->data5; // WHS: DestAddr	
	sisusb->par7 = y->data10; // WHS: user buf

	#endif
				
    SrcX = sisusb->par0;			
    SrcY = sisusb->par1;			
    DstX = sisusb->par0;			
    DstY = sisusb->par1;		  
            
    ColorFormat = sisusb->ColorFormat;    

	#ifdef tusbvga_DBG

	printk("DoCopyVramExa: ColorFormat: %x \n", ColorFormat);

	#endif
		
    if(ColorFormat == YUV444)	/// YUV444
  	{ColorInfo = 0x08;bytes=3;}	/// YUV444 enable
  if(ColorFormat == RGB32)	/// 32-bit mode
  	{ColorInfo = 0x0e;bytes=4;}	/// 32-bit enable
  if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
  	{ColorInfo=0x04;bytes=2;}	/// BGR565 enable
  if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
  	{ColorInfo = 0x8a;bytes=2;}	/// YUV422 enable
  if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
  	{ColorInfo = 0x00;bytes=1;}	/// BitMap enable, yulw set to 0
	if(ColorFormat == RGB24)
		{ColorInfo = 0x00;bytes=3;} 

		
		HScaleFactor = 0x4000;
		VScaleFactor = 0x4000;	
		//HScaleFactor = sisusb->par9;
		//VScaleFactor = sisusb->par10;	
		 
		  
//    len=SrcX*(sisusb->bpp/8)*SrcY;
    
    //if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
	//		printk(KERN_ERR
	//								"triggerusb: Failed to allocate memory .\n");
	//			return ;
    //}

	buf = sisusb->par7;

	// only blue color on main screen can be overlayed
    //dst_ckey[0] = sisusb->par8 & 0xff; 
	//dst_ckey[1] = (sisusb->par8 & 0xff00) >> 8;  
	//dst_ckey[2] = (sisusb->par8 & 0xff0000) >> 16;  

	//printk("%x: dst_ckey[0]:%x, dst_ckey[1]: %x, dst_ckey[2]: %x\n", sisusb->par8, dst_ckey[0], dst_ckey[1], dst_ckey[2]);

	//if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
	//		printk(KERN_ERR
	//								"triggerusb: Failed to allocate memory .\n");
	//			return ;
    //}
	//memset(buf, 127, TEST_BLK_SIZE);
	
    //Trigger only RGB24(3bytes) format on VRAM 
    //ScreenSize = sisusb->hres * sisusb->vres * 3;
    
    // Demo loop
    //for(i=0; i<32; i++) {  
   		//if(sisusb->flipable) { // VRAM > 8M: WHS: flip = 1
   		
   		//	if(!(i%2)) {
   		//		DestStartAddr = ScreenSize;		//Overlay Frame 0
   		//		FrameBaseAddr = ScreenSize >>2;
   		//	}
   		//	else {
   		//		DestStartAddr = 2*ScreenSize;  //Overlay Frame 1
   		//		FrameBaseAddr = (ScreenSize *2)>>2;	//DWORD alignment
   		//	}
   		//}

	DestStartAddr = sisusb->par2*3;
	
	#ifdef tusbvga_DBG

	printk("DoCopyVramExa: DestStartAddr: %d \n", DestStartAddr);

	#endif
			
		
   		//}
   		//else {
   		//	DestStartAddr = ScreenSize;		//Overlay Frame 0
   		//	FrameBaseAddr = ScreenSize >>2;
   		//}
   			
	//if(sisusb->flipable) DestStartAddr <<= (sisusb->flipBuf); // WHS: flips

	//FrameBaseAddr = DestStartAddr >> 2;
		
   		//Shift to beginning offset of overlay region
   		//Trigger only RGB24(3bytes) format on VRAM 
   		//DestStartAddr += (((O_top-1)*sisusb->hres) + (O_left)) * 3;		
		//DestStartAddr += ((320*sisusb->hres) + 240) * 3;		
   		
    	//len=SrcX*(sisusb->bpp/8)*SrcY; 
		//printk("len = %d \n", len);
    	len=SrcX*SrcY*bytes; 
		
	#ifdef tusbvga_DBG

	printk("DoCopyVramExa: len = %d \n", len);		
	
	#endif
    	
    	//the command phase of copying data to overlay region 
    	//ret = Trigger_VGA_TX_Cmd(sisusb, DestStartAddr, SrcX, SrcY, DstX, DstY, HScaleFactor,
        //             	 	 VScaleFactor, ColorFormat, len);
		
		if(ret < 0){

			printk(KERN_ERR"triggerusb: DoCopyVramExa: command phase error\n");
			return ret;
			
		}
    	    	
		#if 1

		begTime = jiffies;
		
    	while(len) {
    		txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   	   		
	   		//the data phase of copying data to overlay region 		
	   		//ret = Trigger_VGA_TX_Data(sisusb, txlen, buf);
			if(ret < 0){

				printk(KERN_ERR"triggerusb: DoCopyVramExa: data phase error\n");
				return ;
			
			}
	   		len -= txlen;         
			buf += txlen;
   		}	   			

		endTime = jiffies;

		#ifdef tusbvga_OPT

		printk(KERN_INFO"triggerusb: WHS: data phase ellapse time: %d\n", endTime - begTime);

		#endif

		// WHS: NULL, NULL: no color key
				
   		//ret = Trigger_VGA_Overlay_Cmd(sisusb, FrameBaseAddr, O_left, O_top, O_right, O_bottom, NULL, dst_ckey, 0);   		

		//if(ret < 0){

		//	printk(KERN_INFO"triggerusb: WHS: overlay command phase 2 error\n");
		//	return ret;
			
		//}
		
   		//mdelay(200);

		#endif

		
  	//}
   	//disable overlay
   	//WriteRegisterByte(sisusb, 0xFB60, 0);   
   	//mdelay(100);
    //kfree(buf);   	    

	#endif

	//sisusb->flipBuf ^= 1;
	
	return ret;

}

	
/*
						sisusb->hres
(0,0)--------------------- 
|													|
|		 	O(l,t)------        |
|			|						|				|
|			|	Overlay	  |				|sisusb->vres
|			|	(400x300)	|				|
|			 -------O(r,b)			|
|													|
|-------------------------

Main screen size 		= sisusb->hres*sisusb->vres*(sisusb->bpp/8)
Overlay screen size = 400*300*(sisusb->bpp/8);

O_left 		= (sisusb->hres/2) - (400/2);
O_top			= (sisusb->vres/2) - (300/2);
O_right 	= (sisusb->hres/2) + (400/2);
O_bottom 	= (sisusb->vres/2) + (300/2);

*/

#if 1

static int DoOverlay(struct sisusb_usb_data *sisusb){

	unsigned long begTime = jiffies, endTime;
		
	int ret = 0;

	#if 1
	
	u32              SrcX,SrcY,DstX,DstY,ColorFormat;
    u16							 O_left, O_top, O_right, O_bottom;	
    u32	             bytes, len;     		       		       
    u32              DestStartAddr,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo;
    unsigned char    *buf;
    u32              txlen=0, r=0, g=0, b=0, x=0, i=0, k=0, begin_idx = 0;
    unsigned long		 ScreenSize, FrameBaseAddr, showBaseAddr;
	unsigned char 	 src_ckey[3], dst_ckey[3];

	//int ret = 0;

	#ifdef tusbvga_DBG
	
	printk(KERN_INFO "WHS: DoOverlay\n");

	#endif

	//WriteRegisterByte(sisusb, 0xFB60, 1);   
		
		// Draw main screen with RGB(255.0.255) first
		//Trigger_Draw_Backgroud(sisusb, 255, 255, 255);

	#if 0

	sisusb->par0 = y->data3; // WHS: DestX
	sisusb->par1 = y->data4; // WHS: DestY
	sisusb->par2 = y->data5; // WHS: ScrPointX
	sisusb->par3 = y->data6; // WHS: ScrPointY		
	sisusb->par4 = y->data7; // WHS: ColorFormat	
	sisusb->par5 = y->data8; // WHS: SrcX
	sisusb->par6 = y->data9; // WHS: SrcY
	sisusb->par7 = y->data10; // WHS: user buf

	#endif
		
		// Copy size for Overlay region 
  	//SrcX = 400;			//overlay source width
    //SrcY = 300;			//overlay source high
    //DstX = 400;			//overlay dest	 width
    //DstY = 300;		  //overlay dest	 high
    SrcX = sisusb->par5;			//overlay source width
    SrcY = sisusb->par6;			//overlay source high
    DstX = sisusb->par0;			//overlay dest	 width
    DstY = sisusb->par1;		  //overlay dest	 high
    
    // coordinate axes of Overlay Region
    //O_left 		= (sisusb->hres/2) - (400/2);
	//	O_top			= (sisusb->vres/2) - (300/2);
	//	O_right 	= (sisusb->hres/2) + (400/2) - 1;
	//	O_bottom 	= (sisusb->vres/2) + (300/2) - 1;
    
	//O_left = 200;
	//O_top = 200;
	O_left = sisusb->par2;
	O_top = sisusb->par3;
	O_right = O_left + DstX - 1;
    O_bottom = O_top + DstY - 1;
    
    ColorFormat = sisusb->par4;    

		//printk(" ColorFormat = %x \n", ColorFormat);
    if(ColorFormat == YUV444)	/// YUV444
  	{ColorInfo = 0x08;bytes=3;}	/// YUV444 enable
  if(ColorFormat == RGB32)	/// 32-bit mode
  	{ColorInfo = 0x0e;bytes=4;}	/// 32-bit enable
  if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
  	{ColorInfo=0x04;bytes=2;}	/// BGR565 enable
  if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
  	{ColorInfo = 0x8a;bytes=2;}	/// YUV422 enable
  if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
  	{ColorInfo = 0x00;bytes=1;}	/// BitMap enable, yulw set to 0
	if(ColorFormat == RGB24)
		{ColorInfo = 0x00;bytes=3;} 

		
		HScaleFactor = 0x4000;
		VScaleFactor = 0x4000;	
		HScaleFactor = sisusb->par9;
		VScaleFactor = sisusb->par10;	
		 
		  
//    len=SrcX*(sisusb->bpp/8)*SrcY;
    
    //if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
	//		printk(KERN_ERR
	//								"triggerusb: Failed to allocate memory .\n");
	//			return ;
    //}

	buf = sisusb->par7;

	// only blue color on main screen can be overlayed
    dst_ckey[0] = sisusb->par8 & 0xff; 
	dst_ckey[1] = (sisusb->par8 & 0xff00) >> 8;  
	dst_ckey[2] = (sisusb->par8 & 0xff0000) >> 16;  

	#ifdef tusbvga_DBG

	printk("%x: dst_ckey[0]:%x, dst_ckey[1]: %x, dst_ckey[2]: %x\n", sisusb->par8, dst_ckey[0], dst_ckey[1], dst_ckey[2]);

	#endif

	//if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
	//		printk(KERN_ERR
	//								"triggerusb: Failed to allocate memory .\n");
	//			return ;
    //}
	//memset(buf, 127, TEST_BLK_SIZE);
	
    //Trigger only RGB24(3bytes) format on VRAM 
    ScreenSize = sisusb->hres * sisusb->vres * 3;
    
    // Demo loop
    //for(i=0; i<32; i++) {  
   		//if(sisusb->flipable) { // VRAM > 8M: WHS: flip = 1
   		
   		//	if(!(i%2)) {
   		//		DestStartAddr = ScreenSize;		//Overlay Frame 0
   		//		FrameBaseAddr = ScreenSize >>2;
   		//	}
   		//	else {
   		//		DestStartAddr = 2*ScreenSize;  //Overlay Frame 1
   		//		FrameBaseAddr = (ScreenSize *2)>>2;	//DWORD alignment
   		//	}
   		//}

			DestStartAddr = ScreenSize;		//Overlay Frame 0   			
		
   		//}
   		//else {
   		//	DestStartAddr = ScreenSize;		//Overlay Frame 0
   		//	FrameBaseAddr = ScreenSize >>2;
   		//}
   			
		if(sisusb->flipable) DestStartAddr <<= (sisusb->flipBuf); // WHS: flips

		FrameBaseAddr = DestStartAddr >> 2;
		
   		//Shift to beginning offset of overlay region
   		//Trigger only RGB24(3bytes) format on VRAM 
   		DestStartAddr += (((O_top-1)*sisusb->hres) + (O_left)) * 3;		
		//DestStartAddr += ((320*sisusb->hres) + 240) * 3;		
   		
    	//len=SrcX*(sisusb->bpp/8)*SrcY; 
		//printk("len = %d \n", len);
    	len=SrcX*SrcY*bytes; 

		#ifdef tusbvga_DBG
		printk("WHS: len = %d \n", len);		
		#endif
    	
    	//the command phase of copying data to overlay region 
    	ret = Trigger_VGA_TX_Cmd(sisusb, DestStartAddr, SrcX, SrcY, DstX, DstY, HScaleFactor,
                     	 	 VScaleFactor, ColorFormat, len);
		//ret = Trigger_VGA_TX_Cmd(sisusb, DestStartAddr*(sisusb->flipBuf + 1), SrcX, SrcY, DstX, DstY, HScaleFactor,
        //             	 	 VScaleFactor, ColorFormat, len);

		if(ret < 0){

			printk(KERN_INFO"triggerusb: WHS: overlay command phase error\n");
			return ret;
			
		}
    	    
		#if 1

		begTime = jiffies;
		
    	while(len) {
    		txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   	   		
	   		//the data phase of copying data to overlay region 		
	   		ret = Trigger_VGA_TX_Data(sisusb, txlen, buf);
			if(ret < 0){

				printk(KERN_INFO"triggerusb: WHS: overlay data phase error\n");
				return ;
			
			}
	   		len -= txlen;         
			buf += txlen;
   		}	   			

		endTime = jiffies;

		#ifdef tusbvga_OPT

		printk(KERN_INFO"triggerusb: WHS: data phase ellapse time: %d\n", endTime - begTime);

		#endif

		// WHS: NULL, NULL: no color key
				
   		//ret = Trigger_VGA_Overlay_Cmd(sisusb, FrameBaseAddr, O_left, O_top, O_right, O_bottom, NULL, dst_ckey, 0);   		
   		ret = Trigger_VGA_Overlay_Cmd(sisusb, FrameBaseAddr, O_left, O_top, O_right, O_bottom, NULL, dst_ckey, sisusb->flipBuf);   		

		if(ret < 0){

			printk(KERN_INFO"triggerusb: WHS: overlay command phase 2 error\n");
			return ret;
			
		}
		
   		//mdelay(128);

		#endif

		#if 0

		//begin_idx = 0;
    	while(len) {
    		txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   	   		
	   		Trigger_VGA_TX_Data(sisusb, txlen, buf);
	   		len-=txlen;                             			
   		}	   			
   		#endif
  	//}
   	//disable overlay
   	//WriteRegisterByte(sisusb, 0xFB60, 0);   
   	//mdelay(100);
    //kfree(buf);   	    

	#endif

	sisusb->flipBuf ^= 1;
	
	return ret;

}

#endif

#if 0

static int DoOverlay(struct sisusb_usb_data *sisusb){

	unsigned long begTime = jiffies, endTime;
		
	int ret = 0;

	#if 1
	
	u32              SrcX,SrcY,DstX,DstY,ColorFormat;
    u16							 O_left, O_top, O_right, O_bottom;	
    u32	             bytes, len;     		       		       
    u32              DestStartAddr,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo;
    unsigned char    *buf;
    u32              txlen=0, r=0, g=0, b=0, x=0, i=0, k=0, begin_idx = 0;
    unsigned long		 ScreenSize, FrameBaseAddr;
	unsigned char 	 src_ckey[3], dst_ckey[3];

	//int ret = 0;
    
	printk(KERN_INFO "WHS: DoOverlay\n");
		
		// Draw main screen with RGB(255.0.255) first
		//Trigger_Draw_Backgroud(sisusb, 255, 255, 255);

	#if 0

	sisusb->par0 = y->data3; // WHS: DestX
	sisusb->par1 = y->data4; // WHS: DestY
	sisusb->par2 = y->data5; // WHS: ScrPointX
	sisusb->par3 = y->data6; // WHS: ScrPointY		
	sisusb->par4 = y->data7; // WHS: ColorFormat	
	sisusb->par5 = y->data8; // WHS: SrcX
	sisusb->par6 = y->data9; // WHS: SrcY
	sisusb->par7 = y->data10; // WHS: user buf

	#endif
		
		// Copy size for Overlay region 
  	//SrcX = 400;			//overlay source width
    //SrcY = 300;			//overlay source high
    //DstX = 400;			//overlay dest	 width
    //DstY = 300;		  //overlay dest	 high
    SrcX = sisusb->par5;			//overlay source width
    SrcY = sisusb->par6;			//overlay source high
    DstX = sisusb->par0;			//overlay dest	 width
    DstY = sisusb->par1;		  //overlay dest	 high
    
    // coordinate axes of Overlay Region
    //O_left 		= (sisusb->hres/2) - (400/2);
	//	O_top			= (sisusb->vres/2) - (300/2);
	//	O_right 	= (sisusb->hres/2) + (400/2) - 1;
	//	O_bottom 	= (sisusb->vres/2) + (300/2) - 1;
    
	//O_left = 200;
	//O_top = 200;
	O_left = sisusb->par2;
	O_top = sisusb->par3;
	O_right = O_left + DstX - 1;
    O_bottom = O_top + DstY - 1;
    
    ColorFormat = sisusb->par4;    

		//printk(" ColorFormat = %x \n", ColorFormat);
    if(ColorFormat == YUV444)	/// YUV444
  	{ColorInfo = 0x08;bytes=3;}	/// YUV444 enable
  if(ColorFormat == RGB32)	/// 32-bit mode
  	{ColorInfo = 0x0e;bytes=4;}	/// 32-bit enable
  if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
  	{ColorInfo=0x04;bytes=2;}	/// BGR565 enable
  if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
  	{ColorInfo = 0x8a;bytes=2;}	/// YUV422 enable
  if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
  	{ColorInfo = 0x00;bytes=1;}	/// BitMap enable, yulw set to 0
	if(ColorFormat == RGB24)
		{ColorInfo = 0x00;bytes=3;} 

		
		HScaleFactor = 0x4000;
		VScaleFactor = 0x4000;	
		HScaleFactor = sisusb->par9;
		VScaleFactor = sisusb->par10;	
		 
		  
//    len=SrcX*(sisusb->bpp/8)*SrcY;
    
    //if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
	//		printk(KERN_ERR
	//								"triggerusb: Failed to allocate memory .\n");
	//			return ;
    //}

	buf = sisusb->par7;

	// only blue color on main screen can be overlayed
    dst_ckey[0] = sisusb->par8 & 0xff; 
	dst_ckey[1] = (sisusb->par8 & 0xff00) >> 8;  
	dst_ckey[2] = (sisusb->par8 & 0xff0000) >> 16;  

	printk("%x: dst_ckey[0]:%x, dst_ckey[1]: %x, dst_ckey[2]: %x\n", sisusb->par8, dst_ckey[0], dst_ckey[1], dst_ckey[2]);

	//if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
	//		printk(KERN_ERR
	//								"triggerusb: Failed to allocate memory .\n");
	//			return ;
    //}
	//memset(buf, 127, TEST_BLK_SIZE);
	
    //Trigger only RGB24(3bytes) format on VRAM 
    ScreenSize = sisusb->hres * sisusb->vres * 3;
    
    // Demo loop
    //for(i=0; i<32; i++) {  
   		//if(sisusb->flip) { // VRAM > 8M: WHS: flip = 1
   		//if(0) { // VRAM > 8M: WHS: flip = 1
   		//	if(!(i%2)) {
   		//		DestStartAddr = ScreenSize;		//Overlay Frame 0
   		//		FrameBaseAddr = ScreenSize >>2;
   		//	}
   		//	else {
   		//		DestStartAddr = 2*ScreenSize;  //Overlay Frame 1
   		//		FrameBaseAddr = (ScreenSize *2)>>2;	//DWORD alignment
   		//	}
   		//}
   		//else {
   			DestStartAddr = ScreenSize;		//Overlay Frame 0
   			FrameBaseAddr = ScreenSize >>2;
   		//}
   			
   		
   		//Shift to beginning offset of overlay region
   		//Trigger only RGB24(3bytes) format on VRAM 
   		DestStartAddr += (((O_top-1)*sisusb->hres) + (O_left)) * 3;		
		//DestStartAddr += ((320*sisusb->hres) + 240) * 3;		
   		
    	//len=SrcX*(sisusb->bpp/8)*SrcY; 
		//printk("len = %d \n", len);
    	len=SrcX*SrcY*bytes; 
		printk("WHS: len = %d \n", len);
		
    	
    	//the command phase of copying data to overlay region 
    	ret = Trigger_VGA_TX_Cmd(sisusb, DestStartAddr, SrcX, SrcY, DstX, DstY, HScaleFactor,
                     	 	 VScaleFactor, ColorFormat, len);

		if(ret < 0){

			printk(KERN_INFO"triggerusb: WHS: overlay command phase error\n");
			return ret;
			
		}
    	
    	//switch(i % 4) {
      //case 0:
      //	r=0;g=255;b=0;
      //	break;
      //case 1:
      //	r=255;g=0;b=0;
      //	break;
   	//	case 2:
      //	r=0;g=0;b=255;
      //	break;
      //case 3:
      //	r=0;g=255;b=255;
       // break;        	
     // } 
    	//begin_idx = 0;

		#if 1
    	while(len) {
    		txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   	   		
	   		//the data phase of copying data to overlay region 		
	   		ret = Trigger_VGA_TX_Data(sisusb, txlen, buf);
			if(ret < 0){

				printk(KERN_INFO"triggerusb: WHS: overlay data phase error\n");
				return ;
			
			}
	   		len -= txlen;         
			buf += txlen;
   		}	   			

		// WHS: NULL, NULL: no color key
   		ret = Trigger_VGA_Overlay_Cmd(sisusb, FrameBaseAddr, O_left, O_top, O_right, O_bottom, NULL, dst_ckey, 0);

		if(ret < 0){

			printk(KERN_INFO"triggerusb: WHS: overlay command phase 2 error\n");
			return ret;
			
		}
		
   		//mdelay(200);

		#endif

		#if 0

		//begin_idx = 0;
    	while(len) {
    		txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   	   		
	   		Trigger_VGA_TX_Data(sisusb, txlen, buf);
	   		len-=txlen;                             			
   		}	   			
   		#endif
  	//}
   	//disable overlay
   	//WriteRegisterByte(sisusb, 0xFB60, 0);   
   	//mdelay(100);
    //kfree(buf);   	    

	#endif
					
	return ret;

}

#endif


static void Trigger_Demo_Overlay(struct sisusb_usb_data *sisusb, int flip)
{
    u32              SrcX,SrcY,DstX,DstY,ColorFormat;
    u16							 O_left, O_top, O_right, O_bottom;	
    u32	             len;     		       		       
    u32              DestStartAddr,HScaleFactor,VScaleFactor;
    unsigned char    ColorInfo;
    unsigned char    *buf;
    u32              txlen=0, r=0, g=0, b=0, x=0, i=0, k=0, begin_idx = 0;
    unsigned long		 ScreenSize, FrameBaseAddr;
	int ret = 0;
    
		
		// Draw main screen with RGB(255.0.255) first
		Trigger_Draw_Backgroud(sisusb, 255, 255, 255);
		
		
		// Copy size for Overlay region 
  	SrcX = 700;			//overlay source width
    SrcY = 480;			//overlay source high
    DstX = 700;			//overlay dest	 width
    DstY = 480;		  //overlay dest	 high

	//SrcX = 200;			//overlay source width
    //SrcY = 200;			//overlay source high
    //DstX = 200;			//overlay dest	 width
    //DstY = 200;		  //overlay dest	 high
    
    // coordinate axes of Overlay Region
    O_left 		= (sisusb->hres/2) - (400/2);
		O_top			= (sisusb->vres/2) - (300/2);  
		O_right 	= (sisusb->hres/2) + (400/2) - 1;
		O_bottom 	= (sisusb->vres/2) + (300/2) - 1;
    
	O_left = 100;
	O_top = 100;
	O_right = O_left + SrcX - 1;
    O_bottom = O_top + SrcY - 1;
	O_right = O_left + DstX - 1;
    O_bottom = O_top + DstY - 1;
    
    ColorFormat = sisusb->ColorFormat;    
	//printk(" ColorFormat = %x \n", ColorFormat);
    if(ColorFormat == YUV444)	/// YUV444
       ColorInfo = 0x08;	/// YUV444 enable
    if(ColorFormat == RGB32)	/// 32-bit mode
       ColorInfo = 0x0e;	/// 32-bit enable
    if(ColorFormat == RGB565)	/// 16-bit mode	BGR565
       ColorInfo=0x04;		/// BGR565 enable
    if(ColorFormat == YUV422)	/// 16-bit mode	YUV422
       ColorInfo = 0x8a;	       /// YUV422 enable
    if(ColorFormat == COLOR_PALETTE)   /// 8-bit mode BIT-Map enable
       ColorInfo = 0x00;	/// BitMap enable, yulw set to 0
		if (ColorFormat == RGB24)
	    ColorInfo = 0x00;       

		
		HScaleFactor = 0x4000;  
		VScaleFactor = 0x4000;	
		//HScaleFactor = 0x4c4a;  
		//VScaleFactor = 0x5a9e;	
		 			  
    len=SrcX*(sisusb->bpp/8)*SrcY;
    
    if (!(buf = kmalloc(TEST_BLK_SIZE, GFP_KERNEL))) {
			printk(KERN_ERR
									"triggerusb: Failed to allocate memory .\n");
				return ;
    }
	
	//return;
    
    //Trigger only RGB24(3bytes) format on VRAM 
    ScreenSize = sisusb->hres * sisusb->vres * 3;
    
    // Demo loop
    for(i=0; i<2; i++) {  
   		if(flip) { // VRAM > 8M: WHS: flip = 1
   		//if(0) { // VRAM > 8M: WHS: flip = 1
   			if(!(i%2)) {
   				DestStartAddr = ScreenSize;		//Overlay Frame 0
   				FrameBaseAddr = ScreenSize >>2;
   			}
   			else {
   				DestStartAddr = 2*ScreenSize;  //Overlay Frame 1
   				FrameBaseAddr = (ScreenSize *2)>>2;	//DWORD alignment
   			}
   		}
   		else {
   			DestStartAddr = ScreenSize;		//Overlay Frame 0
   			FrameBaseAddr = ScreenSize >>2;
   		}
   			
   		
   		//Shift to beginning offset of overlay region
   		//Trigger only RGB24(3bytes) format on VRAM 
   		//DestStartAddr += (((O_top-1 + i*5)*sisusb->hres) + (O_left + i*5)) * 3;		
   		DestStartAddr += (((O_top-1)*sisusb->hres) + (O_left)) * 3;		
		//DestStartAddr += ((320*sisusb->hres) + 240) * 3;		
   		
    	len=SrcX*(sisusb->bpp/8)*SrcY; 
    	
    	//the command phase of copying data to overlay region 
    	ret = Trigger_VGA_TX_Cmd(sisusb, DestStartAddr, SrcX, SrcY, DstX, DstY, HScaleFactor,
                     	 	 VScaleFactor, ColorFormat, len);

		if(ret < 0){

			printk(KERN_INFO"triggerusb: WHS: VGA command phase error\n");
			return ;
			
		}
    	
    	switch(i % 4) {
      case 0:
      	r=0;g=255;b=0;
      	break;
      case 1:
      	r=255;g=0;b=0;
      	break;
   		case 2:
      	r=0;g=0;b=255;
      	break;
      case 3:
      	r=0;g=255;b=255;
        break;        	
      } 
    	begin_idx = 0;
    	while(len) {
    		txlen = (len>TEST_BLK_SIZE) ? TEST_BLK_SIZE : len;	   	   
	   		for(k=0;k<begin_idx;k++) buf[k] = 0;
	   		for(x=0;x<(txlen/(sisusb->bpp/8));x++) {
	   			buf[(x*(sisusb->bpp/8))+begin_idx] = b;
	   			buf[(x*(sisusb->bpp/8))+1+begin_idx] = g;
	   			buf[(x*(sisusb->bpp/8))+2+begin_idx] = r;
	   		}
	   		//add zero pading DWORD alignment
	   		if((txlen % (sisusb->bpp/8)) != 0) {
	   			for(k=0;k<(txlen % (sisusb->bpp/8));k++) 
	   				buf[(x*(sisusb->bpp/8))+k] = 0x0;	
	   		
	   			begin_idx = (begin_idx + (sisusb->bpp/8) - (txlen % (sisusb->bpp/8))) % (sisusb->bpp/8);
	   		}else
	   			begin_idx = begin_idx;
	   		//the data phase of copying data to overlay region 		
	   		ret = Trigger_VGA_TX_Data(sisusb, txlen, buf);

			if(ret < 0){

				printk(KERN_INFO"triggerusb: WHS: VGA data phase error\n");
				return ;
			
			}
			
	   		len-=txlen;                             			
   		}	   			

		// WHS: NULL, NULL: no color key
   		Trigger_VGA_Overlay_Cmd(sisusb, FrameBaseAddr, O_left+i*0, O_top+i*0, O_right+i*0, O_bottom+i*0, NULL, NULL, i%2);
   		mdelay(200);
   		
  	}
   	//disable overlay
   	WriteRegisterByte(sisusb, 0xFB60, 0);   
   	//mdelay(100);
    kfree(buf);   	    
}

static void Trigger_Demo_Color_Key(struct sisusb_usb_data *sisusb, int flip)
{
    u32              SrcX,SrcY,DstX,DstY,ColorFormat;
    u16							 O_left, O_top, O_right, O_bottom;	
    u32	             len = 0;    
    u32              DestStartAddr,HScaleFactor,VScaleFactor;
    unsigned long		 ScreenSize, FrameBaseAddr;
    int							 x_offset = 0, y_offset = 0, i, BMP_bpp = 0;
    unsigned char 	 src_ckey[3], dst_ckey[3];
    
    
  
		
		// Draw main screen with RGB(255.255.255) first
		Trigger_Draw_Backgroud(sisusb, 0, 0, 255);
		Trigger_Draw_Rectangle_On_MainScreen(sisusb, (sisusb->hres/2) - 200, (sisusb->vres/2) - 150,
																				400, 300, 100, 100, 0);
		
		// Copy size for Overlay region 
		
	
  	SrcX = *((long *)(&ball1[18]));			//overlay source width , offset 18 of BMP file
  																			// is the bitmap width in pixels (signed integer).												
    SrcY = *((long *)(&ball1[22]));			//overlay source high , offset 22 of BMP file
  																			// is the bitmap height  in pixels (signed integer).		
    
    DstX = SrcX;
    DstY = SrcY;
    
    ColorFormat = sisusb->ColorFormat;    

  
		HScaleFactor = 0x4000;
		VScaleFactor = 0x4000;	
		 
		BMP_bpp = (int)(ball1[28]);
		
   
    len=SrcX*(BMP_bpp/8)*SrcY; 

    
  	//printk("%s: SrcX = %d, SrcY =%d len=%d BMP_bpp = %d\n", __FUNCTION__, SrcX, SrcY, len, BMP_bpp);
  
    //Trigger only RGB24(3bytes) format on VRAM 
    ScreenSize = sisusb->hres * sisusb->vres * 3;
    //printk("ScreenSize = %d\n", ScreenSize);
    
    
    O_left = O_top	= 0;
    O_right 	= O_left+SrcX -1;
    O_bottom 	= O_top+SrcY-1;
    // Demo loop
    x_offset = y_offset = 5;
    
    //it means hide black color on overlay region 
    src_ckey[0] = src_ckey[1] = src_ckey[2] = 0; // black
   
   	// only blue color on main screen can be overlayed
    dst_ckey[0] = 255; dst_ckey[1] = 0; dst_ckey[2] = 0; // blue
    
    
    for(i = 0; i<2048; i++) {
    	// coordinate axes of Overlay Region
    	if(x_offset > 0 && ((O_right+x_offset) >= sisusb->hres) )
    		x_offset = -x_offset;
    	else
    	if(x_offset < 0 && ((O_left+x_offset) <= 0) )
    		x_offset = -x_offset;
    	
    	if(y_offset > 0 && ((O_bottom+y_offset) >= sisusb->vres) )
    		y_offset = -y_offset;
    	else
    	if(y_offset < 0 && ((O_top+y_offset) <= 0) )
    		y_offset = -y_offset;
    	
    	
    	
    	O_left 		+=  x_offset;
			O_top			+= 	y_offset;
			O_right 	+= 	x_offset;
			O_bottom 	+=  y_offset;
    
     	//printk("overlay left = %d ,top =%d , right= %d, bottom = %d\n", O_left, O_top, O_right, O_bottom);
   		if(flip) { //VRAM > 8M
   			if(!(i%2)) {
   				DestStartAddr = ScreenSize;		// Overlay Frame 0
   				FrameBaseAddr = ScreenSize >>2; // DWORD alignment: WHS: shift left
   			}
   			else {
   				DestStartAddr = 2*ScreenSize;  // Overlay Frame 1
   				FrameBaseAddr = (ScreenSize *2)>>2;	// DWORD alignment: WHS: shift left
   			}
   		}
   		else {
   			DestStartAddr = ScreenSize;		//Overlay Frame 0
   			FrameBaseAddr = ScreenSize >>2;
   		}
   			
   		
   		//Shift to beginning offset of overlay region
   		 //Trigger only RGB24(3bytes) format on VRAM 
   		
   		DestStartAddr += (((O_top-1)*sisusb->hres) + O_left) * 3;		
   		//printk("DestStartAddr=%ld\n", DestStartAddr);
    	
    	 len=SrcX*(BMP_bpp/8)*SrcY; 
    	//the command phase of copying data to overlay region 
    	Trigger_VGA_TX_Cmd(sisusb, DestStartAddr, SrcX, SrcY, DstX, DstY, HScaleFactor,
                     	 	 VScaleFactor, ColorFormat, len);
    	
    	if(i%2)
	   		Trigger_VGA_TX_Data(sisusb, len, ball2+54);
	   		//Trigger_VGA_TX_Data(sisusb, len, ball2);
	   	else
	   		Trigger_VGA_TX_Data(sisusb, len, ball1+54);
	   		//Trigger_VGA_TX_Data(sisusb, len, ball1);
	   	Trigger_VGA_Overlay_Cmd(sisusb, FrameBaseAddr, O_left, O_top, O_right, O_bottom, src_ckey, dst_ckey,i%2);
	   	//Trigger_VGA_Overlay_Cmd(sisusb, FrameBaseAddr, O_left, O_top, O_right, O_bottom, NULL, NULL,i%2);
   		mdelay(50);
  	}
   	//disable overlay
   	//WriteRegisterByte(sisusb, 0xFB60, 0);   
   	mdelay(100);   
   	//kfree(ball1);
   	//kfree(ball2);
}



void MyDelay(unsigned long mSec)
{
  struct timeval stv,etv;
  
  do_gettimeofday(&stv);
  do_gettimeofday(&etv);
  
  while( (etv.tv_usec-stv.tv_usec) < (mSec*2000) ){
//  		printk("%s: gSetMode: %d\n", __FUNCTION__, gSetMode);
         do_gettimeofday(&etv);;

  	}
  	
}	

//***********************************************************************
unsigned char SetModeInfo (struct sisusb_usb_data *sisusb, unsigned long modeIndex)
{
    switch(modeIndex)
    {
		case MODE_320X240X8X60:
	    sisusb->ColorFormat = COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=320;
	    sisusb->vres=240;	
	    sisusb->HW_hres=320;
	    sisusb->HW_vres=240;	    	
		break;
		case MODE_320X240X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=320;
	    sisusb->vres=240;
	    sisusb->HW_hres=320;
	    sisusb->HW_vres=240;	    	
		break;
		case MODE_320X240X32X60:
	    sisusb->ColorFormat= RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=320;
	    sisusb->vres=240;
	    sisusb->HW_hres=320;
	    sisusb->HW_vres=240;	    		
		break;
		case MODE_800X480X8X60:
	    sisusb->ColorFormat= COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=800;
	    sisusb->vres=480;
	    sisusb->HW_hres=800;
	    sisusb->HW_vres=480;	    		
		break;
		case MODE_800X480X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=800;
	    sisusb->vres=480;	
	    sisusb->HW_hres=800;
	    sisusb->HW_vres=480;	    	
		break;
		case MODE_800X480X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=800;
	    sisusb->vres=480;	
	    sisusb->HW_hres=800;
	    sisusb->HW_vres=480;	    	
		break;
       case MODE_640X480X8X60:
       case MODE_640X480X8X75:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=640;
	    sisusb->vres=480;
	    sisusb->HW_hres=640;
	    sisusb->HW_vres=480;	    
       break;
       case MODE_640X480X16X60:
       case MODE_640X480X16X75:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=640;
	    sisusb->vres=480;
	    sisusb->HW_hres=640;
	    sisusb->HW_vres=480;	    
       break;
       case MODE_640X480X24X60:
       case MODE_640X480X24X75:
	    sisusb->ColorFormat =RGB24;
	    sisusb->bpp=24;
	    sisusb->hres=640;
	    sisusb->vres=480;
	    sisusb->HW_hres=640;
	    sisusb->HW_vres=480;	    
       break;
       case MODE_640X480X32X60:
	   case MODE_640X480X32X75:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=640;
	    sisusb->vres=480;
	    sisusb->HW_hres=640;
	    sisusb->HW_vres=480;	    
       break;
       case MODE_800X600X8X60:
       case MODE_800X600X8X75:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=800;
	    sisusb->vres=600;
	    sisusb->HW_hres=800;
	    sisusb->HW_vres=600;	    
       break;
       case MODE_800X600X16X60:
       case MODE_800X600X16X75:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=800;
	    sisusb->vres=600;
	    sisusb->HW_hres=800;
	    sisusb->HW_vres=600;	    
       break;
       case MODE_800X600X24X60:
       case MODE_800X600X24X75:
	    sisusb->ColorFormat =RGB24;
	    sisusb->bpp=24;
	    sisusb->hres=800;
	    sisusb->vres=600;
	    sisusb->HW_hres=800;
	    sisusb->HW_vres=600;	    
       break;       
       case MODE_800X600X32X60:
       case MODE_800X600X32X75:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=800;
	    sisusb->vres=600;
	    sisusb->HW_hres=800;
	    sisusb->HW_vres=600;	    
       break;
       case MODE_1024X768X8X60:
       case MODE_1024X768X8X75:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1024;
	    sisusb->vres=768;
	    sisusb->HW_hres=1024;
	    sisusb->HW_vres=768;	    
       break;
       case MODE_1024X768X16X60:
       case MODE_1024X768X16X75:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1024;
	    sisusb->vres=768;
	    sisusb->HW_hres=1024;
	    sisusb->HW_vres=768;	    
       break;
       case MODE_1024X768X24X60:
       case MODE_1024X768X24X75:
	    sisusb->ColorFormat =RGB24;
	    sisusb->bpp=24;
	    sisusb->hres=1024;
	    sisusb->vres=768;
	    sisusb->HW_hres=1024;
	    sisusb->HW_vres=768;	    
       break;
       case MODE_1024X768X32X60:
       case MODE_1024X768X32X75:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1024;
	    sisusb->vres=768;
	    sisusb->HW_hres=1024;
	    sisusb->HW_vres=768;	    
       break;
		case MODE_1152X864X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1152;
	    sisusb->vres=864;	
	    sisusb->HW_hres=1152;
	    sisusb->HW_vres=864;	    
		break;
		case MODE_1152X864X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1152;
	    sisusb->vres=864;		
	    sisusb->HW_hres=1152;
	    sisusb->HW_vres=864;
		break;
		case MODE_1152X864X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1152;
	    sisusb->vres=864;	
	    sisusb->HW_hres=1152;
	    sisusb->HW_vres=864;	    	
		break;       
		case MODE_1280X720X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1280;
	    sisusb->vres=720;		
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=720;	    
		break;
		case MODE_1280X720X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1280;
	    sisusb->vres=720;	
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=720;	    	
		break;
		case MODE_1280X720X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1280;
	    sisusb->vres=720;	
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=720;	    	
		break;
		case MODE_1280X768X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1280;
	    sisusb->vres=768;	
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=768;		    	
		break;
		case MODE_1280X768X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1280;
	    sisusb->vres=768;
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=768;	    		
		break;
		case MODE_1280X768X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1280;
	    sisusb->vres=768;	
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=768;	    	
		break;		
		case MODE_1280X800X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1280;
	    sisusb->vres=800;	
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=800;		    	
		break;
		case MODE_1280X800X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1280;
	    sisusb->vres=800;	
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=800;	    	
		break;
		case MODE_1280X800X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1280;
	    sisusb->vres=800;
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=800;	    		
		break;       
		case MODE_1280X960X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1280;
	    sisusb->vres=960;		
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=960;			    
		break;
		case MODE_1280X960X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1280;
	    sisusb->vres=960;	
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=960;	    	
		break;
		case MODE_1280X960X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1280;
	    sisusb->vres=960;
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=960;	    		
		break;		
       case MODE_1280X1024X8X60:
       case MODE_1280X1024X8X75:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1280;
	    sisusb->vres=1024;
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=1024;	    
       break;
       case MODE_1280X1024X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1280;
	    sisusb->vres=1024;
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=1024;	    
       break;
       case MODE_1280X1024X24X60:
	    sisusb->ColorFormat =RGB24;
	    sisusb->bpp=24;
	    sisusb->hres=1280;
	    sisusb->vres=1024;
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=1024;	    
       break;          
       case MODE_1280X1024X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1280;
	    sisusb->vres=1024;
	    sisusb->HW_hres=1280;
	    sisusb->HW_vres=1024;	    
       break; 
		case MODE_1360X768X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1360;
	    sisusb->vres=768;		
	    sisusb->HW_hres=1360;
	    sisusb->HW_vres=768;	    
		break;
		case MODE_1360X768X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1360;
	    sisusb->vres=768;	
	    sisusb->HW_hres=1360;
	    sisusb->HW_vres=768;	    	
		break;
		case MODE_1360X768X24X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=24;
	    sisusb->hres=1360;
	    sisusb->vres=768;	
	    sisusb->HW_hres=1360;
	    sisusb->HW_vres=768;	    	
		break;      
		case MODE_1360X768X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1360;
	    sisusb->vres=768;	
	    sisusb->HW_hres=1360;
	    sisusb->HW_vres=768;	    	
		break;       
		case MODE_1366X768X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1366;
	    sisusb->vres=768;
	    sisusb->HW_hres=1366;
	    sisusb->HW_vres=768;	    		
		break;
		case MODE_1366X768X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1366;
	    sisusb->vres=768;	
	    sisusb->HW_hres=1366;
	    sisusb->HW_vres=768;	    	
		break;
		case MODE_1366X768X24X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=24;
	    sisusb->hres=1366;
	    sisusb->vres=768;	
	    sisusb->HW_hres=1366;
	    sisusb->HW_vres=768;	    	
		break;
		case MODE_1366X768X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1366;
	    sisusb->vres=768;	
	    sisusb->HW_hres=1366;
	    sisusb->HW_vres=768;	    	
		break;
		case MODE_1400X1050X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1400;
	    sisusb->vres=1050;	
	    sisusb->HW_hres=1408;
	    sisusb->HW_vres=1050;		    	
		break;
		case MODE_1400X1050X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1408;
	    sisusb->vres=1050;	
	    sisusb->HW_hres=1400;
	    sisusb->HW_vres=1050;	    	
		break;
		case MODE_1400X1050X24X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=24;
	    sisusb->hres=1408;
	    sisusb->vres=1050;	
	    sisusb->HW_hres=1408;
	    sisusb->HW_vres=1050;	    	
		break;
		case MODE_1400X1050X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1408;
	    sisusb->vres=1050;
	    sisusb->HW_hres=1408;
	    sisusb->HW_vres=1050;	    		
		break;       
       case MODE_1440X900X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1440;
	    sisusb->vres=900;
	    sisusb->HW_hres=1440;
	    sisusb->HW_vres=900;	    
       break;
       case MODE_1440X900X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1440;
	    sisusb->vres=900;
	    sisusb->HW_hres=1440;
	    sisusb->HW_vres=900;	    
       break;      
		case MODE_1440X900X24X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=24;
	    sisusb->hres=1440;
	    sisusb->vres=900;
	    sisusb->HW_hres=1440;
	    sisusb->HW_vres=900;	    
       break;     
       case MODE_1440X900X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1440;
	    sisusb->vres=900;
	    sisusb->HW_hres=1440;
	    sisusb->HW_vres=900;	    
       break;
       case MODE_1600X1200X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1600;
	    sisusb->vres=1200;
	    sisusb->HW_hres=1600;
	    sisusb->HW_vres=1200;	    
       break;
       case MODE_1600X1200X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1600;
	    sisusb->vres=1200;
	    sisusb->HW_hres=1600;
	    sisusb->HW_vres=1200;	    
       break;       
		case MODE_1600X1200X24X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=24;
	    sisusb->hres=1600;
	    sisusb->vres=1200;
	    sisusb->HW_hres=1600;
	    sisusb->HW_vres=1200;	    
       break;   
       case MODE_1600X1200X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1600;
	    sisusb->vres=1200;
	    sisusb->HW_hres=1600;
	    sisusb->HW_vres=1200;	    
       break;              
       case MODE_1680X1050X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1680;
	    sisusb->vres=1050;
	    sisusb->HW_hres=1680;
	    sisusb->HW_vres=1050;	    
       break;
       case MODE_1680X1050X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1680;
	    sisusb->vres=1050;
	    sisusb->HW_hres=1680;
	    sisusb->HW_vres=1050;	    
       break;  
		case MODE_1680X1050X24X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=24;
	    sisusb->hres=1680;
	    sisusb->vres=1050;
	    sisusb->HW_hres=1680;
	    sisusb->HW_vres=1050;	    
       break;         
       case MODE_1680X1050X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1680;
	    sisusb->vres=1050;
	    sisusb->HW_hres=1680;
	    sisusb->HW_vres=1050;	    
       break;  
       case MODE_1920X1080X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1920;
	    sisusb->vres=1080;
	    sisusb->HW_hres=1920;
	    sisusb->HW_vres=1080;	    
       break;
       case MODE_1920X1080X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1920;
	    sisusb->vres=1080;
	    sisusb->HW_hres=1920;
	    sisusb->HW_vres=1080;	    
       break;       
		case MODE_1920X1080X24X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=24;
	    sisusb->hres=1920;
	    sisusb->vres=1080;
	    sisusb->HW_hres=1920;
	    sisusb->HW_vres=1080;	    
       break;    
       case MODE_1920X1080X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1920;
	    sisusb->vres=1080;
	    sisusb->HW_hres=1920;
	    sisusb->HW_vres=1080;	    
       break;    
       case MODE_1920X1200X8X60:
	    sisusb->ColorFormat =COLOR_PALETTE;
	    sisusb->bpp=8;
	    sisusb->hres=1920;
	    sisusb->vres=1200;
	    sisusb->HW_hres=1920;
	    sisusb->HW_vres=1200;	    
       break;
       case MODE_1920X1200X16X60:
	    sisusb->ColorFormat =RGB16;
	    sisusb->bpp=16;
	    sisusb->hres=1920;
	    sisusb->vres=1200;
	    sisusb->HW_hres=1920;
	    sisusb->HW_vres=1200;	    
       break;       
		case MODE_1920X1200X24X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=24;
	    sisusb->hres=1920;
	    sisusb->vres=1200;
	    sisusb->HW_hres=1920;
	    sisusb->HW_vres=1200;	    
       break;   
       case MODE_1920X1200X32X60:
	    sisusb->ColorFormat =RGB32;
	    sisusb->bpp=32;
	    sisusb->hres=1920;
	    sisusb->vres=1200;
	    sisusb->HW_hres=1920;
	    sisusb->HW_vres=1200;	    
       break;                        
       default:
       return 0;
    }
    return 1;
} 


unsigned char SetVGATiming (struct sisusb_usb_data *sisusb ,unsigned char FirstTimePnP ,unsigned long modeIndex)
{
    unsigned char Cmd[100];
    unsigned char i;
	int ret;
    	
	Cmd[0] = 0x04;
	Cmd[2] = 0;
	switch (modeIndex)
	{
		//new mode
	case MODE_320X240X8X60:
	case MODE_320X240X16X60:
	case MODE_320X240X32X60:
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
    	Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x04;
		//WriteRegisterByte(DevExt, 0xFCF1,0x13);	
    	Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x05;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x00);	
    	Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x00;		
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x01);	6MHz
    	Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF4,0x01);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFCF6,0x0a);
    	Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x15);
    	Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x01;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
    	Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
    	Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i=3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
    	Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x81);
		if (FirstTimePnP)
		{
    		Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;		
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x80);
    	Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
    	Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
    	Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;	    		
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
    	Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    	
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
    	Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;    		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0x7f);  /// HGATE    : 320
    	Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x3f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x02);	  /// HGATE
    	Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x2c);	  /// HGATEDEL :	40 -4
    	Cmd[i++] = 0xc6;
    	Cmd[i++] = 0x24;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x5f);	  /// HSYNC    : 	32
    	Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x1f;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x1f);	  /// HLEN     : 400
    	Cmd[i++] = 0xce;
    	Cmd[i++] = 0x8f;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x03);	  /// HLEN
    	Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x01;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xdf);	 /// VGATE    :	240
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xef;
		//WriteRegisterByte(DevExt, 0xFCC9,0x01);	 /// VGATE	
    	Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	6
    	Cmd[i++] = 0xca;
    	Cmd[i++] = 0x05;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x01);	 /// VSYNC    :	4
    	Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x03;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x0c);	 /// VLEN     :	253
    	Cmd[i++] = 0xcc;
    	Cmd[i++] = 0xfc;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x02);	 /// VLEN
    	Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x00;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;
    	    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
	break;
	case MODE_800X480X8X60:
	case MODE_800X480X16X60:
	case MODE_800X480X32X60:
	{
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);	//99
    	Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x04;
		//WriteRegisterByte(DevExt, 0xFCF1,0x21);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x19;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x00);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x00;		
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x01);	38.5MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x18;		
		//WriteRegisterByte(DevExt, 0xFCF4,0x01);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x0b;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x03);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x0a);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x07;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		

		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	ret = SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
				
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x81);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x80);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00; 			
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;
    	
		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0x1f);  /// HGATE    : 800
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x1f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x54);	  /// HGATEDEL :	112
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0x6D;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x7f);	  /// HSYNC    : 80
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x4F;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x1f);	  /// HLEN     : 1032
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x07;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x04);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x04;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0x57);	 /// VGATE    :	480
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xDF;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x14);	 /// VGATEDEL :	78
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x4D;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x03);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x73);	 /// VLEN     :	622
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x6D;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x02);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x02;

    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
	}
	break;
	case MODE_1280X800X8X60:
	case MODE_1280X800X16X60:
	case MODE_1280X800X32X60:
	{
    	i = 3;
//    	if (!ReadRegistryAndWriteRister (DevExt, Reg_InitPLL_1280x800x60))
//    	{
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);	//99
    	Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFCF1,0x21);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x00);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;	
//    	}	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x01);	83.46MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x16;		
		//WriteRegisterByte(DevExt, 0xFCF4,0x01);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x33;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x03);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x1;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x0a);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x03;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x81);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x80);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;	
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
    	Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		    			
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    	
    	
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
    	Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00; 		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0x1f);  /// HGATE    : 1280
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x54);	  /// HGATEDEL :	200
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xc4;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x7f);	  /// HSYNC    : 136
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x87;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x1f);	  /// HLEN     : 1680
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x8F;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x04);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x06;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0x57);	 /// VGATE    :	800
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0x1F;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x14);	 /// VGATEDEL :	24 , FP can't less than 3
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x13;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x03);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x73);	 /// VLEN     :	828
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x3B;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x02);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x03;

    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;
    	    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
	}
	break;	
	//===========================
	case MODE_640X480X8X60:
	case MODE_640X480X16X60:
	case MODE_640X480X24X60:
	case MODE_640X480X32X60:
    {
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
    	Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFCF1,0x13);	
    	Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x00);	
    	Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;		
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x01);	25.175MHz
    	Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF4,0x01);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFCF6,0x0a);
    	Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x0a;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x15);
    	Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x15;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
    	Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
    	Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i=3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
    	Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x81);
		if (FirstTimePnP)
		{
    		Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;		
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x80);
    	Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
    	Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
    	Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
    	Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
    	Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0x7f);  /// HGATE    : 640
    	Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x7f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x02);	  /// HGATE
    	Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x2c);	  /// HGATEDEL :	48 -4
    	Cmd[i++] = 0xc6;
    	Cmd[i++] = 0x2C;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x5f);	  /// HSYNC    : 	96
    	Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x5f;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x1f);	  /// HLEN     : 800
    	Cmd[i++] = 0xce;
    	Cmd[i++] = 0x1f;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x03);	  /// HLEN
    	Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x03;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xdf);	 /// VGATE    :	480
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xdf;
		//WriteRegisterByte(DevExt, 0xFCC9,0x01);	 /// VGATE	
    	Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	33
    	Cmd[i++] = 0xca;
    	Cmd[i++] = 0x20;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x01);	 /// VSYNC    :	2
    	Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x01;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x0c);	 /// VLEN     :	525
    	Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x0c;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x02);	 /// VLEN
    	Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x02;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x03;
    	    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);    
    }
	break;
	case MODE_640X480X8X75:
	case MODE_640X480X16X75:
	case MODE_640X480X24X75:
	case MODE_640X480X32X75:
    {
    	// Enable the PLL
    	//SDRAM
    	i=3;
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x19);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x00);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;		
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x01);	31.5MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF4,0x01);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x08);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x08;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x15);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x15;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x81);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x80);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    		
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4,0x7f);	 /// HGATE	640
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x7f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x02);	 /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x50);	 /// HGATEDEL	: 120
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0x74;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x2f);	 /// HSYNC	: 64
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x3f;		
		
		//WriteRegisterByte(DevExt, 0xFCCE,0x1f);	// HLEN	:	840
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x47;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x03);
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x03;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xdf);	 /// VGATE	:	480
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xdf;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x01);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x10);	 /// VGATEDEL	:	16
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x0F;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x01);	/// VSYNC	: 3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;		
		
		//WriteRegisterByte(DevExt, 0xFCCC,0x0c);	// VLEN	500
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0xF3;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x02);
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x01;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x03;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
	break;
	case MODE_800X600X8X60:
	case MODE_800X600X16X60:
	case MODE_800X600X24X60:
	case MODE_800X600X32X60:	
    {

		printk("%s: gSetMode: %d\n", __FUNCTION__, gSetMode); 
		
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);	//99
    	Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFCF1,0x21);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x00);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;		
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x01);	40MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF4,0x01);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x03);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x0a);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x0a;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		

		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data				
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	ret = SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms

		if(ret < 0){

			printk("%s: 800x600_60: timing phase 1 failed\n", __FUNCTION__);
			return 0;

		}			

		printk("%s: gSetMode: %d\n", __FUNCTION__, gSetMode); 
		
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x81);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x80);
		//else{ // WHS: try
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x66;
		//}
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    		
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0x1f);  /// HGATE    : 800
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x1f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x54);	  /// HGATEDEL :	88
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0x54;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x7f);	  /// HSYNC    : 128
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x7f;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x1f);	  /// HLEN     : 1056
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x1f;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x04);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x04;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0x57);	 /// VGATE    :	600
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0x57;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x14);	 /// VGATEDEL :	23
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x16;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x03);	 /// VSYNC    :	4
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x03;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x73);	 /// VLEN     :	628
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x73;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x02);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x02;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	ret = SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	

		if(ret < 0){

			printk("%s: 800x600_60: timing phase 2 failed\n", __FUNCTION__);
			return 0;

		}			

		printk("%s: gSetMode: %d\n", __FUNCTION__, gSetMode); 
		
	}
	break;
	case MODE_800X600X8X75:
	case MODE_800X600X16X75:
	case MODE_800X600X24X75:
	case MODE_800X600X32X75:	
    {
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x29);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x00);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;		
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x01);	49.5 MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x08;		
		//WriteRegisterByte(DevExt, 0xFCF4,0x01);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x0b;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x06);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x19);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x03;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data				
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;		
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x81);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x80);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    		
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0x1f);  /// HGATE    : 800
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x1f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x3c);	  /// HGATEDEL :	160
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0x9F;	//0x3c;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x77);	  /// HSYNC    : 80
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x4F;	//0x77;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x0f);	  /// HLEN     : 1056
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x1F;	//0x0f;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x04);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x04;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0x57);	 /// VGATE    :	600
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0x57;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x14);	 /// VGATEDEL :	21	
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x14;	//0x13;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;	//0x05;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x99);	 /// VLEN     :	625
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x70;	//0x99;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x02);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x02;		
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;
    	    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
	break;
	case MODE_1024X768X8X60:
	case MODE_1024X768X16X60:
	case MODE_1024X768X24X60:
	case MODE_1024X768X32X60:	
    {
    	i = 3;
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    

		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	65MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x0c;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x0d;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x05;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
		Cmd[i++] = 0x00;	

		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data			

		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
		Cmd[i++] = 0x07;
		Cmd[1] = i-3;
//printk("%s: ==1==\n", __FUNCTION__);
		SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);    
		//MyDelay (15);  //10ms
		mdelay (15); // WHS: try
		i = 3;
//		if (!ReadRegistryAndWriteRister (DevExt, Reg_InitSDRAM_1024x768x60))
//		{
		
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		//else{ // WHS: try
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x66;
		//}
    	
//		}		
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1024
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	160
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0x9D;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 136
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x87;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     : 1344
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x3F;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x05;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	768
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	29
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x1C;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	6
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x05;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	806
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x25;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x03;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x03;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
	break;
	case MODE_1024X768X8X75:
	case MODE_1024X768X16X75:
	case MODE_1024X768X24X75:
	case MODE_1024X768X32X75:	
    {
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x39);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;				
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;		
    	
		// for testing
		//Cmd[i++] = 0xfa;
		//Cmd[i++] = 0x01;
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x04);	78.75MHz
		Cmd[i++] = 0xf3;
    	//Cmd[i++] = 0x04;	
    	Cmd[i++] = 0x08;
		//WriteRegisterByte(DevExt, 0xFCF4,0x15);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x15;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x04);
		Cmd[i++] = 0xf6;
    	//Cmd[i++] = 0x04;
    	Cmd[i++] = 0x02;	
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x05;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
	
		i = 3;
//		if (!ReadRegistryAndWriteRister (DevExt, Reg_InitSDRAM_1024x768x75))
//		{
		
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
    	
//		}
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    	
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1024
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x9f);	  /// HGATEDEL :	176
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xAD;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x5f);	  /// HSYNC    :  	96
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x5D;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x10);	  /// HLEN     : 1312
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x1f;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x05;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	768
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	28
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x1B;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x01);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	800
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x1F;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x03;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;
    	    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
	break;
	case MODE_1280X1024X8X60:
	case MODE_1280X1024X16X60:
	case MODE_1280X1024X24X60:
	case MODE_1280X1024X32X60:	
    {
    	
    	i = 3;
//    	if (!ReadRegistryAndWriteRister (DevExt, Reg_InitPLL_1280x1024x60))
//    	{
   	
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x39);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;	
   		
//		}
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x03);	108MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF4,0x0b);
		Cmd[i++] = 0xf4;
    	// Cmd[i++] = 0x03;		
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	// Cmd[i++] = 0x03;		
    	Cmd[i++] = 0x09;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x11;		

		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
//		if (!ReadRegistryAndWriteRister (DevExt, Reg_InitSDRAM_1280x1024x60))
//		{
		
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
   	
//		}
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1280
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x04);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCC6,0xff);	  /// HGATEDEL :	248
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xf5;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x78);	  /// HSYNC    : 112
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x6f;		

		//WriteRegisterByte(DevExt, 0xFCCE,0xbe);	  /// HLEN     : 1688
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x97;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x06);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x06;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	1024
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x03);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x10);	 /// VGATEDEL :	38
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x25;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x04);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x20);	 /// VLEN     :	1066
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x29;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x04);	 /// VLEN    	
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x04;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
	break;
	case MODE_1280X1024X8X75:
	case MODE_1280X1024X16X75:
	case MODE_1280X1024X24X75:
	case MODE_1280X1024X32X75:	
    {
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x39);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;		
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x02);	// 135MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF4,0x09);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x2d;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x01;		

		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data					
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;		
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM		
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1280
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x04);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCC6,0xef);	  /// HGATEDEL :	248
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xF4;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x78);	  /// HSYNC    : 144
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x8F;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x97);	  /// HLEN     : 1688
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x97;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x06);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x06;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	1024
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x03);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x20);	 /// VGATEDEL :	38
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x22;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x02);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x37);	 /// VLEN     :	1066
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x29;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x04);	 /// VLEN 
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x04;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;
    	    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
	break;
	case MODE_1440X900X8X60:
	case MODE_1440X900X16X60:
	case MODE_1440X900X24X60:
	case MODE_1440X900X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
	#if V12
	//aaa
	Cmd[i++] = 0x14;
	#else
    	Cmd[i++] = 0x15;		
	#endif
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	106.4MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x0f;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x13;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x07;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x01;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x87;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x86;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1440
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x9f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x05;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	232
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xE5;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 152
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x97;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     : 1904
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x6f;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x07;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	900
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0x83;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	24 
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x17;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x03;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	932
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0xA3;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x03;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
    break;	
	case MODE_1600X1200X8X60:
	case MODE_1600X1200X16X60:
	case MODE_1600X1200X24X60:
	case MODE_1600X1200X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
	#if V12
	Cmd[i++] = 0x14;
	#else
    	Cmd[i++] = 0x15;		
	#endif
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	130.25MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x06;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x0d;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x05;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x01;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x87;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x86;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1600
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x3f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x06;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	80
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0x4d;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 32
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x1F;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     : 1760
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0xdf;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x06;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	1200
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xAF;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	28
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x1B;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	4
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x04;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	1235
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0xD2;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x04;

    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
    break;	
	case MODE_1680X1050X8X60:
	case MODE_1680X1050X16X60:
	case MODE_1680X1050X24X60:	
	case MODE_1680X1050X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
	#if V12
	Cmd[i++] = 0x14;
	#else
    	Cmd[i++] = 0x15;		
	#endif
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	146MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x05;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x08;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x05;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x26;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x01;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x87;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x86;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1680
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x8f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x06;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x78;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     : 
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0xb2;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x08;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	1050
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0x19;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	29
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x10;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	6
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x04;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	806
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x40;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x04;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
    break;
	case MODE_1920X1080X8X60:
	case MODE_1920X1080X16X60:
	case MODE_1920X1080X24X60:
	case MODE_1920X1080X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x16;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	148.5MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x04;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x09;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x0B;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x01;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x97;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x96;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1680
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x7f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x07;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0x6f;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x48;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     : 
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x98;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x08;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	1050
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0x37;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	29
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x10;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	6
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x04;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	806
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x65;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x04;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
    	break;

	case MODE_1152X864X8X60:
	case MODE_1152X864X16X60:
	case MODE_1152X864X24X60:	
	case MODE_1152X864X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	81.6MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x05;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x11;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x02;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1152
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x7f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	184
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xb5;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 120
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x77;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     :  1520
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0xef;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x05;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	864
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0x5f;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	27
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x18;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	895
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x7e;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x03;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    }
    	break;  
	case MODE_1280X720X8X60:
	case MODE_1280X720X16X60:
	case MODE_1280X720X24X60:
	case MODE_1280X720X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	74.5MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x35;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x2f;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x07;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		//MyDelay (10);  //10ms
		mdelay (10); // WHS: try
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1280
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	192
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xbd;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 136
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x87;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     :  1664
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x7f;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x06;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	720
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xcf;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	20 (real 22)
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x13;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	746
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0xe9;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x02;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    	}
    	break;     	
	case MODE_1280X768X8X60:
	case MODE_1280X768X16X60:
	case MODE_1280X768X24X60:
	case MODE_1280X768X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	79.5MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x02;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x35;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x10;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		MyDelay (10);  //10ms   	
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1280
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	192
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xbd;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 128
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x7F;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     :  1664
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x7f;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x06;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	768
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	20
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x13;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	7
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x06;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	798
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x1d;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x03;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    	}
    	break;    	  	
	case MODE_1280X960X8X60:
	case MODE_1280X960X16X60:
	case MODE_1280X960X24X60:	
	case MODE_1280X960X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x12;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	108MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x03;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x01;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		MyDelay (10);  //10ms   	
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x67;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x66;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1280
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	312 > 255
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 112
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x6F;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     :  1800
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x07;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x07;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	960
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xbf;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x03;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	34 (real 36)
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x21;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	3
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x02;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	1000
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0xe7;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x03;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    	}
    	break; 	    	
	case MODE_1366X768X8X60:
	case MODE_1366X768X16X60:
	case MODE_1366X768X24X60:
	case MODE_1366X768X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x15;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	87.75 MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x10;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x27;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x03;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		MyDelay (10);  //10ms   	
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x87;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x86;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1366
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x5f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x05;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	216
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xd5;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 184
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0xb7;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     :  1806
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x0d;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x07;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	768
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	29
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x1c;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	6
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x05;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	806
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x25;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x03;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	 	
    	}
    	break;   
	case MODE_1360X768X8X60:
	case MODE_1360X768X16X60:
	case MODE_1360X768X24X60:
	case MODE_1360X768X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
    	Cmd[i++] = 0x15;		
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	85.5 MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x08;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x13;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x03;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x00;		
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		MyDelay (10);  //10ms   	
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x87;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x86;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1360
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x4f;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x05;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	256
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xFD;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 112
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x6F;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     :  1792
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x06;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	768
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0xff;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x02;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	18
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x11;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	6
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x05;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	795
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x1a;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x03;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x00;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	 	  	
    	}
    	break;    	 
	case MODE_1400X1050X8X60:
	case MODE_1400X1050X16X60:
	case MODE_1400X1050X24X60:
	case MODE_1400X1050X32X60:	
    {   	
    	i = 3;
    	// Enable the PLL
    	//SDRAM
    	//WriteRegisterByte(DevExt, 0xFCF0,0x04);
		Cmd[i++] = 0xf0;
    	Cmd[i++] = 0x01;    	
		//WriteRegisterByte(DevExt, 0xFCF1,0x35);	
		Cmd[i++] = 0xf1;
	#if V12
	Cmd[i++] = 0x14;
	#else
    	Cmd[i++] = 0x15;		
	#endif
		// range
		//WriteRegisterByte(DevExt, 0xFCF2,0x01);	
		Cmd[i++] = 0xf2;
    	Cmd[i++] = 0x01;
	
		// VGA
		//WriteRegisterByte(DevExt, 0xFCF3,0x06);	121.75 MHz
		Cmd[i++] = 0xf3;
    	Cmd[i++] = 0x29;
		//WriteRegisterByte(DevExt, 0xFCF4,0x0d);
		Cmd[i++] = 0xf4;
    	Cmd[i++] = 0x34;		
		//WriteRegisterByte(DevExt, 0xFCF6,0x02);
		Cmd[i++] = 0xf6;
    	Cmd[i++] = 0x01;		
		//WriteRegisterByte(DevExt, 0xFCF7,0x05);
		Cmd[i++] = 0xf7;
    	Cmd[i++] = 0x08;		
		
		//WriteRegisterByte(DevExt, 0xFCF5,0x00);
		Cmd[i++] = 0xf5;
    	Cmd[i++] = 0x01; 	
		
		// waiting the PLL stable time (counter setting)
		Cmd[i++] = 0x4c;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4d;  /// address
		Cmd[i++] = 0x80; /// data

		Cmd[i++] = 0x4e;  /// address
		Cmd[i++] = 0x00; /// data
		Cmd[i++] = 0x4f;  /// address
		Cmd[i++] = 0x80; /// data		
		// clock enable
		//WriteRegisterByte(DevExt, 0xFC4b,0x07);
		Cmd[i++] = 0x4b;
    	Cmd[i++] = 0x07;
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
		MyDelay (10);  //10ms   	
		
		i = 3;
		// initial SDRAM
		//WriteRegisterByte(DevExt, 0xFCA4,0x37);
		Cmd[i++] = 0xa4;
    	Cmd[i++] = 0x39;		
		//WriteRegisterByte(DevExt, 0xFCA3,0x83);
		if (FirstTimePnP)
		{
			Cmd[i++] = 0xa3;
    		Cmd[i++] = 0x87;
    	}
		//WriteRegisterByte(DevExt, 0xFCA3,0x82);
		Cmd[i++] = 0xa3;
    	Cmd[i++] = 0x86;
	
		// Clear the buffer
		//WriteRegisterByte(DevExt, 0xFC28,0x01);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x01;
		//WriteRegisterByte(DevExt, 0xFC28,0x00);
		Cmd[i++] = 0x28;
    	Cmd[i++] = 0x00;
		//WriteRegisterByte(DevExt, 0xFC32,0x00);
		Cmd[i++] = 0x32;
    	Cmd[i++] = 0x00;		
		//WriteRegisterByte(DevExt, 0xFC59,0x01);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x01;	
		//WriteRegisterByte(DevExt, 0xFC59,0x00);
    	Cmd[i++] = 0x59;
    	Cmd[i++] = 0x00;    			
		//WriteRegisterByte(DevExt, 0xFC34,0x00);
		Cmd[i++] = 0x34;
    	Cmd[i++] = 0x00;		

		/// Horizontal
		//WriteRegisterByte(DevExt, 0xFCC4, 0xff);  /// HGATE    : 1408 ????
		Cmd[i++] = 0xc4;
    	Cmd[i++] = 0x7F;		
		//WriteRegisterByte(DevExt, 0xFCC5,0x03);	  /// HGATE
		Cmd[i++] = 0xc5;
    	Cmd[i++] = 0x05;		
		//WriteRegisterByte(DevExt, 0xFCC6,0x96);	  /// HGATEDEL :	232
		Cmd[i++] = 0xc6;
    	Cmd[i++] = 0xe5;		
		//WriteRegisterByte(DevExt, 0xFCC7,0x87);	  /// HSYNC    : 144
		Cmd[i++] = 0xc7;
    	Cmd[i++] = 0x8F;		

		//WriteRegisterByte(DevExt, 0xFCCE,0x3a);	  /// HLEN     :  1864
		Cmd[i++] = 0xce;
    	Cmd[i++] = 0x47;		
		//WriteRegisterByte(DevExt, 0xFCCF,0x05);	  /// HLEN
		Cmd[i++] = 0xcf;
    	Cmd[i++] = 0x07;		

		/// Vertical
		//WriteRegisterByte(DevExt, 0xFCC8,0xff);	 /// VGATE    :	1050
		Cmd[i++] = 0xc8;
    	Cmd[i++] = 0x19;		
		//WriteRegisterByte(DevExt, 0xFCC9,0x02);	 /// VGATE
		Cmd[i++] = 0xc9;
    	Cmd[i++] = 0x04;		
		//WriteRegisterByte(DevExt, 0xFCCa,0x1d);	 /// VGATEDEL :	32
		Cmd[i++] = 0xca;
    	Cmd[i++] = 0x1F;		
		//WriteRegisterByte(DevExt, 0xFCCb,0x05);	 /// VSYNC    :	4
		Cmd[i++] = 0xcb;
    	Cmd[i++] = 0x03;		

		//WriteRegisterByte(DevExt, 0xFCCC,0x2a);	 /// VLEN     :	1089
		Cmd[i++] = 0xcc;
    	Cmd[i++] = 0x40;		
		//WriteRegisterByte(DevExt, 0xFCCD,0x03);	 /// VLEN
		Cmd[i++] = 0xcd;
    	Cmd[i++] = 0x04;
    	
    	//for trigger 2 to set HSync and VSync porarity 
    	Cmd[i++] = 0x6F;
    	Cmd[i++] = 0x01;    	
    	Cmd[1] = i-3;
    	SendBlockCommand(sisusb,3,
                          Cmd,
                          0,      //users or kernels buffer.
                          i);     	
    	}
    	break;     		
    	default:
    		return 0;    	
	}	// end of switch
	return 1;
}	

void VgaModeSetting_DDR(struct sisusb_usb_data *sisusb, u32 modeIndex)
{
	unsigned long OutVRes, OutHRes, Refresh, ColorFormat;
    unsigned char ColorInfo;
    unsigned char Cmd1[50];
    unsigned char  i, FCA3, FirstTimePnP;
	int ret;

    printk("%s: VGA Set mode DDR =%x\n", __FUNCTION__, modeIndex);
    
	if (!SetModeInfo (sisusb, modeIndex)){
		
		printk("%s: SetModeInfo failed\n", __FUNCTION__);	
		return;

	}
	
	OutHRes = sisusb->HW_hres;
	OutVRes = sisusb->HW_vres;

	ColorFormat = sisusb->ColorFormat;
	ReadRegisterByte (sisusb, 0xFCA3, &FCA3);
	printk("%s: 0xFCA3&0x06 = %x\n", __FUNCTION__, FCA3&0x06);
	if ((FCA3&0x06) == 0x06)
		FirstTimePnP = 0;
	else
		FirstTimePnP = 1;
		
	printk("%s: 0xFCA3 = %x\n", __FUNCTION__, FCA3);

	//if(FirstTimePnP){

// trigger2: fix small block update
    if (sisusb->hres > 1280)
    {
    	WriteRegisterByte(sisusb, 0xFC2F,0x03);
    	WriteRegisterByte(sisusb, 0xFBF6,0x02);
    }
    else
    {
    	WriteRegisterByte(sisusb, 0xFC2F,0x00);
    	WriteRegisterByte(sisusb, 0xFBF6,0x00);
    }
    
     //yulw: 0512 for trigger 2 clear buffer
    WriteRegisterByte(sisusb, 0xFC28,0x01); 
    WriteRegisterByte(sisusb, 0xFC59,0x01); 
    WriteRegisterByte(sisusb, 0xFC32,0x00);
    WriteRegisterByte(sisusb, 0xFC34,0x00);
    
    //     
    WriteRegisterByte(sisusb, 0xFCB0,0);    //disable.
    WriteRegisterByte(sisusb, 0xFCB0,0x20);
    // Disable PLL
	WriteRegisterByte(sisusb, 0xFC4b,0x00);
	
	//yulw: 0512 for trigger2 
	printk("%s: gSetMode: %d\n", __FUNCTION__, gSetMode); 
	//MyDelay (15);	
	mdelay (15); // WHS: try
	
	
	//for Trigger2: DDR enable DDR
	
	WriteRegisterByte(sisusb, 0xFBF2,0x04);
	WriteRegisterByte(sisusb, 0xFBF4,0x03);

	//}

	printk("%s: gSetMode: %d\n", __FUNCTION__, gSetMode); 
	
	//MyDelay (15); // WHS: try
	mdelay (15);
	if (!SetVGATiming (sisusb, FirstTimePnP, modeIndex)) {
		printk("SetVGATiming return Fail\n"); 
		return;
	}

	printk("SetVGATiming done\n"); 
	
     /// 640x480x60
	Cmd1[0] = 0x04;
	Cmd1[2] = 0;

	//DevExt->bFirstTimePnP = FALSE;
	
	i = 3;
	// scaling factor
    //WriteRegisterByte(DevExt, 0xFC40,0);    //low
	Cmd1[i++] = 0x40;
   	Cmd1[i++] = 0x00;    
    //WriteRegisterByte(DevExt, 0xFC41,0x40); //high
	Cmd1[i++] = 0x41;
   	Cmd1[i++] = 0x40;    
    //WriteRegisterByte(DevExt, 0xFC42,0);    //low
	Cmd1[i++] = 0x42;
   	Cmd1[i++] = 0x00;    
    //WriteRegisterByte(DevExt, 0xFC43,0x40); //high
	Cmd1[i++] = 0x43;
   	Cmd1[i++] = 0x40;    

	// resolution
    //WriteRegisterByte(DevExt, 0xFC36,(UCHAR)((OutHRes)&0xff));	   //low
	Cmd1[i++] = 0x36;
   	Cmd1[i++] = (unsigned char)((OutHRes)&0xff);    
    //WriteRegisterByte(DevExt, 0xFC37,(UCHAR)((OutHRes>>8)&0xff)); //high
	Cmd1[i++] = 0x37;
	Cmd1[i++] = (unsigned char)((OutHRes>>8)&0xff);
    //WriteRegisterByte(DevExt, 0xFC38,(UCHAR)((OutVRes)&0xff));	   //low
	Cmd1[i++] = 0x38;
	Cmd1[i++] = (unsigned char)((OutVRes)&0xff);
    //WriteRegisterByte(DevExt, 0xFC39,(UCHAR)((OutVRes>>8)&0xff)); //high
	Cmd1[i++] = 0x39;
	Cmd1[i++] = (unsigned char)((OutVRes>>8)&0xff);
     
    // color format 
    if(ColorFormat == YUV444)	 /// YUV444
		ColorInfo = 0x08;	 /// YUV444 enable
    else if(ColorFormat == RGB32)	 /// 32-bit mode
		ColorInfo = 0x0e;	 /// 32-bit enable
    else if(ColorFormat == RGB565)	 /// 16-bit mode BGR565
		ColorInfo=0x04;		 /// BGR565 enable
    else if(ColorFormat == YUV422)	 /// 16-bit mode YUV422
		ColorInfo = 0x8a;	 /// YUV422 enable
	else if (ColorFormat == RGB24)
		ColorInfo = 0x00;
	else if (ColorFormat == YUV411)
		ColorInfo = 0x96;			
	else 
		ColorInfo = 0x04;
    //WriteRegisterByte(DevExt, 0xFC34, ColorInfo);
    Cmd1[i++] = 0x34;
    Cmd1[i++] = ColorInfo;
    Cmd1[1] = i-3;
    ret = SendBlockCommand(sisusb,3,
                          Cmd1,
                          0,      //users or kernels buffer.
                          i);	
	if(ret < 0){

		printk("%s: format phase error\n", __FUNCTION__); 
		
	}

    if(ColorFormat == COLOR_PALETTE)	 /// 8-bit mode	BIT-Map	enable
		WriteRegisterByte(sisusb, 0xFC32,0x02);	 /// Bit Map Fill Start
    else
		WriteRegisterByte(sisusb, 0xFC32,0x00);	 /// Bit Map Fill Start
	if (1)
	{
		unsigned long ScreenSize;
		unsigned long Overlay0Addr, Overlay1Addr;
		
		ScreenSize = sisusb->HW_hres * sisusb->HW_vres *3;
#if 1		
		Overlay0Addr = ScreenSize>>2; //Dword alignment DDR
		Overlay1Addr = ScreenSize*2>>2; //Dword alignment DDR		
#else		
		Overlay0Addr = ScreenSize>>1; //word alignment
		Overlay1Addr = ScreenSize*2>>1; //word alignment
#endif		
		
		Cmd1[0] = 0x22;
		Cmd1[1] = 8;
		i=2;
		//set OverlayFrame 0 base
		Cmd1[i++] = 0xFB;
		Cmd1[i++] = 0x65;
		Cmd1[i++] = 0x01;
		Cmd1[i++] = 0xFB;
		Cmd1[i++] = 0x62;
		Cmd1[i++] = (unsigned char) Overlay0Addr;
		Cmd1[i++] = 0xFB;
		Cmd1[i++] = 0x63;
		Cmd1[i++] = (unsigned char)(Overlay0Addr>>8);
		Cmd1[i++] = 0xFB;
		Cmd1[i++] = 0x64;
		Cmd1[i++] = (unsigned char)(Overlay0Addr>>16);
		//set OverlayFrame 1 base
		Cmd1[i++] = 0xFB;
		Cmd1[i++] = 0x65;
		Cmd1[i++] = 0x02;
		Cmd1[i++] = 0xFB;
		Cmd1[i++] = 0x62;
		Cmd1[i++] = (unsigned char) Overlay1Addr;
		Cmd1[i++] = 0xFB;
		Cmd1[i++] = 0x63;
		Cmd1[i++] = (unsigned char)(Overlay1Addr>>8);
		Cmd1[i++] = 0xFB;
		Cmd1[i++] = 0x64;
		Cmd1[i++] = (unsigned char)(Overlay1Addr>>16);		
		
		ret = SendBlockCommand(sisusb,3,
                          Cmd1,
                          0,      //users or kernels buffer.
                          i);	

		if(ret < 0){

			printk("%s: overlay phase error\n", __FUNCTION__); 
		
		}
		
	}
//yulw: set on screen frame 0 address to 0
     WriteRegisterByte(sisusb, 0xFCB4,0x01);
     WriteRegisterByte(sisusb, 0xFCB1,0x00);
     WriteRegisterByte(sisusb, 0xFCB2,0x00);
     WriteRegisterByte(sisusb, 0xFCB3,0x00);
     
     WriteRegisterByte(sisusb, 0xFB65,0x11);
     WriteRegisterByte(sisusb, 0xFB60,0x0);  // disable overlay
     WriteRegisterByte(sisusb, 0xFCB5,0x03);
     // make the screen black
     TransferBlockData (sisusb, 
     					sisusb->hres, sisusb->vres, 
     					sisusb->hres, sisusb->vres,
     					ColorFormat, sisusb->bpp, 
     					0, 0, sisusb->hres*3, 
     					NO_SCALE, NO_SCALE, 
     					NULL,0, 0);
     	
     WriteRegisterByte(sisusb, 0xFCB4,0x10);	// Display Frame 0
     WriteRegisterByte(sisusb, 0xFCB0,0x09);    //Enable.
     printk("VgaModeSetting_DDR Exit\n");
	 //gSetMode = 0;
}

VgaModeSetting(struct sisusb_usb_data *sisusb, u32 modeIndex)
{
	//VgaModeSetting_DDR(wifi_sisusb, modeIndex);
	in_sisusb = sisusb;
	in_modeindex = modeIndex;
	gSetMode = 1;
	//wake_up(&setmode_wq);
	printk("%s: gSetMode: %d\n", __FUNCTION__, gSetMode);
	wake_up_interruptible(&setmode_wq);
	//interruptible_sleep_on(&done_setmode_wq); // WHS: try
	//wait_event_interruptible(done_setmode_wq, gSetMode != 1);	
	wait_event(done_setmode_wq, gSetMode != 1);
	//while(gSetMode){}
	
	printk("%s done\n", __FUNCTION__);
}


static int 
sisusb_set_default_mode(struct sisusb_usb_data *sisusb, int hres,int vres,int bpp, int refrate)
{
	int ret = 0;
	
	return ret;
}

static int
sisusb_init_gfxcore(struct sisusb_usb_data *sisusb)
{
	int ret = 0;
	
	//TO DO.
	return ret;
}


static int
sisusb_do_init_gfxdevice(struct sisusb_usb_data *sisusb)
{	
	int    ret=0;	

	/* Do some magic */
	//TO DO.

	return ret;
}

/* Initialize the graphics device (return 0 on success)
 * This initializes the net2280 as well as the PCI registers
 * of the graphics board.
 */
static int
sisusb_init_gfxdevice(struct sisusb_usb_data *sisusb, int initscreen)
{
	int ret = 0, test = 0;
	
	if (sisusb->devinit == 1) {
		/* Read PCI BARs and see if they have been set up */
		//TO DO.
		
	}

	/* No? So reset the device */
	if ((sisusb->devinit == 0) || (test != 3)) 
	{		
		ret |= sisusb_do_init_gfxdevice(sisusb);
		
		if (ret == 0)
		    sisusb->devinit = 1;
	}

	if (sisusb->devinit) 
	{
		/* Initialize the graphics core */
		if (sisusb_init_gfxcore(sisusb) == 0) 
		{
			sisusb->gfxinit = 1;			
			ret |= sisusb_set_default_mode(sisusb, 1024,768,24,60);			
		}
	}

	return ret;
}


/* fops */

/* fops */

static int
triggerusb_open(struct inode *inode, struct file *file) // WHS: 
{
	struct sisusb_usb_data *sisusb;
	struct usb_interface *interface;
	int subminor = iminor(inode);

	if (!(interface = usb_find_interface(&triggerusb_driver, subminor)))
		return -ENODEV;

	if (!(sisusb = usb_get_intfdata(interface)))
		return -ENODEV;

	//mutex_lock(&sisusb->lock);
	down(&sisusb->lock);

	if (!sisusb->present || !sisusb->ready) {
		//mutex_unlock(&sisusb->lock);
		up(&sisusb->lock);
		return -ENODEV;
	}
/*
	if (sisusb->isopen) {
		//mutex_unlock(&sisusb->lock);
		up(&sisusb->lock);
		return -EBUSY;
	}
*/
	if (!sisusb->devinit) {
		if (sisusb->sisusb_dev->speed == USB_SPEED_HIGH) {
			if (sisusb_init_gfxdevice(sisusb, 0)) {
				//mutex_unlock(&sisusb->lock);
				up(&sisusb->lock);
				dev_err(&sisusb->sisusb_dev->dev, "Failed to initialize device\n");
				return -EIO;
			}
		} else {
			//mutex_unlock(&sisusb->lock);
			up(&sisusb->lock);
			dev_err(&sisusb->sisusb_dev->dev, "Device not attached to USB 2.0 hub\n");
			return -EIO;
		}
	}

	/* Increment usage count for our sisusb */
	kref_get(&sisusb->kref);

	sisusb->isopen = 1;

	file->private_data = sisusb;

	//mutex_unlock(&sisusb->lock);
	up(&sisusb->lock);

	return 0;

}

void
triggerusb_delete(struct kref *kref)
{
	struct sisusb_usb_data *sisusb = to_sisusb_dev(kref);

	if (!sisusb)
		return;

	sisusb_free_buffers(sisusb);
	sisusb_free_urbs(sisusb);

	if (sisusb->sisusb_dev)
		usb_put_dev(sisusb->sisusb_dev);

	sisusb->sisusb_dev = NULL;
	
#ifdef INCL_SISUSB_CON
	kfree(sisusb->SiS_Pr);
#endif
	kfree(sisusb);
}

static int
triggerusb_release(struct inode *inode, struct file *file)
{
	struct sisusb_usb_data *sisusb;

	if (!(sisusb = (struct sisusb_usb_data *)file->private_data))
		return -ENODEV;

	//mutex_lock(&sisusb->lock);
	down(&sisusb->lock);

	if (sisusb->present) {
		/* Wait for all URBs to finish if device still present */
		if (!sisusb_wait_all_out_complete(sisusb))
			sisusb_kill_all_busy(sisusb);
	}

	//Disable VGA function here, otherwise, the display will have noise(line).
	//WriteRegisterByte(sisusb, 0xfcb0, 0x20);

	sisusb->isopen = 0;
	file->private_data = NULL;

	//mutex_unlock(&sisusb->lock);
	up(&sisusb->lock);

	/* decrement the usage count on our device */
	kref_put(&sisusb->kref, triggerusb_delete);

	 struct kref *kref = &sisusb->kref;
        //printk("WHS: kref: %d\n", atomic_read(&kref->refcount));
        if(atomic_read(&kref->refcount) < 2){
                //Disable VGA function here, otherwise, the display will have n$
                WriteRegisterByte(sisusb, 0xfcb0, 0x20);
        }


	return 0;
}

static ssize_t
triggerusb_read(struct file *file, char __user *buffer, size_t count, loff_t *ppos)
{

	#if 0

	struct sisusb_usb_data *sisusb;
	ssize_t bytes_read = 0;
	int errno = 0;
	u8 buf8;
	u16 buf16;
	u32 buf32, address;

	if (!(sisusb = (struct sisusb_usb_data *)file->private_data))
		return -ENODEV;

	mutex_lock(&sisusb->lock);

	/* Sanity check */
	if (!sisusb->present || !sisusb->ready || !sisusb->sisusb_dev) {
		mutex_unlock(&sisusb->lock);
		return -ENODEV;
	}

	if ((*ppos) >= SISUSB_PCI_PSEUDO_IOPORTBASE &&
	    (*ppos) <  SISUSB_PCI_PSEUDO_IOPORTBASE + 128) {

		address = (*ppos) -
			SISUSB_PCI_PSEUDO_IOPORTBASE +
			SISUSB_PCI_IOPORTBASE;

		/* Read i/o ports
		 * Byte, word and long(32) can be read. As this
		 * emulates inX instructions, the data returned is
		 * in machine-endianness.
		 */
		switch (count) {

			case 1:
				if (sisusb_read_memio_byte(sisusb,
							SISUSB_TYPE_IO,
							address, &buf8))
					errno = -EIO;
				else if (put_user(buf8, (u8 __user *)buffer))
					errno = -EFAULT;
				else
					bytes_read = 1;

				break;

			case 2:
				if (sisusb_read_memio_word(sisusb,
							SISUSB_TYPE_IO,
							address, &buf16))
					errno = -EIO;
				else if (put_user(buf16, (u16 __user *)buffer))
					errno = -EFAULT;
				else
					bytes_read = 2;

				break;

			case 4:
				if (sisusb_read_memio_long(sisusb,
							SISUSB_TYPE_IO,
							address, &buf32))
					errno = -EIO;
				else if (put_user(buf32, (u32 __user *)buffer))
					errno = -EFAULT;
				else
					bytes_read = 4;

				break;

			default:
				errno = -EIO;

		}

	} else if ((*ppos) >= SISUSB_PCI_PSEUDO_MEMBASE &&
		   (*ppos) <  SISUSB_PCI_PSEUDO_MEMBASE + sisusb->vramsize) {

		address = (*ppos) -
			SISUSB_PCI_PSEUDO_MEMBASE +
			SISUSB_PCI_MEMBASE;

		/* Read video ram
		 * Remember: Data delivered is never endian-corrected
		 */
		errno = sisusb_read_mem_bulk(sisusb, address,
					NULL, count, buffer, &bytes_read);

		if (bytes_read)
			errno = bytes_read;

	} else  if ((*ppos) >= SISUSB_PCI_PSEUDO_MMIOBASE &&
		    (*ppos) <  SISUSB_PCI_PSEUDO_MMIOBASE + SISUSB_PCI_MMIOSIZE) {

		address = (*ppos) -
			SISUSB_PCI_PSEUDO_MMIOBASE +
			SISUSB_PCI_MMIOBASE;

		/* Read MMIO
		 * Remember: Data delivered is never endian-corrected
		 */
		errno = sisusb_read_mem_bulk(sisusb, address,
					NULL, count, buffer, &bytes_read);

		if (bytes_read)
			errno = bytes_read;

	} else  if ((*ppos) >= SISUSB_PCI_PSEUDO_PCIBASE &&
		    (*ppos) <= SISUSB_PCI_PSEUDO_PCIBASE + 0x5c) {

		if (count != 4) {
			mutex_unlock(&sisusb->lock);
			return -EINVAL;
		}

		address = (*ppos) - SISUSB_PCI_PSEUDO_PCIBASE;

		/* Read PCI config register
		 * Return value delivered in machine endianness.
		 */
		if (sisusb_read_pci_config(sisusb, address, &buf32))
			errno = -EIO;
		else if (put_user(buf32, (u32 __user *)buffer))
			errno = -EFAULT;
		else
			bytes_read = 4;

	} else {

		errno = -EBADFD;

	}

	(*ppos) += bytes_read;

	mutex_unlock(&sisusb->lock);

	return errno ? errno : bytes_read;

	#endif

	return 0;	

}

static ssize_t
triggerusb_write(struct file *file, const char __user *buffer, size_t count,loff_t *ppos)
{

	#if 0

	struct sisusb_usb_data *sisusb;
	int errno = 0;
	ssize_t bytes_written = 0;

	if (!(sisusb = (struct sisusb_usb_data *)file->private_data))
	     return -ENODEV;

	down(&sisusb->lock);

	/* Sanity check */
	if (!sisusb->present || !sisusb->ready || !sisusb->sisusb_dev) 
	{
		up(&sisusb->lock);
		return -ENODEV;
	}

        switch(*ppos)
        {
          case 1:
          break;
          case 2:
          break;        	
        	
        }                        		

	up(&sisusb->lock);

	return errno ? errno : bytes_written;

	#endif

	//printk(KERN_INFO "WHS: trigger write\n");

	struct sisusb_usb_data *sisusb;
	int errno = 0;	

	#if 1

	if(!(sisusb = (struct sisusb_usb_data *)file->private_data)) return -ENODEV;
	
	#ifdef tusbvga_SMP

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: write to get lock\n");

	#endif
	
	spin_lock(&sisusb->overlayLock);

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: write has got lock\n");

	#endif

	#else

	//mutex_lock(&sisusb->lock);
	down(&sisusb->lock);

	#endif

	/* Sanity check */
	if (!sisusb->present || !sisusb->ready || !sisusb->sisusb_dev) {
		#ifdef tusbvga_SMP
	
	spin_unlock(&sisusb->overlayLock);

	#else

	//mutex_unlock(&sisusb->lock);
	up(&sisusb->lock);

	#endif
		return -ENODEV;
	}	

	//printk(KERN_INFO "WHS: write par: DestX: %d DestY: %d ScrXPoint: %d ScrYPoint: %d Color: %x\n", sisusb->par0, sisusb->par1, sisusb->par2, sisusb->par3, sisusb->par4);

	if(!buffer){

		#ifdef tusbvga_DBG

		printk(KERN_INFO "WHS: Fill Rect\n");

		#endif

		FillBlockData
		(
			sisusb,
			sisusb->par0, // SrcX,
			sisusb->par1, // SrcY,
			sisusb->par0, // DestX,
			sisusb->par1, // DestY,
			sisusb->ColorFormat, // ColorFormat
			0, // NULL argument
			sisusb->par2, // ScrXPoint
			sisusb->par3, // ScrYPoint,
			sisusb->hres*3, // ScreenLineLen,
			1, // HScale,
			1, // VScale,
			(unsigned char*)NULL,
			1, // user/kernel buffer
			0,
			sisusb->par4 // WHS: color
		);

	}
	else{

		errno = TransferBlock
		(
			sisusb,
			sisusb->par5, // SrcX,
			sisusb->par6, // SrcY,
			sisusb->par0, // DestX,
			sisusb->par1, // DestY,
			sisusb->ColorFormat, // ColorFormat
			0, // NULL argument
			sisusb->par2, // ScrXPoint
			sisusb->par3, // ScrYPoint,
			sisusb->hres*3, // ScreenLineLen,
			sisusb->par9, // HScale,
			sisusb->par10, // VScale,
			(unsigned char*)buffer,
			1, // user/kernel buffer
			0			
		);

	}

	#ifdef tusbvga_SMP
	
	spin_unlock(&sisusb->overlayLock);

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: write has got lock unlocked\n");

	#endif

	#else

	//mutex_unlock(&sisusb->lock);
	up(&sisusb->lock);

	#endif

	#endif

	return errno;
	
}

static loff_t
triggerusb_lseek(struct file *file, loff_t offset, int orig)
{
	struct sisusb_usb_data *sisusb;
	loff_t ret;

	if (!(sisusb = (struct sisusb_usb_data *)file->private_data))
		return -ENODEV;

	down(&sisusb->lock);

	/* Sanity check */
	if (!sisusb->present || !sisusb->ready || !sisusb->sisusb_dev) 
	{
		up(&sisusb->lock);
		return -ENODEV;
	}

	switch (orig) 
	{
		case 0:
			file->f_pos = offset;
			ret = file->f_pos;
			/* never negative, no force_successful_syscall needed */
			break;
		case 1:
			file->f_pos += offset;
			ret = file->f_pos;
			/* never negative, no force_successful_syscall needed */
			break;
		default:
			/* seeking relative to "end of file" is not supported */
			ret = -EINVAL;
	}

	up(&sisusb->lock);
	return ret;
}

static int
sisusb_handle_command(struct sisusb_usb_data *sisusb, struct sisusb_command *y,unsigned long arg)
{
	int 	retval;
	unsigned long begTime, endTime;
	__u8 bpp;
	
	/* All our commands require the device
	 * to be initialized.
	 */
	if (!sisusb->devinit)
	    return -ENODEV;
      
        
	switch(y->operation) 
	{
	
		case SUCMD_SEND_BULKOUT_PIPE2:	
		retval = 0;			
		break;

		case SUCMD_SEND_BULKOUT_PIPE1:	
		retval = 0;			
		break;		     

		case SUCMD_SETMODE:                
		/* Gfx core must be initialized, SiS_Pr must exist */
		if(!sisusb->gfxinit) return -ENODEV;
		retval = 0;			
		VgaModeSetting(sisusb, y->data3);					
		break;	
		
		case TUCMD_COPYVRAM:

		//printk(KERN_INFO "WHS: COPYVRAM\n");

		/* Gfx core must be initialized, SiS_Pr must exist */
		if(!sisusb->gfxinit) return -ENODEV;
		retval = 0;			
		sisusb->par0 = y->data3; // WHS: DestX
		sisusb->par1 = y->data4; // WHS: DestY
		sisusb->par2 = y->data5; // WHS: ScrPointX
		sisusb->par3 = y->data6; // WHS: ScrPointY	
		sisusb->par5 = y->data7; // WHS: SrcX
		sisusb->par6 = y->data8; // WHS: SrcY	
		sisusb->par7 = y->data10; // WHS: user buf
		sisusb->par9 = y->data12; // WHS: RatioX
		sisusb->par10 = y->data13; // WHS: RatioY

		retval = TransferBlock
		(
			sisusb,
			sisusb->par5, // SrcX,
			sisusb->par6, // SrcY,
			sisusb->par0, // DestX,
			sisusb->par1, // DestY,
			sisusb->ColorFormat, // ColorFormat
			NULL, // NULL argument
			sisusb->par2, // ScrXPoint
			sisusb->par3, // ScrYPoint,
			sisusb->hres*3, // ScreenLineLen,
			sisusb->par9, // HScale,
			sisusb->par10, // VScale,
			sisusb->par7, // WHS: user buf
			1, // user/kernel buffer
			//0,
			0			
		);
		
		break;	
		
		case TUCMD_DISABLE:

		//printk(KERN_INFO "WHS: COPYVRAM\n");

		/* Gfx core must be initialized, SiS_Pr must exist */
		if(!sisusb->gfxinit) return -ENODEV;
		WriteRegisterByte(sisusb, 0xFCB0, 0x00); // disable: WHS: difference between trigger and trigger plus
		retval = 0;
		
		break;	
		
		case TUCMD_DOOVERLAY:

		#ifdef tusbvga_DBG
		printk(KERN_INFO "WHS: DOOVERLAY\n");
		#endif

		if(!sisusb->gfxinit) return -ENODEV;
		sisusb->par0 = y->data3; // WHS: DestX
		sisusb->par1 = y->data4; // WHS: DestY
		sisusb->par2 = y->data5; // WHS: ScrPointX
		sisusb->par3 = y->data6; // WHS: ScrPointY		
		sisusb->par4 = y->data7; // WHS: ColorFormat	
		sisusb->par5 = y->data8; // WHS: SrcX
		sisusb->par6 = y->data9; // WHS: SrcY
		sisusb->par7 = y->data10; // WHS: user buf
		sisusb->par8 = y->data11; // WHS: dst color key
		sisusb->par9 = y->data12; // WHS: RatioX
		sisusb->par10 = y->data13; // WHS: RatioY

		#ifdef tusbvga_DBG
		printk(KERN_INFO "RatioX: %x, RatioY: %x, colorKey: %x, DstX: %d, DstY: %d, ScrX: %d, ScrY: %d, ColorFormat: %d, SrcX: %d, SrcY: %d\n", 
			sisusb->par9, sisusb->par10, sisusb->par8, sisusb->par0, sisusb->par1, sisusb->par2, sisusb->par3, sisusb->par4, sisusb->par5, sisusb->par6);
		#endif
		if(sisusb->par4 == 0xffff) {// disable overlay
			printk("%s: Close Overlay\n", __FUNCTION__);
			WriteRegisterByte(sisusb, 0xFB60, 0);   
   			mdelay(100);
			break;
		}

		begTime = jiffies; 
		retval = DoOverlay(sisusb);
		endTime = jiffies; 
		
		#ifdef tusbvga_OPT

		printk(KERN_INFO"triggerusb: WHS: DoOverlay ellapse time: %d\n", endTime - begTime);

		#endif
		
		break;  

		case TUCMD_COPYVRAM_EXA:

		#ifdef tusbvga_DBG

		printk(KERN_INFO "WHS: COPYVRAM_EXA\n");

		#endif

		if(!sisusb->gfxinit) return -ENODEV;
		retval = 0;			
		sisusb->par0 = y->data3; // WHS: DestX
		sisusb->par1 = y->data4; // WHS: DestY
		sisusb->par2 = y->data5; // WHS: DestAddr
		//sisusb->par3 = y->data6; // WHS: ScrPointY		
		sisusb->par7 = y->data10; // WHS: user buf

		begTime = jiffies; 
		retval = DoCopyVramExa(sisusb);
		endTime = jiffies; 

		break;

		case TUCMD_USERTEST:
		if(!sisusb->gfxinit) return -ENODEV;
		retval = 0;			
		sisusb->par0 = y->data3; // WHS: DestX
		sisusb->par1 = y->data4; // WHS: DestY
		sisusb->par2 = y->data5; // WHS: ScrPointX
		sisusb->par3 = y->data6; // WHS: ScrPointY
		sisusb->par4 = y->data7; // WHS: Color
		
		break;

		#ifdef tusbvga_MT

		case TUCMD_OVERLAY_PAR:

		#ifdef tusbvga_DBG
		printk(KERN_INFO "WHS: OVERLAY PAR\n");
		#endif
			
		if(!sisusb->gfxinit) return -ENODEV;
		retval = 0;			

		sisusb->par0 = y->data3; // WHS: DestX
		sisusb->par1 = y->data4; // WHS: DestY
		sisusb->par2 = y->data5; // WHS: ScrPointX
		sisusb->par3 = y->data6; // WHS: ScrPointY		
		sisusb->par4 = y->data7; // WHS: ColorFormat	
		sisusb->par5 = y->data8; // WHS: SrcX
		sisusb->par6 = y->data9; // WHS: SrcY

		//sisusb->par7 = y->data10; // WHS: user buf

		//if(copy_from_user(gBuf, (void __user*)y->data10, y->data8*y->data9*2)) retval = -EFAULT;
		
		sisusb->par7 = gBuf;
		
		sisusb->par8 = y->data11; // WHS: dst color key
		sisusb->par9 = y->data12; // WHS: RatioX
		sisusb->par10 = y->data13; // WHS: RatioY

		#ifdef tusbvga_DBG
		printk(KERN_INFO "RatioX: %x, RatioY: %x, colorKey: %x, DstX: %d, DstY: %d, ScrX: %d, ScrY: %d, ColorFormat: %d, SrcX: %d, SrcY: %d\n\n", 
			sisusb->par9, sisusb->par10, sisusb->par8, sisusb->par0, sisusb->par1, sisusb->par2, sisusb->par3, sisusb->par4, sisusb->par5, sisusb->par6);
		#endif
			
		break;

		case TUCMD_OVERLAY_DO:

		#ifdef tusbvga_DBG
		printk(KERN_INFO "WHS: DOOVERLAY DO\n\n");
		#endif
		
		if(!sisusb->gfxinit) return -ENODEV;
		retval = 0;			

		retval = DoOverlay(sisusb);
			
		break;

		#endif

		#ifdef tusbvga_EDID

		case TUCMD_EDID:

		SCOPE_BEG

		unsigned char *edidbuf;

		#ifdef tusbvga_DBG
		printk(KERN_INFO "WHS: ReadEDID\n\n");
		#endif
		
		if(!sisusb->gfxinit) return -ENODEV;
		retval = 0;			

		sisusb->par7 = y->data10; // WHS: user buf
		
		if((edidbuf = kmalloc(512, GFP_KERNEL)) != NULL){
			
			ReadEDID(sisusb, edidbuf, 512);
			//printk("dump edid 512 bytes\n");
			//dump_buf(edidbuf, 512);
			//parse_edid(edidbuf);
			if(copy_to_user((void __user *)y->data10, edidbuf, 512)){
				printk(KERN_ERR "WHS: ReadEDID copy_to_user failed");
				retval = -EFAULT;
			}
			kfree(edidbuf);
		}
		else{ 
			retval = -EFAULT;
			printk(KERN_ERR "WHS: ReadEDID kmalloc failed");
		}

		//retval = open("xorg.conf", "r");
		
		SCOPE_END

		break;

		#endif
		
		default: retval = -EINVAL;
		
	}

	if (retval > 0)
	    retval = -EIO;

	return retval;
}

static int
triggerusb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,unsigned long arg)
{
		
	struct sisusb_usb_data *sisusb;
	struct sisusb_info x;
	struct sisusb_command y;
	int	retval = 0;
	u32 __user *argp = (u32 __user *)arg;

	if (!(sisusb = (struct sisusb_usb_data *)file->private_data))
		return -ENODEV;

	#ifdef tusbvga_SMP

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: ioctl to get lock\n");

	#endif
	
	spin_lock(&sisusb->overlayLock);

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: ioctl has got lock\n");

	#endif

	#else

	//mutex_lock(&sisusb->lock);
	//printk(KERN_INFO "WHS: ioctl to get lock\n");
	down(&sisusb->lock);
	//printk(KERN_INFO "WHS: ioctl has got lock\n");

	#endif

	/* Sanity check */
	if (!sisusb->present || !sisusb->ready || !sisusb->sisusb_dev) {
		retval = -ENODEV;		
		goto err_out;
	}

	//printk(KERN_INFO "WHS: trigger ioctl!\n");

	switch (cmd) {

		case SISUSB_GET_CONFIG_SIZE:

			if (put_user(sizeof(x), argp))
				retval = -EFAULT;

			break;

		case SISUSB_GET_CONFIG:

			x.sisusb_id	    = SISUSB_ID;
			x.sisusb_version    = SISUSB_VERSION;
			x.sisusb_revision   = SISUSB_REVISION;
			x.sisusb_patchlevel = SISUSB_PATCHLEVEL;
			x.sisusb_gfxinit    = sisusb->gfxinit;
			x.sisusb_vrambase   = SISUSB_PCI_PSEUDO_MEMBASE;
			x.sisusb_mmiobase   = SISUSB_PCI_PSEUDO_MMIOBASE;
			x.sisusb_iobase     = SISUSB_PCI_PSEUDO_IOPORTBASE;
			x.sisusb_pcibase    = SISUSB_PCI_PSEUDO_PCIBASE;
			x.sisusb_vramsize   = sisusb->vramsize;
			x.sisusb_minor	    = sisusb->minor;
			x.sisusb_fbdevactive= 0;
#ifdef INCL_SISUSB_CON
			x.sisusb_conactive  = sisusb->haveconsole ? 1 : 0;
#else
			x.sisusb_conactive  = 0;
#endif

			if (copy_to_user((void __user *)arg, &x, sizeof(x)))
				retval = -EFAULT;

			break;

		case SISUSB_COMMAND:

			#ifdef tusbvga_DBG

			//printk(KERN_INFO "WHS: trigger ioctl SISUSB_COMMAND!\n");

			#endif

			if (copy_from_user(&y, (void __user *)arg, sizeof(y)))
				retval = -EFAULT;
			else
				retval = sisusb_handle_command(sisusb, &y, arg);

			break;

		case TUSB_ISNODE:
			*((unsigned long*)arg) = 0xffff;
			break;

		case TUSB_IOCTL_READ_ATMEL:
			{
				READ_ATMEL readAtmel;
				if (copy_from_user(&readAtmel, (void __user *)arg, sizeof(READ_ATMEL)))
					retval = -EFAULT;
				else {
					UCHAR outBuf[512];
     				ULONG Count;
					
					if (readAtmel.N == 0)
     					Count = 256;
     				else
     					Count = readAtmel.N;

					memset(outBuf, 0, Count);

					if (Read_Atmel_Func (sisusb, &readAtmel, outBuf)){
						if (copy_to_user((void __user *)readAtmel.read_buf, outBuf, Count))
							retval = -EFAULT;
					}
					else
						retval = -EFAULT;
				}
			}
			break;

		case TUSB_IOCTL_WRITE_ATMEL:
			{
				WRITE_ATMEL writeAtmel;
				UCHAR *obuf;

				if (copy_from_user(&writeAtmel, (void __user *)arg, sizeof(WRITE_ATMEL)))
					retval = -EFAULT;
				else {

					obuf =  kmalloc(writeAtmel.size, GFP_KERNEL);
					if (copy_from_user(obuf, writeAtmel.write_buf, writeAtmel.size))
						retval = -EFAULT;
					else {
						if (!Write_Ateml_Func (sisusb, obuf, writeAtmel.size))
							retval = -EFAULT;
						kfree(obuf);
					}	
				}
			}
			break;

		case TUSB_IOCTL_LIGHT_LED:
			{
				int lighton = 0;
				
				if (copy_from_user(&lighton, (void __user *)arg, sizeof(int)))
					retval = -EFAULT;
				else
					LightOn(sisusb, lighton);
			}
			break;
		case TUSB_IOCTL_ACK_POLLING:
			if(!CheckATMel(sisusb))
				retval = -EFAULT;

			break;
		default:
			retval = -ENOTTY;
			break;
	}

err_out:
	
	#ifdef tusbvga_SMP

	spin_unlock(&sisusb->overlayLock);

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: ioctl has got lock unlocked\n");

	#endif

	#else
	
	//mutex_unlock(&sisusb->lock);
	up(&sisusb->lock);
	//printk(KERN_INFO "WHS: ioctl has got lock unlocked\n");

	#endif
	
	return retval;
	
}

#ifdef SISUSB_NEW_CONFIG_COMPAT
static long
sisusb_compat_ioctl(struct file *f, unsigned int cmd, unsigned long arg)
{
	long retval;

	switch (cmd) {
		case SISUSB_GET_CONFIG_SIZE:
		case SISUSB_GET_CONFIG:
		case SISUSB_COMMAND:
			lock_kernel();
			retval = sisusb_ioctl(f->f_path.dentry->d_inode, f, cmd, arg);
			unlock_kernel();
			return retval;

		default:
			return -ENOIOCTLCMD;
	}
}
#endif

static struct file_operations usb_triggerusb_fops = { // WHS: for x driver

	.owner =	THIS_MODULE,
	.open =		triggerusb_open,
	.release =	triggerusb_release,
	.read =		triggerusb_read,
	.write =	triggerusb_write,
	.llseek = 	triggerusb_lseek,
	
        #ifdef SISUSB_NEW_CONFIG_COMPAT
	.compat_ioctl = triggerusb_compat_ioctl,
        #endif
        
	.ioctl =	triggerusb_ioctl

};

// WHS: fb_ops below

static int tusbvga_open(struct fb_info *info, int user)
{

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: fb open!\n");

	#endif

	#if 0

	struct nvidia_par *par = info->par;

	mutex_lock(&par->open_lock);

	if (!par->open_count) {
		save_vga_x86(par);
		nvidia_save_vga(par, &par->initial_state);
	}

	par->open_count++;
	mutex_unlock(&par->open_lock);
	return 0;

	#endif

	#if 1
		
	struct tusbvga_par *par = (struct tusbvga_par*)info->par;
	struct sisusb_usb_data *sisusb = par->sisusb;

	down(&(sisusb->lock));
	
	if(!par->open_count){
		//VgaModeSetting(sisusb, MODE_1024X768X24X75);
	}
	//printk(KERN_INFO "WHS: minor = %d!\n", sisusb->minor);

	par->open_count++;

	up(&(sisusb->lock));

	#endif

	return 0;

}

static int tusbvga_release(struct fb_info *info, int user)
{

	#if 0

	struct nvidia_par *par = info->par;
	int err = 0;

	mutex_lock(&par->open_lock);

	if (!par->open_count) {
		err = -EINVAL;
		goto done;
	}

	if (par->open_count == 1) {
		nvidia_write_regs(par, &par->initial_state);
		restore_vga_x86(par);
	}

	par->open_count--;
done:
	mutex_unlock(&par->open_lock);
	return err;

	#endif

	int err = 0;

	struct tusbvga_par *par = (struct tusbvga_par*)info->par;
	struct sisusb_usb_data *sisusb = par->sisusb;
	
	down(&(sisusb->lock));
	
	if(!par->open_count){
		err = -EINVAL;
		goto done;
	}
	
	par->open_count--;

done:

	up(&(sisusb->lock));	

	return err;

}

static int
tusbvga_handle_command(struct sisusb_usb_data *sisusb, struct tusbvga_command *y, unsigned long arg)
{

	#if 0

	int	retval, port, length;
	u32	address;

	/* All our commands require the device
	 * to be initialized.
	 */
	if (!sisusb->devinit)
		return -ENODEV;

	port = y->data3 -
		SISUSB_PCI_PSEUDO_IOPORTBASE +
		SISUSB_PCI_IOPORTBASE;

	switch (y->operation) {
		case SUCMD_GET:
			retval = sisusb_getidxreg(sisusb, port,
							 y->data0, &y->data1);
			if (!retval) {
				if (copy_to_user((void __user *)arg, y,
							sizeof(*y)))
					retval = -EFAULT;
			}
			break;

		case SUCMD_SET:
			retval = sisusb_setidxreg(sisusb, port,
						y->data0, y->data1);
			break;

		case SUCMD_SETOR:
			retval = sisusb_setidxregor(sisusb, port,
						y->data0, y->data1);
			break;

		case SUCMD_SETAND:
			retval = sisusb_setidxregand(sisusb, port,
						y->data0, y->data1);
			break;

		case SUCMD_SETANDOR:
			retval = sisusb_setidxregandor(sisusb, port,
						y->data0, y->data1, y->data2);
			break;

		case SUCMD_SETMASK:
			retval = sisusb_setidxregmask(sisusb, port,
						y->data0, y->data1, y->data2);
			break;

		case SUCMD_CLRSCR:
			/* Gfx core must be initialized */
			if (!sisusb->gfxinit)
				return -ENODEV;

			length = (y->data0 << 16) | (y->data1 << 8) | y->data2;
			address = y->data3 -
				SISUSB_PCI_PSEUDO_MEMBASE +
				SISUSB_PCI_MEMBASE;
			retval = sisusb_clear_vram(sisusb, address, length);
			break;

		case SUCMD_HANDLETEXTMODE:
			retval = 0;
#ifdef INCL_SISUSB_CON
			/* Gfx core must be initialized, SiS_Pr must exist */
			if (!sisusb->gfxinit || !sisusb->SiS_Pr)
				return -ENODEV;

			switch (y->data0) {
			case 0:
				retval = sisusb_reset_text_mode(sisusb, 0);
				break;
			case 1:
				sisusb->textmodedestroyed = 1;
				break;
			}
#endif
			break;

#ifdef INCL_SISUSB_CON
		case SUCMD_SETMODE:
			/* Gfx core must be initialized, SiS_Pr must exist */
			if (!sisusb->gfxinit || !sisusb->SiS_Pr)
				return -ENODEV;

			retval = 0;

			sisusb->SiS_Pr->IOAddress = SISUSB_PCI_IOPORTBASE + 0x30;
			sisusb->SiS_Pr->sisusb = (void *)sisusb;

			if (SiSUSBSetMode(sisusb->SiS_Pr, y->data3))
				retval = -EINVAL;

			break;

		case SUCMD_SETVESAMODE:
			/* Gfx core must be initialized, SiS_Pr must exist */
			if (!sisusb->gfxinit || !sisusb->SiS_Pr)
				return -ENODEV;

			retval = 0;

			sisusb->SiS_Pr->IOAddress = SISUSB_PCI_IOPORTBASE + 0x30;
			sisusb->SiS_Pr->sisusb = (void *)sisusb;

			if (SiSUSBSetVESAMode(sisusb->SiS_Pr, y->data3))
				retval = -EINVAL;

			break;
#endif

		default:
			retval = -EINVAL;
	}

	if (retval > 0)
		retval = -EIO;

	return retval;

	#endif

	int retval = 0;

	if(!sisusb->devinit) return -ENODEV;

	switch(y->operation) 
	{

		case TCMD_SETMODE:	

		#ifdef tusbvga_DBG	

		printk(KERN_INFO "WHS: fb TCMD_SETMODE\n");

		#endif

		///* Gfx core must be initialized, SiS_Pr must exist */
		//if(!sisusb->gfxinit) return -ENODEV;
		//retval = 0;			
		VgaModeSetting(sisusb, y->par0);			

		break;
       
       case TCMD_SET_WRITE_PAR:

		#ifdef tusbvga_DBG

		printk(KERN_INFO "WHS: fb SET_WRITE_PAR\n");

		#endif

		sisusb->par0 = y->par0; // WHS: DestX
		sisusb->par1 = y->par1; // WHS: DestY
		sisusb->par2 = y->par2; // WHS: ScrPointX
		sisusb->par3 = y->par3; // WHS: ScrPointY
		sisusb->par4 = y->par4; // WHS: Color
				                				
		break;			                

		default: retval = -EINVAL; break;
	}

	//if(retval > 0) retval = -EIO;

	return retval;

}


static int
tusbvga_fb_ioctl(struct fb_info *info, unsigned int cmd, unsigned long arg)
{

	#if 0

	struct sisusb_usb_data *sisusb;
	struct sisusb_info x;
	struct sisusb_command y;
	int 	retval = 0;
	u32 __user *argp = (u32 __user *)arg;

	if(!(sisusb = (struct sisusb_usb_data *)file->private_data)) return -ENODEV;

	down(&sisusb->lock);

	/* Sanity check */
	if(!sisusb->present || !sisusb->ready || !sisusb->sisusb_dev){
		retval = -ENODEV;
		goto err_out;
	}

	switch(cmd){

		case SISUSB_GET_CONFIG_SIZE:

			if (put_user(sizeof(x), argp))
			    retval = -EFAULT;

			break;

		case SISUSB_GET_CONFIG:

			x.sisusb_id   	    = SISUSB_ID;
			x.sisusb_version    = SISUSB_VERSION;
			x.sisusb_revision   = SISUSB_REVISION;
			x.sisusb_patchlevel = SISUSB_PATCHLEVEL;
			x.sisusb_gfxinit    = sisusb->gfxinit;
						
			x.sisusb_vrambase   = SISUSB_PCI_PSEUDO_MEMBASE;
			x.sisusb_mmiobase   = SISUSB_PCI_PSEUDO_MMIOBASE;
			x.sisusb_iobase     = SISUSB_PCI_PSEUDO_IOPORTBASE;
			x.sisusb_pcibase    = SISUSB_PCI_PSEUDO_PCIBASE;
			x.sisusb_vramsize   = sisusb->vramsize;		
			
			x.sisusb_minor	    = sisusb->minor;
			x.sisusb_fbdevactive= 0;
			x.sisusb_conactive  = 0;

			if (copy_to_user((void __user *)arg, &x, sizeof(x)))
			    retval = -EFAULT;

			break;

		case SISUSB_COMMAND:

			if (copy_from_user(&y, (void __user *)arg, sizeof(y)))
			    retval = -EFAULT;
			else
			    retval = sisusb_handle_command(sisusb, &y, arg);

			break;

		default:
			retval = -EINVAL;
			break;
	}

        err_out:
        
	up(&sisusb->lock);
	return retval;

	#endif

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: fb ioctl\n");

	#endif
	
	int retval = 0;
	
	struct sisusb_usb_data *sisusb;
	struct tusbvga_command y;	

	if(!(sisusb = ((struct tusbvga_par*)info->par)->sisusb)) return -ENODEV;

	down(&sisusb->lock);

	/* Sanity check */
	if(!sisusb->present || !sisusb->ready || !sisusb->sisusb_dev){
		retval = -ENODEV;
		goto err_out;
	}

	switch(cmd){
		
		case TUSB_COMMAND:

		#ifdef tusbvga_DBG

		printk(KERN_INFO "WHS: fb ioctl TUSB_COMMAND\n");

		#endif

		if(copy_from_user(&y, (void __user *)arg, sizeof(y))){ 
			printk(KERN_INFO "WHS: fb ioctl copy user data failed\n");
			retval = -EFAULT;
		}
		else retval = tusbvga_handle_command(sisusb, &y, arg);

		break;

		case TUSB_ISNODE:

		*((unsigned long*)arg) = 0xffff;

		break; 

		default: retval = -EINVAL; break;

	}

	err_out:

	up(&sisusb->lock);

	return retval;
	
}

static ssize_t
tusbvga_write(struct fb_info *info, const char __user *buffer, size_t count, loff_t *ppos)
{

	#if 0

	struct sisusb_usb_data *sisusb;
	int errno = 0;
	ssize_t bytes_written = 0;
	u8 buf8;
	u16 buf16;
	u32 buf32, address;

	if (!(sisusb = (struct sisusb_usb_data *)file->private_data))
		return -ENODEV;

	mutex_lock(&sisusb->lock);

	/* Sanity check */
	if (!sisusb->present || !sisusb->ready || !sisusb->sisusb_dev) {
		mutex_unlock(&sisusb->lock);
		return -ENODEV;
	}

	if ((*ppos) >= SISUSB_PCI_PSEUDO_IOPORTBASE &&
	    (*ppos) <  SISUSB_PCI_PSEUDO_IOPORTBASE + 128) {

		address = (*ppos) -
			SISUSB_PCI_PSEUDO_IOPORTBASE +
			SISUSB_PCI_IOPORTBASE;

		/* Write i/o ports
		 * Byte, word and long(32) can be written. As this
		 * emulates outX instructions, the data is expected
		 * in machine-endianness.
		 */
		switch (count) {

			case 1:
				if (get_user(buf8, (u8 __user *)buffer))
					errno = -EFAULT;
				else if (sisusb_write_memio_byte(sisusb,
							SISUSB_TYPE_IO,
							address, buf8))
					errno = -EIO;
				else
					bytes_written = 1;

				break;

			case 2:
				if (get_user(buf16, (u16 __user *)buffer))
					errno = -EFAULT;
				else if (sisusb_write_memio_word(sisusb,
							SISUSB_TYPE_IO,
							address, buf16))
					errno = -EIO;
				else
					bytes_written = 2;

				break;

			case 4:
				if (get_user(buf32, (u32 __user *)buffer))
					errno = -EFAULT;
				else if (sisusb_write_memio_long(sisusb,
							SISUSB_TYPE_IO,
							address, buf32))
					errno = -EIO;
				else
					bytes_written = 4;

				break;

			default:
				errno = -EIO;
		}

	} else if ((*ppos) >= SISUSB_PCI_PSEUDO_MEMBASE &&
		   (*ppos) <  SISUSB_PCI_PSEUDO_MEMBASE + sisusb->vramsize) {

		address = (*ppos) -
			SISUSB_PCI_PSEUDO_MEMBASE +
			SISUSB_PCI_MEMBASE;

		/* Write video ram.
		 * Buffer is copied 1:1, therefore, on big-endian
		 * machines, the data must be swapped by userland
		 * in advance (if applicable; no swapping in 8bpp
		 * mode or if YUV data is being transferred).
		 */
		errno = sisusb_write_mem_bulk(sisusb, address, NULL,
					count, buffer, 0, &bytes_written);

		if (bytes_written)
			errno = bytes_written;

	} else  if ((*ppos) >= SISUSB_PCI_PSEUDO_MMIOBASE &&
		    (*ppos) <  SISUSB_PCI_PSEUDO_MMIOBASE + SISUSB_PCI_MMIOSIZE) {

		address = (*ppos) -
			SISUSB_PCI_PSEUDO_MMIOBASE +
			SISUSB_PCI_MMIOBASE;

		/* Write MMIO.
		 * Buffer is copied 1:1, therefore, on big-endian
		 * machines, the data must be swapped by userland
		 * in advance.
		 */
		errno = sisusb_write_mem_bulk(sisusb, address, NULL,
					count, buffer, 0, &bytes_written);

		if (bytes_written)
			errno = bytes_written;

	} else  if ((*ppos) >= SISUSB_PCI_PSEUDO_PCIBASE &&
		    (*ppos) <= SISUSB_PCI_PSEUDO_PCIBASE + SISUSB_PCI_PCONFSIZE) {

		if (count != 4) {
			mutex_unlock(&sisusb->lock);
			return -EINVAL;
		}

		address = (*ppos) - SISUSB_PCI_PSEUDO_PCIBASE;

		/* Write PCI config register.
		 * Given value expected in machine endianness.
		 */
		if (get_user(buf32, (u32 __user *)buffer))
			errno = -EFAULT;
		else if (sisusb_write_pci_config(sisusb, address, buf32))
			errno = -EIO;
		else
			bytes_written = 4;


	} else {

		/* Error */
		errno = -EBADFD;

	}

	(*ppos) += bytes_written;

	mutex_unlock(&sisusb->lock);

	return errno ? errno : bytes_written;

	#endif

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: fb write\n");

	#endif

	struct sisusb_usb_data *sisusb;
	int errno = 0;	

	#if 1

	if(!(sisusb = ((struct tusbvga_par*)info->par)->sisusb)) return -ENODEV;
	
	//mutex_lock(&sisusb->lock);
	down(&sisusb->lock);

	#if 0

	__u32 par0; // WHS: DestX
	__u32 par1; // WHS: DestY
	__u32 par2; // WHS: ScrXPoint
	__u32 par3; // WHS: ScrYPoint

	TransferBlockData(struct sisusb_usb_data *sisusb,
		       u32		 SrcX,
		       u32		 SrcY,
		       u32		 DestX,
		       u32		 DestY,
		       u32		 ColorFormat,
		       u32		 SrcBitsPlane,
		       u32		 ScrXPoint,
		       u32		 ScrYPoint,
		       u32		 ScreenLineLen,
		       u32		 HScale,
		       u32		 VScale,
		       unsigned char*	 pdata,
		       int               users,
		       u32		 len);

	#endif

	#ifdef tusbvga_DBG

	printk(KERN_INFO "WHS: write par: DestX: %d DestY: %d ScrXPoint: %d ScrYPoint: %d Color: %x\n", sisusb->par0, sisusb->par1, sisusb->par2, sisusb->par3, sisusb->par4);

	#endif

	if(!buffer){

		#ifdef tusbvga_DBG

		printk(KERN_INFO "WHS: Fill Rect\n");

		#endif

		FillBlockData
		(
			sisusb,
			sisusb->par0, // SrcX,
			sisusb->par1, // SrcY,
			sisusb->par0, // DestX,
			sisusb->par1, // DestY,
			sisusb->ColorFormat, // ColorFormat
			NULL, // NULL argument
			sisusb->par2, // ScrXPoint
			sisusb->par3, // ScrYPoint,
			sisusb->hres*3, // ScreenLineLen,
			1, // HScale,
			1, // VScale,
			NULL,
			1, // user/kernel buffer
			0,
			sisusb->par4 // WHS: color
		);

	}
	else{

		TransferBlockData
		(
			sisusb,
			sisusb->par0, // SrcX,
			sisusb->par1, // SrcY,
			sisusb->par0, // DestX,
			sisusb->par1, // DestY,
			sisusb->ColorFormat, // ColorFormat
			NULL, // NULL argument
			sisusb->par2, // ScrXPoint
			sisusb->par3, // ScrYPoint,
			sisusb->hres*3, // ScreenLineLen,
			1, // HScale,
			1, // VScale,
			buffer,
			1, // user/kernel buffer
			0			
		);

	}

	//mutex_unlock(&sisusb->lock);
	up(&sisusb->lock);

	#endif

	return errno;

}


static struct fb_ops tusbvga_fb_ops = {

	#if 1

	.owner = THIS_MODULE,
	.fb_open = tusbvga_open,
	.fb_release = tusbvga_release,
	.fb_write = tusbvga_write,
	.fb_ioctl = tusbvga_fb_ioctl,	
	//.fb_check_var   = tusbvga_check_var,
	//.fb_set_par     = tusbvga_set_par,
	//.fb_setcolreg   = nvidiafb_setcolreg,
	//.fb_pan_display = nvidiafb_pan_display,
	//.fb_blank       = nvidiafb_blank,
	//.fb_fillrect    = tusbvga_fillrect,
	//.fb_copyarea    = tusbvga_copyarea,
	//.fb_imageblit   = tusbvga_imageblit,
	//.fb_cursor      = nvidiafb_cursor,
	.fb_sync = NULL,

	#endif
		
};

static struct usb_class_driver usb_triggerusb_class = {
	.name =		"triggerusbvga%d",
	.fops =		&usb_triggerusb_fops,
	.minor_base =	SISUSB_MINOR
};

static int tusb_demo(void *sisusb)
{
	int vram_size, f1;
printk("%s\n", __FUNCTION__);
	//printk(KERN_ERR"WHS: scalefactor: %x\n", Trigger_ComputeScaleFactor(0.7));

	//Trigger_ComputeScaleFactor(0.7);

	//return 0;
	
//	daemonize("tusb_demo");
	vram_size = Trigger_Get_VRAM_Size((struct sisusb_usb_data *)sisusb);
	VgaModeSetting((struct sisusb_usb_data *)sisusb, MODE_1024X768X24X75);	

	//for(f1=0; f1<16; f1++)
	Trigger_Demo_MainScreen_Display((struct sisusb_usb_data *)sisusb);   
	
	
	Trigger_Demo_Stretch((struct sisusb_usb_data *)sisusb , 2);
	Trigger_Demo_Stretch((struct sisusb_usb_data *)sisusb , 4);
	
	if(vram_size == 16) {

		//for(f1=0; f1<16; f1++)
		//Trigger_Demo_Overlay((struct sisusb_usb_data *)sisusb , 1);
		Trigger_Demo_Color_Key((struct sisusb_usb_data *)sisusb , 1);
	}
	else {
		//Trigger_Demo_Overlay((struct sisusb_usb_data *)sisusb , 0);
		Trigger_Demo_Color_Key((struct sisusb_usb_data *)sisusb , 0);
	}
	
	return 0;
}

static int setmode_thread(void *arg)
{
	
	daemonize("tsetmode");
	while(1) {
		//interruptible_sleep_on(&setmode_wq);
		wait_event_interruptible(setmode_wq, gSetMode); // WHS: try
		if(info_module_exit) break;
		printk("enter %s\n", __FUNCTION__);
		if(in_sisusb->bGetTimingFromPanel)
			VgaSetModeByPanelCfg(in_sisusb, in_modeindex);
		else
			VgaModeSetting_DDR(in_sisusb, in_modeindex);
		printk("VgaModeSetting_DDR done\n");
		
		gSetMode = 0;
		//wake_up_interruptible(&done_setmode_wq);
		wake_up(&done_setmode_wq);
		//wake_up(&done_setmode_wq);
	}

}

static int AdjustHVSyncForce (struct sisusb_usb_data *sisusb, unsigned char Hvalue, unsigned char Vvalue)
{
	WriteRegisterByte (sisusb, 0xfc6a,  0x0A);
   	WriteRegisterByte (sisusb, 0xfc6b, Hvalue);	
   	WriteRegisterByte (sisusb, 0xfc6a,  0x0B);
   	WriteRegisterByte (sisusb, 0xfc6b, Vvalue);	
   	return 1;
}

static BOOLEAN GetPannelTiming (struct sisusb_usb_data *sisusb)
{
     unsigned char FrameRate=0, DramSize=0;
     unsigned long PixelClock=0, i;
     unsigned short HTotal=0, HSync=0, HBP=0, HSize=0;
     unsigned short VTotal=0, VSync=0, VBP=0, VSize=0;
     unsigned long size, HBlank, VBlank;	
	 unsigned char * pEDID = sisusb->pEDID;
	
	if (pEDID[0x20] == 1)
		sisusb->VGA_Output_Type = DVO_12;
	else
		sisusb->VGA_Output_Type = DVO_24;

	if (pEDID[0x21] == 1)
		sisusb->DRAM_TYPE = TYPE_SDRAM;
	else
		sisusb->DRAM_TYPE = TYPE_DDR1;
		
	if (pEDID[0x22] == 1)
		sisusb->bEnableDAC = TRUE;
	else
		sisusb->bEnableDAC = FALSE;	
		
	FrameRate = pEDID[0x23];
	PixelClock = *((unsigned long *)(pEDID + 0x24));
	HTotal = *((unsigned short *)(pEDID + 0x28));
	HSize = *((unsigned short *)(pEDID + 0x2A));
	HSync = *((unsigned short *)(pEDID + 0x2C));
	HBP = *((unsigned short *)(pEDID +0x2E));
	VTotal = *((unsigned short *)(pEDID + 0x30));
	VSize = *((unsigned short *)(pEDID +0x32));
	VSync = *((unsigned short *)(pEDID +0x34));
	VBP = *((unsigned short *)(pEDID + 0x36));
	DramSize = pEDID[0x38];
	//2009/06/22 added by yulw
	sisusb->bBLGpio = pEDID[0x39];
	sisusb->BLGpioPin = pEDID[0x3A];
	if ((sisusb->BLGpioPin == 0x00) || (sisusb->BLGpioPin == 0xFF))
		sisusb->BLGpioPin = BL_GPIO;	//default value.
	sisusb->Phase = pEDID[0x3B];
	if (sisusb->Phase == 0xFF)
		sisusb->Phase = 0x00;	//using default;
		
	if ((HSize ==0 ) || (HTotal == 0) || (HSync == 0) || (HBP == 0) ||
		(VSize == 0) || (VTotal == 0) || (VSync == 0) || (VBP == 0) ||
		(PixelClock == 0))
	{
		printk ("Invalid timeing parameters\n");
		return FALSE;
	}
	
	memset(sisusb->MonitorName, 0, MONITORNAME_BUFFER_SIZE);
	for (i=0; i<15; i++)
	{
		sisusb->MonitorName[i] = pEDID[0x40+i];
		if (pEDID[0x40+i] == 0x00)
			break; 
	}	
	
	//all value can't less than 2
	if (HBP > 255) HBP = 255;
	if (HBP < 4) HBP = 4;
	if (VBP > 255) VBP = 255;
	if (HSync > 255) HSync = 255;
	if (VSync > 255) VSync = 255;
	if (HBP <=2) HBP = 2;
	if (VBP <=1) VBP = 2;
	if (HSync <=2) HSync = 2;
	if (VSync <=2) VSync = 2;
	if ((HTotal - HSize - HSync - HBP) < 2)
		HBP--;
	if ((VTotal - VSize - VSync - VBP) < 2)
		VBP--;
		

	sisusb->HSync = HSync;
	sisusb->HBP = HBP;
    sisusb->HTotal = HTotal;
    sisusb->HActive = HSize;
    sisusb->VTotal = VTotal;
    sisusb->VActive = VSize;
    sisusb->VSync = VSync;
    sisusb->VBP = VBP;
    sisusb->FrameRate = FrameRate;
    sisusb->PixClock = PixelClock;
    
    if (sisusb->DRAM_TYPE == TYPE_DDR1)
    	sisusb->HW_VRAMSize = 16;
    else if ((sisusb->DRAM_TYPE == TYPE_SDRAM) && (DramSize == 16)) {
    	sisusb->HW_VRAMSize = DramSize;
    	sisusb->A3Bit2 = 0x04;
    }
    else {
    	sisusb->A3Bit2 = 0;
    	sisusb->HW_VRAMSize = 8;
    }
    	
    printk ("PixelClock : %d\n", sisusb->PixClock);
    printk ("H Total : %d\n", sisusb->HTotal);
    printk ("H Active: %d\n", sisusb->HActive);
    printk ("H Sync: %d\n", sisusb->HSync);
    printk ("H Back Porch : %d\n", sisusb->HBP);
    printk ("V Total : %d\n", sisusb->VTotal);
    printk ("V Active: %d\n", sisusb->VActive);
    printk ("V Sync: %d\n", sisusb->VSync);
    printk ("V Back Porch : %d\n", sisusb->VBP);    
    printk ("DRAM Size : %dMB\n", sisusb->HW_VRAMSize);
    printk ("MonitorName: %s\n", sisusb->MonitorName);
    printk ("Enable Bl GPIO: %d\n", sisusb->bBLGpio);
    printk ("GPIO Pin: %x\n", sisusb->BLGpioPin);
    printk ("Phase: %x\n", sisusb->Phase);
	sisusb->hres = sisusb->HW_hres = sisusb->HActive;
	sisusb->vres = sisusb->HW_vres = sisusb->VActive;
	if (sisusb->hres == 1400)	// bug bug
		sisusb->HW_hres = 1408;	
	return TRUE;
}

static BOOLEAN MCT_ReadEEPROM (struct sisusb_usb_data *sisusb, ULONG addr, ULONG *data)
{
	UCHAR cmdBuf[32];
	UCHAR buf[512];
	UCHAR *p;
	UCHAR FC6F, c;
	BOOLEAN ret;
	UCHAR startaddr;
	
	if (addr > 255)
		return FALSE;
	if (addr == 255) {
		startaddr = 1;
		addr = 254;
	}
	else
		startaddr = 0;
		
	
	ReadRegisterByte (sisusb, 0xfc6f, &FC6F);
	c = FC6F | 0x04;	//GPIO using by 8032
	WriteRegisterByte (sisusb, 0xfc6f, c); 
	//bug bug: read 255 words to avoid USB 1.1 IF issue
	cmdBuf[0] = 0x25; // read EEPROM command
	cmdBuf[1] = startaddr; // word alignment
	cmdBuf[2] = 255; // 1 words
	cmdBuf[3] = 0;
	
	if (SendBlockCommand(sisusb,3, cmdBuf, 0, 4) >= 0){	
		int byte_read;
		if(sisusb_recv_bulk_msg(sisusb, 1, 512, (void *) buf, NULL, &byte_read, 0)>=0) {
			*data = 0;
			p = (UCHAR *)data;
			p[1] = buf[addr*2+0]; //High byte
			p[0] = buf[addr*2+1]; //Low byte
			printk("MCT_ReadEEPROM:%x %x\n", p[0], p[1]);
			ret = TRUE;
		}
		else {
			printk("MCT_ReadEEPROM Fail\n");	
			ret = FALSE;
		}
	}
	else
		ret = FALSE;
	
	WriteRegisterByte (sisusb, 0xfc6f, FC6F); //GPIO function
	return ret;
}

static void PowerDownDDR (struct sisusb_usb_data *sisusb)
{
	UCHAR CmdBuf[64];
	ULONG len;	
	
	printk("PowerDownDDR is called\n");
	len = 43;
	CmdBuf[0] = 0x04;
	CmdBuf[1] = 40;
	CmdBuf[2] = 0x00;
	
/// SDRAM CLK
	CmdBuf[3] = 0x6a;  /// address
	CmdBuf[4] = 0x00; /// data
	CmdBuf[5] = 0x6b;  /// address
	CmdBuf[6] = 0x0a; /// data

/// SDRAM CSB
	CmdBuf[7] = 0x6a;  /// address
	CmdBuf[8] = 0x01; /// data
	CmdBuf[9] = 0x6b;  /// address
	CmdBuf[10] = 0x0a; /// data

/// SDRAM CKEB
	CmdBuf[11] = 0x6a;  /// address
	CmdBuf[12] = 0x02; /// data
	CmdBuf[13] = 0x6b;  /// address
	CmdBuf[14] = 0x0a; /// data

/// SDRAM ADDR
	CmdBuf[15] = 0x6a;  /// address
	CmdBuf[16] = 0x03; /// data
	CmdBuf[17] = 0x6b;  /// address
	CmdBuf[18] = 0x0a; /// data

/// SDRAM BANK
	CmdBuf[19] = 0x6a;  /// address
	CmdBuf[20] = 0x04; /// data
	CmdBuf[21] = 0x6b;  /// address
	CmdBuf[22] = 0x0a; /// data

/// SDRAM RASB
	CmdBuf[23] = 0x6a;  /// address
	CmdBuf[24] = 0x05; /// data
	CmdBuf[25] = 0x6b;  /// address
	CmdBuf[26] = 0x0a; /// data

/// SDRAM CASB
	CmdBuf[27] = 0x6a;  /// address
	CmdBuf[28] = 0x06; /// data
	CmdBuf[29] = 0x6b;  /// address
	CmdBuf[30] = 0x0a; /// data

/// SDRAM WEB
	CmdBuf[31] = 0x6a;  /// address
	CmdBuf[32] = 0x07; /// data
	CmdBuf[33] = 0x6b;  /// address
	CmdBuf[34] = 0x0a; /// data

/// SDRAM DATAM
	CmdBuf[35] = 0x6a;  /// address
	CmdBuf[36] = 0x08; /// data
	CmdBuf[37] = 0x6b;  /// address
	CmdBuf[38] = 0x0a; /// data

/// SDRAM DATA
	CmdBuf[39] = 0x6a;  /// address
	CmdBuf[40] = 0x09; /// data
	CmdBuf[41] = 0x6b;  /// address
	CmdBuf[42] = 0x0a; /// data		
	SendBlockCommand(sisusb, 3, CmdBuf, 0, len);
}

static void PowerUpDDR (struct sisusb_usb_data *sisusb)
{
	UCHAR CmdBuf[64];
	ULONG len;	
	
	printk ("PowerUpDDR is called\n");
	len = 43;
	CmdBuf[0] = 0x04;
	CmdBuf[1] = 40;
	CmdBuf[2] = 0x00;
	
/// SDRAM CLK
	CmdBuf[3] = 0x6a;  /// address
	CmdBuf[4] = 0x00; /// data
	CmdBuf[5] = 0x6b;  /// address
	CmdBuf[6] = 0x22; /// data

/// SDRAM CSB
	CmdBuf[7] = 0x6a;  /// address
	CmdBuf[8] = 0x01; /// data
	CmdBuf[9] = 0x6b;  /// address
	CmdBuf[10] = 0x22; /// data

/// SDRAM CKEB
	CmdBuf[11] = 0x6a;  /// address
	CmdBuf[12] = 0x02; /// data
	CmdBuf[13] = 0x6b;  /// address
	CmdBuf[14] = 0x22; /// data

/// SDRAM ADDR
	CmdBuf[15] = 0x6a;  /// address
	CmdBuf[16] = 0x03; /// data
	CmdBuf[17] = 0x6b;  /// address
	CmdBuf[18] = 0x22; /// data

/// SDRAM BANK
	CmdBuf[19] = 0x6a;  /// address
	CmdBuf[20] = 0x04; /// data
	CmdBuf[21] = 0x6b;  /// address
	CmdBuf[22] = 0x22; /// data

/// SDRAM RASB
	CmdBuf[23] = 0x6a;  /// address
	CmdBuf[24] = 0x05; /// data
	CmdBuf[25] = 0x6b;  /// address
	CmdBuf[26] = 0x22; /// data

/// SDRAM CASB
	CmdBuf[27] = 0x6a;  /// address
	CmdBuf[28] = 0x06; /// data
	CmdBuf[29] = 0x6b;  /// address
	CmdBuf[30] = 0x22; /// data

/// SDRAM WEB
	CmdBuf[31] = 0x6a;  /// address
	CmdBuf[32] = 0x07; /// data
	CmdBuf[33] = 0x6b;  /// address
	CmdBuf[34] = 0x22; /// data

/// SDRAM DATAM
	CmdBuf[35] = 0x6a;  /// address
	CmdBuf[36] = 0x08; /// data
	CmdBuf[37] = 0x6b;  /// address
	CmdBuf[38] = 0x22; /// data

/// SDRAM DATA
	CmdBuf[39] = 0x6a;  /// address
	CmdBuf[40] = 0x09; /// data
	CmdBuf[41] = 0x6b;  /// address
	CmdBuf[42] = 0x22; /// data
	SendBlockCommand(sisusb, 3, CmdBuf, 0, len);
}
//************************************************************
static void PowerDownDevice (struct sisusb_usb_data *sisusb)
{
	UCHAR c;
	
	WriteRegisterByte (sisusb, 0xFCB0, 0x20);
	PowerDownDDR (sisusb);
	WriteRegisterByte (sisusb, 0xFC4B, 0x18);
	WriteRegisterByte (sisusb, 0xFC4D, 0x80);
	WriteRegisterByte (sisusb, 0xFC4F, 0x80);		
	WriteRegisterByte (sisusb, 0xFC6F, 0x04);	//return GPIO to 8032
}

void WriteGPIO(struct sisusb_usb_data *sisusb, UCHAR Mask, UCHAR andMask, UCHAR orMask)
{
	UCHAR m6c, m6d;
	
	ReadRegisterByte (sisusb, 0xfc6c, &m6c);
	ReadRegisterByte (sisusb, 0xfc6d, &m6d);
	m6c = m6c|Mask;
	WriteRegisterByte (sisusb, 0xFC6C, m6c);
	m6d = m6d&andMask|orMask;	
	printk ("IOMask: %x  value: %x\n", m6c, m6d);
	WriteRegisterByte (sisusb, 0xFC6D, m6d);	
}

//******************************************************
void LightOn(struct sisusb_usb_data *sisusb, BOOLEAN On)
{
	UCHAR Light = 0x22;
	
	if (On)
		WriteGPIO (sisusb, Light, 0xff, Light);
	else
		WriteGPIO (sisusb, Light, ~Light, 00);
}

//***************************************************************
static BOOLEAN SuspendNotify (struct sisusb_usb_data *sisusb)
{
	WriteGPIO (sisusb, sisusb->BLGpioPin, ~(sisusb->BLGpioPin), 0);
	mdelay (10);	//10ms
	printk("1\n");
	WriteGPIO (sisusb, sisusb->BLGpioPin, 0xFF, sisusb->BLGpioPin);
	mdelay (10);	//10ms
	printk("2\n");
	WriteGPIO (sisusb, sisusb->BLGpioPin, ~(sisusb->BLGpioPin), 0);
	mdelay (10);	//10ms
	printk("3\n");
	WriteGPIO (sisusb, sisusb->BLGpioPin, 0xFF, sisusb->BLGpioPin);
	mdelay (10);	//10ms
	printk("4\n");
	WriteGPIO (sisusb, sisusb->BLGpioPin, ~(sisusb->BLGpioPin), 0);
	mdelay (10);	//10ms
	printk("5\n");
	WriteGPIO (sisusb, sisusb->BLGpioPin, 0xFF, sisusb->BLGpioPin);
	mdelay (10);	//10ms
	printk("6\n");
	WriteGPIO (sisusb, sisusb->BLGpioPin, ~(sisusb->BLGpioPin), 0);

	return TRUE;
}	
//*************************************************************
static BOOLEAN WakeupNotify (struct sisusb_usb_data *sisusb)
{
	WriteGPIO (sisusb, sisusb->BLGpioPin, ~(sisusb->BLGpioPin), 0);
	WriteGPIO (sisusb, sisusb->BLGpioPin, 0xFF, sisusb->BLGpioPin);
	mdelay (5);	//5ms
	WriteGPIO (sisusb, sisusb->BLGpioPin, ~(sisusb->BLGpioPin), 0);
	mdelay (15);	//5ms
	WriteGPIO (sisusb, sisusb->BLGpioPin, 0xFF, sisusb->BLGpioPin);
	mdelay (5);	//5ms
	WriteGPIO (sisusb, sisusb->BLGpioPin, ~(sisusb->BLGpioPin), 0);
	mdelay (15);	//15ms
	WriteGPIO (sisusb, sisusb->BLGpioPin, 0xFF, sisusb->BLGpioPin);
	mdelay (5);	//5ms
	WriteGPIO (sisusb, sisusb->BLGpioPin, ~(sisusb->BLGpioPin), 0);
	mdelay (15);	//15ms
	return TRUE;
}	

static int triggerusb_probe(struct usb_interface *intf,
			const struct usb_device_id *id)
{
	struct usb_device *dev = interface_to_usbdev(intf);
	struct sisusb_usb_data *sisusb;
	unsigned char TVER;
	int retval = 0, i;
	
	struct fb_info *info; // WHS: fb supporting	

	int vram_size;

	printk(KERN_INFO "triggerusb: USB2VGA plus dongle found at address %d\n",
		dev->devnum);

	/* Allocate memory for our private */
	if (!(sisusb = kmalloc(sizeof(*sisusb), GFP_KERNEL))) {
		printk(KERN_ERR
			"sisusb: Failed to allocate memory for private data\n");
		return -ENOMEM;
	}

	

	memset(sisusb, 0, sizeof(*sisusb));
	kref_init(&sisusb->kref);

	init_MUTEX(&(sisusb->lock));

	/* Register device */
	if ((retval = usb_register_dev(intf, &usb_triggerusb_class))) {
		printk(KERN_ERR
			"triggerusb: Failed to get a minor for device %d\n",
			dev->devnum);
		retval = -ENODEV;
		goto error_1;
	}

	sisusb->sisusb_dev = dev;
	sisusb->minor      = intf->minor;
	printk(KERN_INFO "WHS: minor = %d!\n", sisusb->minor);		
	
	/* Everything else is zero */

//޷|X{allocD,B|Ψdmaݭns,
//]hShusb hostdmas
	/* Allocate buffers */
	sisusb->ibufsize = SISUSB_IBUF_SIZE;
	if (!(sisusb->ibuf = usb_buffer_alloc(dev, SISUSB_IBUF_SIZE,
					      GFP_KERNEL, &sisusb->transfer_dma_in))) {
		printk("sisusbvga[%d]: Failed to allocate memory for %s buffer\n", sisusb->minor, "input");
		retval = -ENOMEM;
		goto error_2;
	}
	else{
		printk("sisusbvga[%d]: allocate memory for input %s buffer\n", sisusb->minor, sisusb->transfer_dma_in?"dma":"nodma");
	}

	sisusb->numobufs = 0;
	sisusb->obufsize = SISUSB_OBUF_SIZE;
	for (i = 0; i < NUMOBUFS; i++) {
		if (!(sisusb->obuf[i] = usb_buffer_alloc(dev, SISUSB_OBUF_SIZE,
					GFP_KERNEL,
					&sisusb->transfer_dma_out[i]))) {
			if (i == 0) {
				printk("sisusbvga[%d]: Failed to allocate memory for %s buffer\n", sisusb->minor, "output" );
				retval = -ENOMEM;
				goto error_3;
			}
			else{
				printk("sisusbvga[%d]: allocate memory for output %s buffer\n", sisusb->minor, sisusb->transfer_dma_out[i]?"dma":"nodma");
			}
			break;
		} else
			sisusb->numobufs++;

	}

	/* Allocate URBs */
	if (!(sisusb->sisurbin = usb_alloc_urb(0, GFP_KERNEL))) {
		printk(KERN_ERR
			"triggerusb[%d]: Failed to allocate URBs\n",
			sisusb->minor);
		retval = -ENOMEM;
		goto error_3;
	}
	sisusb->completein = 1;

	for (i = 0; i < sisusb->numobufs; i++) {
		if (!(sisusb->sisurbout[i] = usb_alloc_urb(0, GFP_KERNEL))) {
			printk(KERN_ERR
				"triggerusb[%d]: Failed to allocate URBs\n",
				sisusb->minor);
			retval = -ENOMEM;
			goto error_4;
		}
		sisusb->urbout_context[i].sisusb = (void *)sisusb;
		sisusb->urbout_context[i].urbindex = i;
		sisusb->urbstatus[i] = 0;
	}

	printk(KERN_INFO "triggerusb[%d]: Allocated %d output buffers\n",
					sisusb->minor, sisusb->numobufs);

       

	/* Do remaining init stuff */

	init_waitqueue_head(&sisusb->wait_q);

	usb_set_intfdata(intf, sisusb);

	usb_get_dev(sisusb->sisusb_dev);

	sisusb->present = 1;

        #ifdef SISUSB_OLD_CONFIG_COMPAT
	{
	int ret;
	/* Our ioctls are all "32/64bit compatible" */
	ret =  register_ioctl32_conversion(SISUSB_GET_CONFIG_SIZE, NULL);
	ret |= register_ioctl32_conversion(SISUSB_GET_CONFIG,      NULL);
	ret |= register_ioctl32_conversion(SISUSB_COMMAND,         NULL);
	if (ret)
		printk(KERN_ERR
			"triggerusb[%d]: Error registering ioctl32 "
			"translations\n",
			sisusb->minor);
	else
		sisusb->ioctl32registered = 1;
	}
        #endif

	if (dev->speed == USB_SPEED_HIGH) {
		int initscreen = 1;
		
		
              
                
		if (sisusb_init_gfxdevice(sisusb, initscreen))
			printk(KERN_ERR
				"triggerusb[%d]: Failed to early "
				"initialize device\n",
				sisusb->minor);

	} else
		printk(KERN_INFO
			"triggerusb[%d]: Not attached to USB 2.0 hub, "
			"deferring init\n",
			sisusb->minor);

	sisusb->ready = 1;

        #ifdef SISUSBENDIANTEST
	printk(KERN_DEBUG "triggerusb: *** RWTEST ***\n");
	sisusb_testreadwrite(sisusb);
	printk(KERN_DEBUG "sisusb: *** RWTEST END ***\n");
        #endif
           
    	//#ifdef TRIGGER_DEMO
		//kernel_thread(tusb_demo, (void *) sisusb, 0);
        //VgaModeSetting_DDR(sisusb, 0x16);
        //TransferBlockData_TEST(sisusb);                
        //#endif
	 //Louis add,
	//yulwjv: 0718 Trigger2 select scaling clock
	//WriteRegisterByte (sisusb, 0xfbff, 0x81);
	//the above line will be done in the following, 20090812

	ReadRegisterByte (sisusb, 0xfc01, &TVER);
	printk ("Trigger Version (0xFC01) = %x\n", TVER);

	//LED


#ifdef I2C_ATMEL
	if (!CheckATMel (sisusb)) {
		//there are no atmel AT88SC0808C on i2c bus, turn on led automatically
#else
	if(1){
#endif
		printk ("Turn On LED\n");
		WriteRegisterByte (sisusb, 0xfc6f, 0x00);
		LightOn(sisusb, 1);
		//WriteRegisterByte (sisusb, 0xFC6C, 0xff);
		//WriteRegisterByte (sisusb, 0xFC6D, 0x22); 
	}else{
		//let customer use ioctl to turn on led
	}
	
	//Detect Trigger-plus Package Type 
	
	//enhance the driving strength of HSync and VSync
	AdjustHVSyncForce (sisusb, 0x27, 0x27);
	ReadEDID(sisusb, sisusb->pEDID, 512);
	if ((*((unsigned long *)sisusb->pEDID) == 0x30564741) &&
        (*((unsigned long *)(sisusb->pEDID + 4)) == 0x55534232)) {
		printk ("Pannel Config\n");
		if (!GetPannelTiming (sisusb)){
			printk("unable to get pannel config\n");
		}
		else {
			if (sisusb->DRAM_TYPE == TYPE_DDR1) {
				sisusb->ucScalingClock = 0x81;	//60M
			}else {
				sisusb->ucScalingClock = 0x82; //48M..
			}
			if (sisusb->VGA_Output_Type != DVO_24)
				sisusb->bEnableDAC = TRUE;
			
			printk ("EnableDAC: %d\n", sisusb->bEnableDAC);
			
			WriteRegisterByte (sisusb, 0xfbff, sisusb->ucScalingClock);

			// return the HSync, VSync
			WriteRegisterByte (sisusb, 0xfc6f, 0x00);
			if (sisusb->DRAM_TYPE != TYPE_SDRAM)
				PowerUpDDR (sisusb);			
			sisusb->bGetTimingFromPanel = TRUE;	
		}
	}else{
		UCHAR FC01, FC6F;
		sisusb->bGetTimingFromPanel = FALSE;			
		//make GPIO pin enable
		ReadRegisterByte (sisusb, 0xfc6f, &FC6F);
		printk ("0xFC6F before = %x\n", FC6F);
		WriteRegisterByte (sisusb, 0xfc6f, 0x08);	//HSync, VSync ->input mode, GPIO -> general purpose
		ReadRegisterByte  (sisusb, 0xfc6f, &FC6F);
		printk ("0xFC6F = %x\n", FC6F);

		if (FC6F&0x10)	//HSYNC
			sisusb->VGA_Output_Type = DVO_24;
		else {
			ULONG x;
			if (MCT_ReadEEPROM (sisusb, 0x00, &x)) {
				if (x == 0x55aa) {
					if (MCT_ReadEEPROM (sisusb, 0x50, &x)) {
						if (x == 0x01)
							sisusb->VGA_Output_Type = D_SUB;
						else
							sisusb->VGA_Output_Type = DVO_12;
					}
					else
						sisusb->VGA_Output_Type = D_SUB;
				}
				else
					sisusb->VGA_Output_Type = D_SUB;
			}
			else
				sisusb->VGA_Output_Type = D_SUB;
		}
				
		if (FC6F&0x20)	//VSYNC
			sisusb->DRAM_TYPE = TYPE_DDR1;
		else
			sisusb->DRAM_TYPE = TYPE_SDRAM;
		
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//Louis Tsai , 20090812
		//force DRAM TYPE = ddr1, most pcb use ddr now
		//if we really want to use resistor on HSYNC & VSYNC to detect type,
		//please remove the following line.
		sisusb->DRAM_TYPE = TYPE_DDR1;
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

		if (sisusb->DRAM_TYPE == TYPE_DDR1) {
			sisusb->ucScalingClock = 0x81;	//60M
		}
		else {
			sisusb->ucScalingClock = 0x82; //48M
		}
		
		if (sisusb->VGA_Output_Type != DVO_24)
			sisusb->bEnableDAC = TRUE;
		
		printk ("EnableDAC: %d\n", sisusb->bEnableDAC);
		
		WriteRegisterByte (sisusb, 0xfbff, sisusb->ucScalingClock);
		
		sisusb->HW_VRAMSize = 16;
		// return the HSync, VSync
		WriteRegisterByte (sisusb, 0xfc6f, 0x00);
		if (sisusb->DRAM_TYPE != TYPE_SDRAM)
			PowerUpDDR (sisusb);			
	}


	#ifdef FB_SUPPORT

	sisusb->isModeSet = 0;
	
	// [WHS: framebuffer supporting

    SCOPE_BEG
	
	struct tusbvga_par *par;
	int cmap_len;

	info = framebuffer_alloc(sizeof(struct tusbvga_par), &dev->dev); // WHS: Fbsysfs.c
	if(!info){
		printk(KERN_ERR "WHS: alloc fb_info failed\n");
		retval = -ENODEV;
		goto error_5;
	}

	par = info->par;
	par->sisusb = sisusb;
	par->open_count = 0;

	printk(KERN_ERR "WHS: alloc fb_info succeeded\n");
	
	sisusb->fbInfo = info; // WHS: install info in usb driver
 
	info->screen_base = ~0;
	//info->screen_base = NULL;
    info->fbops = &tusbvga_fb_ops;
    // info->fix = tusbvgafb_fix; // WHS: this will be the only time xxxfb_fix will be used, so mark it as __devinitdata 
    //info->pseudo_palette = pseudo_palette; // WHS: The pseudopalette is an * 16-member array
    	
	// [WHS: check_var & set_par

	// check_var

    // set_par

	// WHS]
	

	if(register_framebuffer(info) < 0){ // WHS: mandatory: register tusbvga's fb dev
		printk(KERN_ERR "WHS: error registering tusbvga framebuffer\n");
		retval = -EINVAL;
		goto error_8;
	}	


	
	#if 0
	    
    printk(KERN_INFO "WHS: fb%d: %s frame buffer device\n", info->node, info->fix.id);	
	//info->par->drvData = sisusb;
	//usb_set_intfdata(intf, sisusb); // WHS: install struct sisusb_usb_data *sisusb in generic device

	#endif
	
	SCOPE_END

	// WHS: framebuffer supporting]	

	#endif

	// WHS: overlay

	//SCOPE_BEG

	vram_size = Trigger_Get_VRAM_Size((struct sisusb_usb_data *)sisusb);
	
	sisusb->flipable = (vram_size == 16) ? 1 : 0;
	sisusb->flipBuf = 0;

	//__find_symbol(NULL, NULL, NULL, 0);
	//lookup_symbol

	#ifdef tusbvga_SMP
	sisusb->overlayLock = SPIN_LOCK_UNLOCKED;
	#endif
	
	//SCOPE_END

	#ifdef tusbvga_HOTPLUG

	SCOPE_BEG

	int msgid;

	//sys_msgget

	//aaa

	msgid = real_msgget((key_t)1234, 0666|IPC_CREAT);
	//msgid = sys_vfork();
	//kernel_thread(tusb_demo, (void *) sisusb, 0);
	//cpu_idle();

	//sys_kill(0, SIGALRM);
	//msgid = open("test", "r");
	//signal(SIGINT, NULL);

	SCOPE_END

	#endif

	#ifdef TRIGGER_DEMO
		tusb_demo(sisusb);
	#endif	
	
	return 0;
    
	error_8:
 
	error_7:
			
	error_6:
	framebuffer_release(info);

	error_5:

        error_4:
	sisusb_free_urbs(sisusb);
        error_3:
	sisusb_free_buffers(sisusb);
        error_2:
	usb_deregister_dev(intf, &usb_triggerusb_class);
        error_1:
	kfree(sisusb);
	return retval;
}

static void triggerusb_disconnect(struct usb_interface *intf)
{
	struct sisusb_usb_data *sisusb;
	int minor;

	//WriteRegisterByte(sisusb, 0xFCB0, 0x00); 

	/* This should *not* happen */
	if (!(sisusb = usb_get_intfdata(intf)))
		return;

   

	/* The above code doesn't need the disconnect
	 * semaphore to be down; its meaning is to
	 * protect all other routines from the disconnect
	 * case, not the other way round.
	 */
	down(&disconnect_sem_t);

	down(&sisusb->lock);

	#ifdef FB_SUPPORT

	// [WHS: clean up for fb supporting

	SCOPE_BEG

	struct fb_info *info = sisusb->fbInfo;	

	if(info){
		unregister_framebuffer(info);
		//fb_dealloc_cmap(&info->cmap);		
		framebuffer_release(info);
	}

	SCOPE_END

	// WHS: clean up for fb supporting]

	#endif

	/* Wait for all URBs to complete and kill them in case (MUST do) */
	if (!sisusb_wait_all_out_complete(sisusb))
	    sisusb_kill_all_busy(sisusb);

	minor = sisusb->minor;
	usb_set_intfdata(intf, NULL);
	usb_deregister_dev(intf, &usb_triggerusb_class);

        #ifdef SISUSB_OLD_CONFIG_COMPAT
	if (sisusb->ioctl32registered) {
		int ret;
		sisusb->ioctl32registered = 0;
		ret =  unregister_ioctl32_conversion(SISUSB_GET_CONFIG_SIZE);
		ret |= unregister_ioctl32_conversion(SISUSB_GET_CONFIG);
		ret |= unregister_ioctl32_conversion(SISUSB_COMMAND);
		if (ret) {
			printk(KERN_ERR
				"sisusbvga[%d]: Error unregistering "
				"ioctl32 translations\n",
				minor);
		}
	}
        #endif

	sisusb->present = 0;
	sisusb->ready = 0;

	up(&sisusb->lock);

	/* decrement our usage count */
	kref_put(&sisusb->kref, triggerusb_delete);

	up(&disconnect_sem_t);

	printk(KERN_INFO "triggerusbvga[%d]: Disconnected\n", minor);
}

#if 0

static struct usb_device_id triggerusb_table [] = {	
	{ USB_DEVICE(0x0711, 0x5000) },
	{ USB_DEVICE(0x0711, 0x5001) },
	{ USB_DEVICE(0x0711, 0x5002) },
	{ USB_DEVICE(0x0711, 0x5003) },
	{ USB_DEVICE(0x182d, 0x021c) },
	{ USB_DEVICE(0x182d, 0x0269) },
	{ }
};

#endif

static struct usb_device_id triggerusb_table [] = 
{	
	{ USB_DEVICE(0x0711, 0x5000) },	
	{ USB_DEVICE(0x0711, 0x5001) },	
	{ USB_DEVICE(0x0711, 0x5002) },	
	{ USB_DEVICE(0x0711, 0x5003) },	
	{ USB_DEVICE(0x0711, 0x5100) },	
	{ USB_DEVICE(0x0711, 0x5101) },	
	{ USB_DEVICE(0x182d, 0x021c) },	
	{ USB_DEVICE(0x182d, 0x0269) },	
	{ }
};

MODULE_DEVICE_TABLE (usb, triggerusb_table);

static struct usb_driver triggerusb_driver = {
	.name =		"triggerusb",
	.probe =	triggerusb_probe,
	.disconnect =	triggerusb_disconnect,
	.id_table =	triggerusb_table, 
};

static int __init usb_triggerusb_init(void)
{
	int retval;
	
	info_module_exit = 0;
	init_waitqueue_head(&setmode_wq);
	init_waitqueue_head(&done_setmode_wq);
	kernel_thread(setmode_thread, NULL, 0);

	if (!(retval = usb_register(&triggerusb_driver))) {

		printk(KERN_INFO "trigger-plus: driver version %d.%d.%d\n",
			SISUSB_VERSION, SISUSB_REVISION, SISUSB_PATCHLEVEL);
		printk(KERN_INFO
			"triggerusb: Original-Copyright (C) 2005 Thomas Winischhofer\n");

	}
	

	return retval;
}

static void __exit usb_triggerusb_exit(void)
{
	//WriteRegisterByte(sisusb, 0xFCB0, 0x00);
	usb_deregister(&triggerusb_driver);
	info_module_exit = 1;
	//wake_up(&setmode_wq);
	gSetMode = 1;
	//wake_up(&setmode_wq);
	wake_up_interruptible(&setmode_wq);
	
}

module_init(usb_triggerusb_init);
module_exit(usb_triggerusb_exit);

MODULE_AUTHOR(" Magic Control Technology <louis@mct.com.tw>");
MODULE_DESCRIPTION("triggerusbvga - Driver dongles");
MODULE_LICENSE("GPL");

